--[[

Copyright (c) 2012-2020 Baby-Bus.com

http://www.baby-bus.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]

----------------------
-- 类
----------------------
local BaseGuideLayer = require("app.common.layer.BaseGuideLayer")
local M = class("Guide", BaseGuideLayer)

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

-- [操作变量]
-- ..

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- [操作变量]
-- ..
--篮球
local Ball    = require("app.ball.node.Ball")
--篮筐
local Basket    = require("app.ball.node.Basket")






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    self._balls     = {}
    self._guideBall = {}
    self._basket    = {}
    self._ballPos   = {
        cc.p(400 + X_OFFSET, -42),
        cc.p(560 + X_OFFSET, -42),
    }
    --4个位置是否有空位
    self._hasVacancy        = {
        true, true, true, true
    }
    --得分
    self._score            = 0
    --预备的球个数    
    self._ballCount        = 4

    -- 皮肤类型表
    self._skirTable        = { 1, 2 }
    -- 性别表
    self._sexTable        = { 1, 2 }
    --背后的4个小孩
    self._children        = {}
    --需要补充的球的id的集合
    self._needAddBallId    = {

    }
    self._gravityRate    = 4
    self._setY            = Y_OFFSET / 2
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- 运行父类方法
    M.super.onRender(self)
    -- 设置本地数据
    LData:getLocalData()
    --背景
    self:loadBg()
    self:loadCloud()
    --物理世界
    self:loadWorld()
    --加载初始篮筐
    self:loadBasket()
    --加载篮球
    self:loadBall()
    --加载地面刚体
    self:loadLandBody()
end

function M:onEnterTransitionFinish()
    -- [超类调用]
    M.super.onEnterTransitionFinish(self)
    sound:playMusicBackground007()
    self:openSC()
    -- self:addBall()
end

-- 开启实时刷新
function M:openSC()
    if self._openSC then return end
    -- 开启物理世界更新
    local openSC = scheduler.scheduleUpdateGlobal(handlerP(self, self.tick))
    self._openSC = openSC
end

--加载背景
function M:loadBg()
    local emptyNode = U.loadNode({ contentSize = cc.size(0.1, 0.1) }):to(self):p(0, -45 + self._setY):unbindTouch()
    self._emptyNode = emptyNode
    local bg = U.loadBg("ball/bg/bg.png"):to(self._emptyNode)
    --草地
    local land = D.img("ball/bg/land.png"):to(self._emptyNode)
    :anchor(cc.p(0.5, 0)):p(V.w_2, -1):scale(1.2):bindTouch():opacity(0)
    --篮球线
    local line = D.img("ball/bg/line.png"):to(self._emptyNode)
    :anchor(cc.p(0.5, 0)):p(V.w_2, 10):scale(1.25)
    --旗子
    self:loadFlags(land)
end

-- 加载云
function M:loadCloud()
    local sprite = import("app.camp.node.Cloud")
    local point = { cc.p(284.8 + X_OFFSET, 771.9), cc.p(333.7 + X_OFFSET, 691.6), cc.p(793.9 + X_OFFSET, 777.6), cc.p(647 + X_OFFSET, 710.3) }
    for i = 1, 4 do
        local cloud = sprite.new({
            imgName = "camp/cloud/cloud_" .. (math.fmod(i, 2) + 1) .. ".png",
            main = self,
            id = i,
            pos = point[i],
            scale = .5
        }):to(self._emptyNode, -5):p(point[i]):scale(.5)
    end
end

--加载旗子
function M:loadFlags(land)
    --大旗子
    local pos = {
        cc.p(700, land:ch() - 10), cc.p(780, land:ch() + 15), cc.p(850, land:ch() - 20)
    }
    local scl = { 0.9, 1, 0.9 }
    for i, v in pairs(pos) do
        local flag = D.img("common/flag/4/10001.png"):to(land, -4):bindTouchLocate()
        :anchor(cc.p(0.5, 0)):p(v):scale(scl[i])
        flag:cycle({
            { "image", "common/flag/4/1000", 8, RD.num(80, 120) / 1000 }
        })
    end
    --小旗子(红1黄2蓝3)
    local id = {
        2, 3, 3, 1, 2, 3, 2, 1, 3, 1,
    }
    --大旗子
    local pos = {
        cc.p(30, land:ch() - 20), cc.p(70, land:ch() - 30), cc.p(335, land:ch() - 20),
        cc.p(420, land:ch() - 35), cc.p(458, land:ch() - 25), cc.p(635, land:ch() - 20),
        cc.p(670, land:ch() - 15), cc.p(710, land:ch() - 30), cc.p(950, land:ch() - 40),
        cc.p(990, land:ch() - 30)
    }
    for i, v in pairs(pos) do
        local flag = D.img("common/flag/" .. id[i] .. "/flag_1.png"):to(land, 1)
        :anchor(cc.p(0.5, 0)):p(v):scale(0.8)
        flag:cycle({
            { "image", "common/flag/" .. id[i] .. "/flag_", 8, 0.1 },
        })
    end
end

--加载篮筐
function M:loadBasket(pos)
    pos = ifnil(pos, cc.p(V.w_2, 440))
    local basket = Basket.new({
        main    = self,
        pos    = pos,
        world = self._world
    }):to(self._emptyNode, 2):p(pos)
    table.insert(self._basket, basket)
end

--加载物理世界
function M:loadWorld()
    -- 创建世界
    local world    = b2World(b2Vec2(0, -25 * self._gravityRate))
    -- 允许静止的物体休眠
    world:SetAllowSleeping(true)
    -- 开启连续物理检测，使模拟更加的真实
    -- world:SetContinuousPhysics(true)
    self._world = world
    bb.si.Box2D.setWorld(self._world)
    local left = -200
    local right = V.w + 200
    -- local debugDraw = GB2DebugDrawLayer:create(self._world, PTM_RATIO)
    -- self:add(debugDraw, 9999)
end

--加载初始篮球
function M:loadBall()
    local pos = self._ballPos
    for i = 1, 2 do
        local ball = Ball.new({
            main    = self,
            id    = i,
            world = self._world,
        }):to(self._emptyNode, 20):bindTouch()
        :p(pos[i])
        table.insert(self._balls, ball)
        table.insert(self._guideBall, ball)
    end
end

--加载一个篮球
function M:addOneBall(id)
    local pos = self._ballPos
    local ball = Ball.new({
        main    = self,
        id    = id,
        world = self._world,
    }):to(self._emptyNode, 20):bindTouch()
    :p(pos[id])
    table.insert(self._balls, ball)
    table.insert(self._guideBall, ball)
end

--加载地板
function M:loadLandBody()
    local categoryBits = { 4, 16 }
    local maskBits    = { 6, 6 }
    local point        = {
        cc.p(V.w_2, 100), cc.p(V.w_2, 380),
    }
    for i = 1, 2 do
        -- 加载刚体
        self:loadBody({
            bodyType    = b2_staticBody,
            point    = point[i],
            userData    = nil,
            -- 密度
            density    = 0,
            -- 摩擦力
            friction    = 1,
            shape    = {
                {
                    shapeType = EShape.POLYGON,
                    points    = {
                        ccp(-V.w_2, -10), ccp(-V.w_2, 0),
                        ccp(-100, 10), ccp(100, 10),
                        ccp(V.w_2, 0), ccp(V.w_2, -10)
                    }
                },
            },
            categoryBits = categoryBits[i],
            maskBits = maskBits[i],
        })
    end
end

--补充球
function M:addBall()
    if self._addBallAction then return end
    local action = A.cycle({
        { "delay", 1 / 30 },
        { "fn", function()
            if #self._needAddBallId > 0 and self._canCheck then
                self._canCheck = false
                self:line({
                    { "delay", self._delayTime },
                    { "fn", function()
                        self._canCheck = true
                        self:addOneBall(self._needAddBallId[1])
                        table.remove(self._needAddBallId, 1)
                    end },
                })
            end
        end },
    }):at(self)
    self._addBallAction = action
end

-- 加载刚体
function M:loadBody(params)
    -- 定义刚体信息
    local bodyInfo            = {}
    -- 类型：静态(b2_staticBody),动态(b2_dynamicBody),平台(b2_kinematicBody)
    bodyInfo.type        = params.bodyType
    -- 位置
    bodyInfo.pos            = params.point
    -- 用户数据：存储用户的数据，可以是任何类型的数据。一般要求存储的数据的类型是一致的
    bodyInfo.userData        = params.userData
    -- 角度阻尼：阻碍刚体旋转的系数，让刚体旋转更容易停下来。
    bodyInfo.angularDamping    = params.angularDamping or 0
    -- 密度：影响刚体的惯性、重量等
    bodyInfo.density            = params.density or 0.00001
    bodyInfo.friction        = params.friction
    bodyInfo.restitution        = params.restitution or 0
    -- 形状
    bodyInfo.shape            = params.shape
    -- 是否子弹刚体
    bodyInfo.bullet            = params.bullet
    -- 旋转角度
    bodyInfo.angle            = params.angle
    -- 固定角度
    bodyInfo.fixedRotation    = params.fixedRotation
    -- 分组编号：用于标记和处理是否产生碰撞 
    bodyInfo.groupIndex        = params.groupIndex
    -- 类别标志位
    bodyInfo.categoryBits    = params.categoryBits
    -- 遮罩标志位 
    bodyInfo.maskBits        = params.maskBits
    return bb.si.Box2D.createBody(bodyInfo)
end

-- 去重随机获取性别和皮肤
function M:getSkirAndSex()
    if #self._skirTable == 0 then
        for i = 1, 2 do
            table.insert(self._skirTable, i)
        end
    end
    local sex = 2
    T.removeOrder(self._sexTable, sex, false)
    local skir = T.random(self._skirTable)
    T.removeOrder(self._skirTable, skir, false)
    return skir, sex
end

-- 同步精灵的位置跟角度
-- delay:刷新频率
function M:tick()
    if not self._world then return end
    -- 速度迭代device、位置迭代
    -- 迭代次数越多越真实，但是性能越低。
    -- 为了模拟真实我一般都是设置成32，但是这款项目刚体过多，模拟比较耗性能，索引改成8和1，网上也有人建议设置为10.
    local velocityIterations, positionIterations = 8, 8
    --物理引擎进行物理模拟，生成模拟后的数据
    self._world:Step(1 / 60, velocityIterations, velocityIterations)
    -- 同步物理世界和渲染世界
    local body = self._world:GetBodyList()
    -- if not self._enterWater then return end
    if body then
        while body do
            local objectData = body:GetUserData()
            if objectData then
                objectData = tolua.cast(objectData, "cc.Node")
                local spr = tolua.cast(body:GetUserData(), "cc.Sprite")
                if body._followPic then
                    local x = spr:worldpoint().x / PTM_RATIO
                    local y = spr:worldpoint().y / PTM_RATIO
                    body:SetTransform(b2Vec2(x, y), CC_DEGREES_TO_RADIANS(-spr:rotate()))
                else
                    local x, y = body:GetPosition().x * PTM_RATIO, body:GetPosition().y * PTM_RATIO
                    spr:setPosition(spr:getParent():convertToNodeSpace(ccp(x, y)))
                    spr:rotate(-body:GetAngle() * 32)
                    -- spr._angle = spr:rotate()
                end
            end
            -- 获得下一个body
            body = body:GetNext()
        end
    end
end

-- 引导开始
function M:guideBegin()
    self:gameStart()
end

-- 游戏开始
function M:gameStart()
    self._emptyNode:bindTouch()
    self:guide()
end

-- 引导
function M:guide(id)
    if self._handGuide then return end
    local posX = 40
    id = 1
    if self._guideBall[id]:px() > V.w_2 then
        posX = 0
    end
    local startPos = ccp(self._guideBall[id]:px() + 35, 85)
    local endPos = ccp(self._guideBall[id]:px() + posX, 275)
    -- 加载引导手
    local hand = D.img("common/guide/hand_1.png"):to(self._emptyNode):p(startPos)
    :scale(.6):z(V.h + 1):hide()
    self._handGuide = hand
    -- 开启引导调度器
    self._guideSch = A.cycle({
        {"show"},
        {"image_range", "common/guide/hand_", 1, 3, .3},
        {"moveto", .9, endPos},
        {"hide"},
        {"moveto", .9, startPos},
    }):at(hand)
end

--  移除引导
function M:removeGuide(id, ball)
    self._delayGuide = CommonFun:stopAction(self, self._delayGuide)
    self:stopAction(self._delayAction)
    self._delayAction = nil
    self:stopAction(self._action)
    self._action = nil
    -- 移除引导手
    if self._handGuide then
        self._handGuide:remove()
        self._handGuide = nil
    end
        local action = A.cycle({
            {"delay", 1/20},
            {"fn", function()
                if (ball:py() < 345 and ball._body:GetLinearVelocity().y < -1) or ball._pitchID ~= 2 then
                    ball._canScore = false
                    ball._pitchID = 2
                    self:stopAction(self._action)
                    self._action = nil
                    if self._score < 2 then
                        if self._score == 0 or (self._score == 1 and #self._guideBall == 0 ) then
                            self:addOneBall(id)
                        end
                    end    
                    local action = A.line({
                        { "delay", 1},
                        { "fn", function()
                            if self._score < 2 then
                                if id == 2 then
                                    self:guide(1)
                                else
                                    self:guide(2)
                                end
                            else
                                SC.close(self._openSC)
                                CommonFun:switchScene("ball", 1, { layerId = 1 })
                            end
                        end },
                    }):at(self)
                    self._delayAction = action
                end
            end},
        }):at(self)
        self._action = action
end

----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()
    -- [超类调用]
	M.super.onExitTransitionStart(self)
end

return M 