--[[
Copyright (c) 2012-2020 Baby-Bus.com

http://www.baby-bus.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]
--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]
----------------------
-- 类
----------------------
local M = classLayerTouch("Main")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- [操作变量]
-- ..
--篮球
local Ball    = require("app.ball.node.Ball")
--篮筐
local Basket    = require("app.ball.node.Basket")



----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 总时间
    self._totalTime        = 80
    -- 星级百分比
    self._percent        = { 30, 60, 96 }
    --篮球集合
    self._balls            = {}
    --得分
    self._score            = 0
    --预备的球个数    
    self._ballCount        = 4
    --有空位时开始计算(补充一个球所需要的时间)
    self._delayTime        = 2
    --初始球坐标
    self._ballPos        = {
        cc.p(330 + X_OFFSET, -42),
        cc.p(430 + X_OFFSET, -42),
        cc.p(530 + X_OFFSET, -42),
        cc.p(630 + X_OFFSET, -42),
    }
    --篮筐
    self._basket            = {}
    --需要补充的球的id的集合
    self._needAddBallId    = {

    }
    --补充球检测
    self._canCheck        = true

    -- 皮肤类型表
    self._skirTable        = { 1, 2 }
    -- 性别表
    self._sexTable        = { 1, 2 }
    --背后的4个小孩
    self._children        = {}
    -- 胖子
    self._fatBoy            = {}
    --
    self._canTouch        = false

    self._gravityRate    = 4

    self._setY            = 0
    self._removeCount    = 0
end








----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- 设置本地数据
    LData:getLocalData()
    --背景
    self:loadBg()
    self:loadCloud()
    --物理世界
    self:loadWorld()
    --加载小孩
    self:loadKid()
    --加载初始篮筐
    self:loadBasket()
    --加载进度条
    self:loadBar()
    --加载篮球
    self:loadBall()
    --加载地面刚体
    self:loadLandBody()
    --加载拖痕
    self:loadTrace()
    --加载苦力胖子
    self:loadFatKid()
end

function M:onEnterTransitionFinish()
    sound:playMusicBackground007()
    for i = 1, 12 do
        R.loadTexture("ball/light/1000" .. i .. ".png")
    end
    self:openSC()
    self:addBall()
    self:cycle({
        { "delay", 1 / 10 },
        { "fn", function()
            for i, v in pairs(self._balls) do
                if v._body then
                    if v:px() < -50 or v:px() > V.w + 50 or v:py() < -50 then
                        self._world:DestroyBody(v._body)
                        T.removeOrder(self._balls, v)
                        v:removeAllTailAction()
                        self._removeCount = self._removeCount + 1
                        -- print("删除"..self._removeCount)
                        v:remove()
                    end
                end
            end
        end },
    })
end

-- 开启实时刷新
function M:openSC()
    if self._openSC then return end
    -- 开启物理世界更新
    local openSC = scheduler.scheduleUpdateGlobal(handlerP(self, self.tick))
    self._openSC = openSC

    -- self.contactListener = bb.si.Box2D.openContactListener(self.contactListenerCallBack, self)
end

-- 游戏开始
function M:gameBegin()
    -- CommonFun:loadTouchNode(self._emptyNode)                            -- 加载全屏触控光圈
    self._progressBar:openSC()
    self:loadCamera()
    self._emptyNode:bindTouch()
end

--加载背景
function M:loadBg()
    local emptyNode = U.loadNode({ contentSize = cc.size(0.1, 0.1) }):to(self):p(0, -45 + self._setY):unbindTouch()
    self._emptyNode = emptyNode
    CommonFun:playFirstGuideVoice(self)
    local bg = U.loadBg("ball/bg/bg.png"):to(self._emptyNode)
    --草地
    local land = D.img("ball/bg/land.png"):to(self._emptyNode)
    :anchor(cc.p(0.5, 0)):p(V.w_2, -1):scale(1.2):bindTouch():opacity(0)
    --篮球线
    local line = D.img("ball/bg/line.png"):to(self._emptyNode)
    :anchor(cc.p(0.5, 0)):p(V.w_2, 10):scale(1.25)
    --旗子
    self:loadFlags(land)
end

-- 加载云
function M:loadCloud()
    local sprite = import("app.camp.node.Cloud")
    local point = { cc.p(284.8 + X_OFFSET, 771.9), cc.p(333.7 + X_OFFSET, 691.6), cc.p(793.9 + X_OFFSET, 777.6), cc.p(647 + X_OFFSET, 710.3) }
    for i = 1, 4 do
        local cloud = sprite.new({
            imgName = "camp/cloud/cloud_" .. (math.fmod(i, 2) + 1) .. ".png",
            main = self,
            id = i,
            pos = point[i],
            scale = .5
        }):to(self._emptyNode, -5):p(point[i]):scale(.5)
    end
end

--加载旗子
function M:loadFlags(land)
    --小旗子(红1黄2蓝3)
    local id = {
        2, 3, 3, 1, 2, 3, 2, 1, 3, 1,
    }
    --小旗子
    local pos = {
        cc.p(30, land:ch() + 5), cc.p(70, land:ch() - 5), cc.p(307, land:ch() + 10),
        cc.p(403, land:ch()), cc.p(439, land:ch() + 5), cc.p(635, land:ch() + 5),
        cc.p(670, land:ch()), cc.p(711, land:ch() - 1), cc.p(950, land:ch() - 15),
        cc.p(990, land:ch() - 5)
    }
    for i, v in pairs(pos) do
        local flag = D.img("common/flag/" .. id[i] .. "/flag_1.png"):to(land, 1)
        :anchor(cc.p(0.5, 0)):p(v):scale(0.8)
        local grass = D.img("ball/bg/flaggrass" .. RD.num(1, 3) .. ".png"):to(flag)
        :p(0, 4)
        flag:cycle({
            { "image", "common/flag/" .. id[i] .. "/flag_", 8, 0.1 },
        })
    end
end

--加载篮筐
function M:loadBasket(pos)
    pos = ifnil(pos, cc.p(V.w_2, 450))
    local basket = Basket.new({
        main    = self,
        pos    = pos,
        world = self._world
    }):to(self._emptyNode, 2):p(pos)
    table.insert(self._basket, basket)
end

--加载进度条
function M:loadBar()
    local sprite    = require("app.ball.node.ProgressBar")
    local progressBar = sprite.new({
        main        = self,
        pos        = cc.p(65.5 + X_OFFSET, 242.7 + Y_OFFSET),
        time        = self._totalTime,
        percent    = self._percent,
    }):to(self, 50):scale(0.9)
    self._progressBar = progressBar
    progressBar:enter(function()
        -- progressBar:openSC()
    end)
end

--加载物理世界
function M:loadWorld()
    -- 创建世界
    local world    = b2World(b2Vec2(0, -25 * self._gravityRate))
    -- 允许静止的物体休眠
    world:SetAllowSleeping(true)
    -- 开启连续物理检测，使模拟更加的真实
    -- world:SetContinuousPhysics(true)
    self._world = world
    bb.si.Box2D.setWorld(self._world)
    local left = -200
    local right = V.w + 200
    -- local debugDraw = GB2DebugDrawLayer:create(self._world, PTM_RATIO)
    -- self:add(debugDraw, 9999)
end

--加载初始篮球
function M:loadBall()
    local pos = self._ballPos
    for i = 1, 4 do
        local ball = Ball.new({
            main    = self,
            id    = i,
            world = self._world,
        }):to(self._emptyNode, 20):bindTouch({ pnRect = true, rectFactorX = 1.5, rectFactorY = 1.5 })
        :p(pos[i])
        table.insert(self._balls, ball)
    end
end

--加载一个篮球
function M:addOneBall(id)
    local pos = self._ballPos
    local ball = Ball.new({
        main    = self,
        id    = id,
        world = self._world,
    }):to(self._emptyNode, 20):bindTouch({ pnRect = true, rectFactorX = 1.5, rectFactorY = 1.5 })
    :p(pos[id])
    ball:setCameraMask(2, true)
    table.insert(self._balls, ball)
end

--加载地板
function M:loadLandBody()
    local categoryBits = { 4, 16 }
    local maskBits    = { 6, 6 }
    local point        = {
        cc.p(V.w_2, 130 + self._setY), cc.p(V.w_2, 410 + self._setY),
    }
    for i = 1, 2 do
        local mask    = U.loadNodeMask({
            contentSize = CCSize(0.1, 0.1),
            color    = ccc4(247, 203, 118, 255),
        }):unbindTouch():hide()
        :to(self):p(point[i])

        -- 加载刚体
        local body = self:loadBody({
            bodyType    = b2_staticBody,
            point    = point[i],
            userData    = mask,
            -- 密度
            density    = 0,
            -- 摩擦力
            friction    = 0.8,
            restitution        = 0.6,
            shape    = {
                {
                    shapeType = EShape.POLYGON,
                    points    = {
                        ccp(-V.w_2, -10), ccp(-V.w_2, 0),
                        -- ccp(-1, 20),  ccp(1, 20),
                        ccp(V.w_2, 0), ccp(V.w_2, -10)
                    }
                },
            },
            categoryBits = categoryBits[i],
            maskBits = maskBits[i],
        })
        mask._land = true
    end
end

--补充球
function M:addBall()
    if self._addBallAction then return end
    local action = A.cycle({
        { "delay", 1 / 30 },
        { "fn", function()
            if #self._needAddBallId > 0 and self._canCheck and not self._cameraMove then
                self._canCheck = false
                self:line({
                    { "delay", self._delayTime },
                    { "fn", function()
                        if not self._gamerOver then
                            self._canCheck = true
                            self:addOneBall(self._needAddBallId[1])
                            table.remove(self._needAddBallId, 1)
                        end
                    end },
                })
            end
        end },
    }):at(self)
    self._addBallAction = action
end

function M:stopAddBall()
    if not self._addBallAction then return end
    self:stopAction(self._addBallAction)
    self._addBallAction = nil
end

-- 加载刚体
function M:loadBody(params)
    -- 定义刚体信息
    local bodyInfo            = {}
    -- 类型：静态(b2_staticBody),动态(b2_dynamicBody),平台(b2_kinematicBody)
    bodyInfo.type        = params.bodyType
    -- 位置
    bodyInfo.pos            = params.point
    -- 用户数据：存储用户的数据，可以是任何类型的数据。一般要求存储的数据的类型是一致的
    bodyInfo.userData        = params.userData
    -- 角度阻尼：阻碍刚体旋转的系数，让刚体旋转更容易停下来。
    bodyInfo.angularDamping    = params.angularDamping or 0
    -- 密度：影响刚体的惯性、重量等
    bodyInfo.density            = params.density or 0.00001
    bodyInfo.friction        = params.friction
    bodyInfo.restitution        = params.restitution or 0
    -- 形状
    bodyInfo.shape            = params.shape
    -- 是否子弹刚体
    bodyInfo.bullet            = params.bullet
    -- 旋转角度
    bodyInfo.angle            = params.angle
    -- 固定角度
    bodyInfo.fixedRotation    = params.fixedRotation
    -- 分组编号：用于标记和处理是否产生碰撞 
    bodyInfo.groupIndex        = params.groupIndex
    -- 类别标志位
    bodyInfo.categoryBits    = params.categoryBits
    -- 遮罩标志位 
    bodyInfo.maskBits        = params.maskBits
    return bb.si.Box2D.createBody(bodyInfo)
end

--加载小孩
function M:loadKid()
    local boneTbale = { "ph", "gh" }
    local sprite = import("app.ball.node.FatKid")
    local path = "ball"
    local bone = "ph"
    local kidPos = cc.p(-(58 + X_OFFSET) + 60 + X_OFFSET, 496)
    local children = sprite.new({
        file            = "bone/" .. path .. "/" .. bone,
        armatureName    = bone,
        main            = self,
        type            = 1, -- 1:胖小孩，2：正常小孩，3：摇绳小孩
        index        = 1, -- 小朋友的索引
        sex            = 2,
        offsetX        = (58 + X_OFFSET),
        pos            = cc.p(60 + X_OFFSET, 456),
    }):to(self._emptyNode, 0):p(kidPos):scale(0.55):anchor(ccp(0.5, 0))
    :flipX(true)
    table.insert(self._children, children)
    children:enter()

    local sprite = import("app.ball.node.Kid")
    local type = "gh"

    local flipX = {true, false, false }
    local pos = {
        cc.p(155 + X_OFFSET, 453),
        cc.p(827 + X_OFFSET, 460),
        cc.p(924 + X_OFFSET, 478),
    }
    local offset_x = {
        (146 + X_OFFSET) + 188,
        V.w + 105 - (810 + X_OFFSET),
        V.w + 200 - (905 + X_OFFSET)
    }
    local number = { 3, 1, 2 }
    local hairId = { 2, 0, 1 }
    local z    = { 0, 0, 0 }
    local breathIds = { 1, "", 1 }
    for i = 1, 3 do
        local kidPos = cc.p(offset_x[i] + pos[i].x, pos[i].y)
        if i < 2 then
            kidPos = cc.p(-offset_x[i] + pos[i].x, pos[i].y)
        end
        local id = RD.num(1, #breathIds)
        local breathID = breathIds[id]
        table.remove(breathIds, id)
        local player = sprite.new({
            file    = "bone/ball/" .. type,
            armatureName = type,
            animName = type .. "_ydtb",
            main    = self,
            type    = type,
            number = number[i],
            hairId = hairId[i],
            pos    = pos[i],
            offsetX = offset_x[i],
            breathID = breathID,
            id    = i,
            scale    = 0.55,
        }):to(self._emptyNode, z[i]):scale(0.55):unbindTouch()
        :p(kidPos, 450):anchor(ccp(0.5, 0)):flipX(flipX[i])
        player:enter()
        print(breathID)
        table.insert(self._children, player)
    end
end

--加载推篮球胖子
function M:loadFatKid()
    -- 获取骨骼类型
    local type = { 1, 1, 1, 1 }
    -- 1女 2男
    local sex = { 2, 2, 2, 2 }
    local pos = {
        cc.p(1170 + X_OFFSET, 296), cc.p(1380 + X_OFFSET, 296),
        cc.p(-300, 296), cc.p(-90, 296),
    }
    local flipX = {true, false, true, true }
    for i = 1, 4 do
        local path = "ball"
        local bone = "ph"
        local children = self:createNode("FatKid", {
            file            = "bone/" .. path .. "/" .. bone,
            armatureName    = bone,
            main            = self,
            type            = type[i], -- 1:胖小孩，2：正常小孩，3：摇绳小孩
            index        = i, -- 小朋友的索引
            sex            = sex[i]
        }):to(self._emptyNode, 3):p(pos[i]):scale(0.58):anchor(ccp(0.5, 0))
        :flipX(flipX[i]):unbindTouch()
        children:play("ph_tui")
        table.insert(self._fatBoy, children)
    end
    --加载 
    --加载第二个篮球框
    self:loadBasket(cc.p(1275 + X_OFFSET, 440))
end

--加载拖痕
function M:loadTrace()
    local trace = U.loadProgressBar({                
    imgName        = "ball/line.png",
    mode            = kCCProgressTimerTypeBar,
    midpoint        = ccp(1, 0),        
    --[可选] 起始点（可以理解为进度的作用锚点）
    barChangeRate = ccp(1, 0),        
    --[可选] 进度改变的方向 
    isAutoRun    = false,
    }):p(V.w + 30, 335):to(self._emptyNode):anchor(cc.p(1, 0.5))
    self._trace = trace
end

--进入两个篮筐
function M:startStep2()
    self._delayTime = 1.6
    self:encourage()
    self._basket[1]:place()
    self:cameraMove()
    local dis = V.w_2 + 180 - self._basket[2]:px()
    self:stepBasketMove(self._basket[2], 3, cc.p(V.w_2 + 180, self._basket[2]:py()))
    for i = 1, 2 do
        self._fatBoy[i]:line({
            { "moveBy", 3, cc.p(dis, 0) },
            { "fn", function()
                self._fatBoy[i]:runEvent()
                self._fatBoy[i]:z(20)
                if i == 2 then
                    self._fatBoy[i]:flipX(true)
                end
            end },
            { "moveBy", 1.3, cc.p(V.w_2 + 50, 0) },
        })
    end
    self:line({
        { "delay", 1 },
        { "fn", function()
            self._trace:progressTo({ value = 100, time = 2 })
        end },
    })
    local pos = {
        cc.p(V.w_2 - 105, 296), cc.p(V.w_2 + 105, 296)
    }
    --左边推
    for i = 3, 4 do
        self._fatBoy[i]:runEvent()
        self._fatBoy[i]:z(20)
        self._fatBoy[i]:line({
            { "moveTo", 1.3, pos[i - 2] },
            { "fn", function()
                self._fatBoy[i]:z(3)
                if i == 4 then
                    self._fatBoy[i]:flipX(false)
                    soundEffect:playEffectsfx26222015()
                end
                self._fatBoy[i]:play("ph_tui")
                if i == 3 then
                    self:stepBasketMove(self._basket[1], 1, cc.p(V.w_2 - 150, self._basket[1]:py()))
                end
            end },
            { "moveBy", 1, cc.p(-150, 0) },
            { "fn", function()
                self._fatBoy[i]:z(20)
                self._fatBoy[i]:runEvent()
                if i == 3 then
                    self._fatBoy[i]:flipX(false)
                end
            end },
            { "moveBy", 1.3, cc.p(-V.w_2, 0) },
            { "fn", function()
                if i == 3 then
                    self:cameraMove(-1)
                end
            end },
        })
    end
end

--篮球框移动
function M:stepBasketMove(basket, time, pos)
    local pos1 = cc.pSub(pos, basket:point())
    basket:line({
        { "moveTo", time, pos },
    })
    basket._mask:line({
        { "moveBy", time, pos1 },
    })
    basket._net:line({
        { "moveBy", time, pos1 },
    })
end

--鼓掌
function M:applause()
    if self._children[2]._cheer then return end
    if self._gamerOver then return end
    local num = RD.num(1, 2)
    if num == 1 then
        soundEffect:playEffectsfx26222020()
    else
        soundEffect:playEffectsfx26222021()
    end
    for i, v in pairs(self._children) do
        if i == 1 then
            v:play("ph_gz")
        else
            v:cheer()
        end
    end
end

--激励
function M:encourage()
    for i, v in pairs(self._children) do
        if i == 1 then
            v:play("ph_hh")
        else
            v:cheer_1()
        end
    end
end

--step3
function M:startStep3()
    self._delayTime = 1
    self:encourage()
    self:cameraMove()
    local pos = {
        cc.p(self._basket[2]:px() - 105, 296), cc.p(self._basket[2]:px() + 105, 296)
    }
    local dis = 50
    local flipX = {false, false, true, true }
    for i, v in pairs(self._fatBoy) do
        v:flipX(flipX[i])
    end
    --右边推
    for i = 1, 2 do
        self._fatBoy[i]:runEvent()
        self._fatBoy[i]:z(20)
        self._fatBoy[i]:line({
            { "moveTo", 1.3, pos[i] },
            { "fn", function()
                self._fatBoy[i]:z(3)
                if i == 2 then
                    self._fatBoy[i]:flipX(false)
                elseif i == 1 then
                    self._fatBoy[i]:flipX(true)
                end
                self._fatBoy[i]:play("ph_tui")
                if i == 1 then
                    self:stepBasketMove(self._basket[2], 0.5, cc.p(V.w_2 + 230, self._basket[2]:py()))
                    soundEffect:playEffectsfx26222019()
                end
            end },
            { "moveBy", 0.5, cc.p(dis, 0) },
            { "fn", function()
                self._fatBoy[i]:runEvent()
                self._fatBoy[i]:flipX(true)
                self._fatBoy[i]:z(20)
            end },
            { "moveBy", 1.3, cc.p(V.w_2, 0) },
            { "fn", function()
                if i == 1 then
                    self:cameraMove(-1)
                end
            end },
            { "remove" }
        })
    end

    local pos = {
        cc.p(self._basket[1]:px() - 105, 296), cc.p(self._basket[1]:px() + 105, 296)
    }
    --左边推
    for i = 3, 4 do
        self._fatBoy[i]:runEvent()
        self._fatBoy[i]:z(20)
        self._fatBoy[i]:line({
            { "moveTo", 1.3, pos[i - 2] },
            { "fn", function()
                self._fatBoy[i]:z(3)
                if i == 4 then
                    self._fatBoy[i]:flipX(false)
                elseif i == 3 then
                    self._fatBoy[i]:flipX(true)
                end
                self._fatBoy[i]:play("ph_tui")
                if i == 3 then
                    self:stepBasketMove(self._basket[1], 0.5, cc.p(V.w_2 - 200, self._basket[1]:py()))
                end
            end },
            { "moveBy", 0.5, cc.p(-dis, 0) },
            { "fn", function()
                self._fatBoy[i]:z(20)
                self._fatBoy[i]:runEvent()
                if i == 3 then
                    self._fatBoy[i]:flipX(false)
                end
            end },
            { "moveBy", 1.3, cc.p(-V.w_2, 0) },
            { "fn", function()
                self:basketMove(2)
            end },
            { "remove" }
        })
    end
end

-- 去重随机获取性别和皮肤
function M:getSkirAndSex()
    if #self._skirTable == 0 then
        for i = 1, 2 do
            table.insert(self._skirTable, i)
        end
    end
    local sex = 2
    local skir = 1
    -- T.removeOrder(self._sexTable, sex, false)
    -- local skir = T.random(self._skirTable)
    -- T.removeOrder(self._skirTable, skir, false)
    return skir, sex
end

--球框移动
function M:basketMove(num)
    for i = 1, num do
        self._basket[i]:satrtMove()
    end
end
----------------------
-- 内部函数
----------------------
function M:createEdge(startP, endPos, density)
    density = density or 0
    if not self._testBody then
        self._testBody = self._world:CreateBody(b2BodyDef())
    end
    if not self._testBox then
        self._testBox = b2EdgeShape()
    end
    self._testBox:Set(b2Vec2(startP.x / PTM_RATIO, startP.y / PTM_RATIO),
    b2Vec2(endPos.x / PTM_RATIO, endPos.y / PTM_RATIO))
    self._testBody:CreateFixtureConst(self._testBox, density)
end

-- 同步精灵的位置跟角度
-- delay:刷新频率
function M:tick()
    if not self._world then return end
    -- 速度迭代device、位置迭代
    -- 迭代次数越多越真实，但是性能越低。
    -- 为了模拟真实我一般都是设置成32，但是这款项目刚体过多，模拟比较耗性能，索引改成8和1，网上也有人建议设置为10.
    local velocityIterations, positionIterations = 8, 8
    --物理引擎进行物理模拟，生成模拟后的数据
    self._world:Step(1 / 60, velocityIterations, velocityIterations)
    -- 同步物理世界和渲染世界
    local body = self._world:GetBodyList()
    -- if not self._enterWater then return end
    if body then
        while body do
            local objectData = body:GetUserData()
            if objectData then
                objectData = tolua.cast(objectData, "cc.Node")
                local spr = tolua.cast(body:GetUserData(), "cc.Sprite")
                if body._followPic then
                    local x = spr:worldpoint().x / PTM_RATIO
                    local y = spr:worldpoint().y / PTM_RATIO
                    body:SetTransform(b2Vec2(x, y), CC_DEGREES_TO_RADIANS(-spr:rotate()))
                else
                    local x, y = body:GetPosition().x * PTM_RATIO, body:GetPosition().y * PTM_RATIO
                    spr:setPosition(spr:getParent():convertToNodeSpace(ccp(x, y)))
                    spr:rotate(-body:GetAngle() * 32)
                    spr._angle = spr:rotate()
                end
            end
            -- 获得下一个body
            body = body:GetNext()
        end
    end
end

-- 逻辑[刚体接触检测]
function M:contactListenerCallBack(contactType, contact, oldManifold)
    if contactType == GB2_CONTACTTYPE_BEGIN_CONTACT then
        -- 获取碰撞的刚体
        local bodyA = contact:GetFixtureA() and contact:GetFixtureA():GetBody() or nil
        local bodyB = contact:GetFixtureB() and contact:GetFixtureB():GetBody() or nil
        if bodyA and bodyB then
            local sprA = tolua.cast(bodyA:GetUserData(), "cc.Sprite")
            local sprB = tolua.cast(bodyB:GetUserData(), "cc.Sprite")
            if (sprA._land or sprB._touchLand) then
                sprB:playTouchLandEffectt()
            elseif (sprB._land or sprA._touchLand) then
                sprA:playTouchLandEffectt()
            elseif (sprA._basket and sprB._touchBasket) then
                sprB:playTouchBasketEffectt()
            elseif (sprB._basket and sprA._touchBasket) then
                sprA:playTouchBasketEffectt()
            end
        end
    elseif contactType == GB2_CONTACTTYPE_END_CONTACT then
        -- print("结束接触")
    elseif contactType == GB2_CONTACTTYPE_PRE_SOLVE then
        -- print("求解前")
    elseif contactType == GB2_CONTACTTYPE_POST_SOLVE then
        -- print("求解后")
    end
end

--摄像机
function M:loadCamera()
    cc.Director:getInstance():setProjection(cc.DIRECTOR_PROJECTION3_D)--cocos2d::DisplayLinkDirector::Projection::_2D
    local mapSize = self._mapSize
    local camera = cc.Camera:create()
    camera:setCameraFlag(2)
    self:addChild(camera)
    camera:setPosition(V.w_2, V.h_2)
    self._camera = camera
    self:setCameraMask(2, true)
    --摄像机位置调整，适配
    -- self:setTouchOffset(cc.pAdd(cc.p(camera:getPosition()), ccp(-V.w_2, -V.h_2)))
end

--游戏进阶成功
function M:cameraMove(flag)
    if not self._camera then return end
    self._progressBar:closeSC()
    self._emptyNode:unbindTouch()
    flag = ifnil(flag, 1)
    local srcPo = ccp(self._camera:getPosition())
    local count = 30
    local rate = 3 * flag
    self:cycle({
        { "fn", function()
            self._camera:setPosition3D({
                x = self._camera:getPositionX(),
                y = self._camera:getPositionY() + (flag * 2),
                z = self._camera:getPosition3D().z - rate,
            })
        end },
        { "delay", 1 / 60 },
    }, count)
    self._cameraMove = true
    if flag == -1 then
        self:line({
            { "delay", count * 1 / 60 },
            { "fn", function()
                self._progressBar:openSC()
                self._emptyNode:bindTouch()
                GLOBAL_TOUCH_LOCK = false
                self._cameraMove = false
            end },
        })
        self._children[2]._cheer = false
    else
        self._children[2]._cheer = true
    end
end

-- 游戏结束
function M:gameFinish()
    print("最终得分为:" .. self._score)
    for i, v in pairs(self._basket) do
        v:finishMove()
        v:smaller()
    end
    self:stopAddBall()
    for i, v in pairs(self._balls) do
        v:unbindTouch()
    end
    self._gamerOver = true
    self:stimulate()
    CCNodeExtend.extend(self._camera)
    self._camera:remove()
    self._camera = nil
    self:setCameraMask(1, true)
end

-- 激励
function M:stimulate()
    for i, v in pairs(self._children) do
        if i > 1 then
            v:cheer_1()
        else
            v:play("ph_hh", 1.5)
        end
    end
    CommonFun:finishParticle(self)
    self:performWithDelay(function()
        CommonFun:loadSettleAccount(self, self._progressBar._starNum)
    end, 3)
end

----------------------
-- 点击
----------------------
--[[--

判断是否点击到内容区域

### Example:

### Parameters:
-   string **event**    事件名称
-   number **x**        点击位置x(相对于屏幕)
-   number **y**        点击位置y(相对于屏幕)

]]
function M:onTouchBegan(x, y)
    return true
end

function M:onTouchMoved(x, y)

end

function M:onTouchEnded(x, y)

end

----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()
    GLOBAL_TOUCH_LOCK = true
    A.stopTarget(self)
    if not self._isNotClear then
        R.removeAllTextures()
    end
end

return M