--[[Copyright (c) 2012-2020 baby-bus.com

http://www.baby-bus.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]
--[[!--

篮球类

]]
----------------------
-- 类
----------------------
local M    = classSprite("Ball")
M.TAG    = "Ball"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return:
-   object              对象实例

]]
function M:ctor(params)
    --J.assert(M.TAG, params.images ~= nil and #params.images == 4,"images size must be 4")
    -- [超类调用]
    M.super.ctor(self, params)
    self._world        = params.world
    -- 编号（第几个球）
    self._id            = params.id
    -- mainLayer
    self._main        = params.main
    --刚体半径
    self._radius        = 0

    self._maskBits    = 8
    --刚体角度
    self._angle        = 0
    --是否可以前后碰撞（只有在正常投射的时候可以）
    self._canFly        = true

    -- 拖尾个数
    self._tailNum    = 15
    -- 尾巴集合
    self._tailTable    = {}
    -- 尾巴坐标X
    self._tailPosX    = {}
    -- 尾巴坐标Y
    self._tailPosY    = {}
    --角速度正为逆时针
    self._angleSpeed    = 10

    self._canTouch    = false

    self._tocuhLand    = 1
    self._touchBasket = true
    self._touchBasketOver = false
end




----------------------
-- 结点渲染
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    self:loadDisplay()
    --加载影子
    self:loadShadow()
    -- 开启尾巴调度器
    self:openTailScheduleGlobal()
end

function M:onEnterTransitionFinish()
    self:line({
        { "delay", 0.1 },
        { "fn", function()
            self:enter()
        end }
    })
end

-- 图片渲染
function M:loadDisplay()
    bb.si.Box2D.setWorld(self._world)
    self:display("ball/ball.png")
end

--进场
function M:enter()
    local num = RD.num(1, 2)
    if num == 1 then
        soundEffect:playEffectsfx26222007()
    else
        soundEffect:playEffectsfx26222008()
    end
    self:line({
        A.union({
            { "moveTo", 0.5, cc.p(self:px(), 115) },
            { "image", "ball/ball/ball_", 3, 5 / 30 }
        }),
        { "fn", function()
            self:display("ball/ball.png")
            -- 加载拖尾
            self:loadTail()
            self._canTouch = true
        end },
    })
end

-- 加载刚体
function M:loadBody()
    -- 定义刚体信息
    local bodyInfo            = {}
    -- 类型：静态(b2_staticBody),动态(b2_dynamicBody),平台(b2_kinematicBody)
    bodyInfo.type            = b2_dynamicBody
    -- 位置
    bodyInfo.pos                = self:worldpoint()
    -- 用户数据：存储用户的数据，可以是任何类型的数据。一般要求存储的数据的类型是一致的
    bodyInfo.userData        = self
    -- 密度：影响刚体的惯性、重量等
    bodyInfo.density            = 2
    -- 旋转角度
    bodyInfo.angle            = self._angle
    --摩擦力
    bodyInfo.friction        = 10
    -- bodyInfo.fixedRotation      = true   
    --弹性系数
    bodyInfo.restitution        = 0.6
    -- 形状
    bodyInfo.shape            = {
        {
            shapeType = EShape.CIRCLE,
            radius    = self._radius
        },
    }
    -- bodyInfo.groupIndex          = -1
    -- 类别标志位
    bodyInfo.categoryBits    = 4
    -- 遮罩标志位 
    bodyInfo.maskBits        = self._maskBits
    local body = bb.si.Box2D.createBody(bodyInfo)
    if self._spped then
        self._body:SetAngularVelocity(self._angleSpeed)
        body:SetLinearVelocity(self._spped)
        body._init = false
    else
        -- body:SetGravityScale(0)
        body._init = false
    end
    self._body = body
    self._pos = nil
end

--更新
function M:update()
    self:line({
        { "delay", 0.25 },
        { "fn", function()
            self._update = A.cycle({
                { "delay", 1 / 60 },
                { "fn", function()
                    if self:py() > 460 and not self._canScore then
                        self:stopAction(self._update)
                        self._update = nil
                        self._canScore = true
                        self._maskBits    = 12
                        self:resetBody()
                        self:setMaskBits(12)
                        self:z(9)
                        self:removeAllTailAction()
                    end
                end },
            }):at(self)
        end },
    })
end

function M:resetBody(rate)
    local rate = ifnil(rate, 2 / 9)
    A.cycle({
        { "delay", 1 / 60 },
        { "fn", function()
            if self._body then
                self._spped = self._body:GetLinearVelocity()
                local flag = true
                if flag then
                    if self._body and not self._body._init then
                        self._world:DestroyBody(self._body)
                    end
                    self._radius = self._radius + rate
                    self._angle = self._body:GetAngle()
                    self:loadBody()
                end
            end
        end },
    }, 4):at(self)
end

function M:setMaskBits(num)
    self._body:GetFixtureList():GetFilterData().maskBits = num or 0
end

--飞到赛道
function M:flyOff()
    self:scaleAction(0.7, 0.5)
    self:z(1)
    self:removeAllTailAction()
    -- T.removeOrder(self._main._balls, self)
    self:resetBody(1 / 5)
end


-- 加载拖尾效果
function M:loadTail()
    if self._isMagic then return end
    local point = self:point()
    local zOrder = self:z() - 5
    for i = 1, self._tailNum do
        local scale = self:scale() * (self._tailNum - i + 1) / self._tailNum
        local opacity = 130 * (self._tailNum - i + 1) / self._tailNum
        local tail    = D.img("ball/tail.png"):to(self._main._emptyNode, 19):p(point):show()
        :setOpacity(opacity):scale(scale)
        tail:setCameraMask(2, true)
        table.insert(self._tailPosX, point.x)
        table.insert(self._tailPosY, point.y)
        table.insert(self._tailTable, tail)
    end
end

-- 开启尾巴调度器
function M:openTailScheduleGlobal()
    if self._isMagic or self._gameSch then return end
    if tolua.isnull(self._emptyNode) then
        local emptyNode = U.loadNode({ contentSize = cc.size(0.1, 0.1) }):to(self, 55)
        self._emptyNode = emptyNode
    end

    local gameSch = A.cycle({
        { "delay", 1 / 60 },
        { "fn", function()
            self:upDataWithTailInfos()
        end },
    }):at(self._emptyNode)

    self._gameSch = gameSch
end

-- 获取2点间的角度
function M:calculateAngleWithTwoPoint(point1, point2)
    local angle = math.deg(math.atan2(-(point2.y - point1.y), (point2.x - point1.x)))
    return angle
end

-- 更新尾巴信息
function M:upDataWithTailInfos()
    self._tailPosX[1] = self:px()
    self._tailPosY[1] = self:py()

    for i = 2, #self._tailTable do
        if self._tailPosX[i] then
            self._tailPosX[i] = self._tailPosX[i] + (self._tailPosX[i - 1] - self._tailPosX[i]) * 0.55
            self._tailPosY[i] = self._tailPosY[i] + (self._tailPosY[i - 1] - self._tailPosY[i]) * 0.55

            local posX = self._tailPosX[i]
            local posY = self._tailPosY[i]
            local scaleX = self:scale() * (self._tailNum - i + 1) / self._tailNum
            local scaleY = self:scale() * 6 * (self._tailNum - i + 1) / self._tailNum
            local point1 = ccp(self._tailPosX[i], self._tailPosY[i])
            local point2 = ccp(self._tailPosX[i - 1], self._tailPosY[i - 1])
            local angle = self:calculateAngleWithTwoPoint(point2, point1)

            self._tailTable[i - 1]:setPosition(posX, posY)
            self._tailTable[i - 1]:scaleX(scaleX):scaleY(scaleY)
            self._tailTable[i - 1]:rotate(angle)
        end
    end
end

-- 关闭尾巴调度器
function M:closeTailScheduleGlobal()
    if self._gameSch and not tolua.isnull(self._emptyNode) then
        self._emptyNode:stopAction(self._gameSch)
        self._gameSch = nil

        for i, v in pairs(self._tailTable) do
            v:z(self:z() - 1)
        end
    end
end

-- 移除所有尾巴动作
function M:removeAllTailAction()
    self:closeTailScheduleGlobal()
    if self._tailTable then
        for i, v in ipairs(self._tailTable) do
            if not tolua.isnull(v) then
                local opacity = v:opacity()
                local time    = opacity / 800
                A.line({
                    A.union({
                        { "fadeOut", time },
                        { "scaleTo", time, 0 },
                    }),
                    { "remove" }
                }):at(v)
            end
        end
    end
end

--加载影子
function M:loadShadow()
    local shadow = D.img("ball/ballshadow.png"):to(self, -1)
    :p(40, 8)
    self._shadow = shadow
end

----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()
    SC.close(self._ropeScheduler)
end

--计算速度Y
function M:calculateSpeed()
    local speedY = 0
    if self._dPos.y > 100 and self._dPos.y < 200 then
        speedY = 25
    elseif self._dPos.y >= 200 then
        speedY = 25 + (self._dPos.y - 200) / 50
    elseif speedY < 100 then
        speedY = 25 - (100 - self._dPos.y) / 10
    end
    return speedY
end


--正常投球
function M:normalPitching(speedY)
    self:update()
    self._body:SetGravityScale(1)
    local speedX = 1
    if self._main._progressBar then
        if self._main._progressBar._starNum == 0 then
            speedX = self:loadPlugin1()
        elseif self._main._progressBar._starNum == 1 then
            speedX = self:loadPlugin2()
        else
            speedX = self:loadPlugin3()
        end
    else
        speedX = self:loadGuidePlugin()
    end
    if speedY <= 25.5 then
        speedY = 25.5
    elseif speedY > 26.5 then
        speedY = 26.5
    end
    self._body:SetLinearVelocity(b2Vec2(speedX, speedY * 2 + 0.5))
    self:scaleAction(0.75, 0.6)
end

--引导外挂
function M:loadGuidePlugin()
    local rate = 50
    local speedX = self._dPos.x / rate
    if self._id == 1 then
        if self._dPos.x > 0 and self._dPos.x < 100 then
            local dis = self._dPos.x / 50
            speedX = 1 + dis
        elseif self._dPos.x >= 100 then
            speedX = 2 + self._dPos.x / rate
        end
    elseif self._id == 2 then
        if self._dPos.x < 0 and self._dPos.x > -100 then
            local dis = self._dPos.x / 50
            speedX = -1 + dis
        elseif self._dPos.x <= 100 then
            speedX = -2 + self._dPos.x / rate
        end
    end
    return speedX
end

function M:loadPlugin1()
    local rate = 50
    local speedX = self._dPos.x / rate
    if self._id == 1 then
        if self._dPos.x > 0 and self._dPos.x < 150 then
            local dis = self._dPos.x / 110
            speedX = 5 + dis
        elseif self._dPos.x >= 150 then
            speedX = 6 + self._dPos.x / rate
        end
    elseif self._id == 2 then
        if self._dPos.x > 0 and self._dPos.x < 100 then
            local dis = self._dPos.x / 50
            speedX = 1 + dis
        elseif self._dPos.x >= 100 then
            speedX = 2 + self._dPos.x / rate
        end
    elseif self._id == 3 then
        if self._dPos.x < 0 and self._dPos.x > -100 then
            local dis = self._dPos.x / 50
            speedX = -1 + dis
        elseif self._dPos.x <= 100 then
            speedX = -2 + self._dPos.x / rate
        end
    elseif self._id == 4 then
        if self._dPos.x < 0 and self._dPos.x > -150 then
            local dis = self._dPos.x / 110
            speedX = -5 + dis
        elseif self._dPos.x <= -150 then
            speedX = -6 + self._dPos.x / rate
        end
    end
    return speedX
end

function M:loadPlugin2()
    local rate = 30
    local speedX = self._dPos.x / rate
    if self._id == 1 then
        if self._dPos.x > -70 and self._dPos.x < 70 then
            local dis = self._dPos.x / 50
            speedX = 0 + dis
        elseif self._dPos.x >= 210 and self._dPos.x <= 380 then
            speedX = 9 + self._dPos.x / 120
        end
    elseif self._id == 2 then
        if self._dPos.x < 0 and self._dPos.x > -150 then
            local dis = self._dPos.x / 50
            speedX = -1 + dis
        elseif self._dPos.x > 70 and self._dPos.x < 280 then
            local dis = self._dPos.x / 80
            speedX = 5.5 + dis
        end
    elseif self._id == 3 then
        if self._dPos.x > 0 and self._dPos.x < 180 then
            local dis = self._dPos.x / 50
            speedX = 1.5 + dis
        elseif self._dPos.x < -50 and self._dPos.x > -250 then
            local dis = self._dPos.x / 120
            speedX = -5.8 + dis
        end
    elseif self._id == 4 then
        if self._dPos.x > -70 and self._dPos.x < 70 then
            local dis = self._dPos.x / 50
            speedX = 0 + dis
        elseif self._dPos.x <= -150 and self._dPos.x >= -350 then
            speedX = -8.5 + self._dPos.x / 120
        end
    end
    return speedX
end

function M:loadPlugin3()
    local rate = 30
    local speedX = self._dPos.x / rate
    if self._id == 1 then
        if self._dPos.x <= 20 and self._dPos.x >= 0 then
            speedX = 0
        elseif self._dPos.x >= 130 and self._dPos.x <= 420 then
            speedX = 11.5 + self._dPos.x / 200
        end
    elseif self._id == 2 then
        if self._dPos.x < 0 and self._dPos.x > -200 then
            local dis = self._dPos.x / 50
            speedX = -3 + dis
        elseif self._dPos.x > 100 and self._dPos.x < 330 then
            local dis = self._dPos.x / 80
            speedX = 7 + dis
        end
    elseif self._id == 3 then
        if self._dPos.x > 0 and self._dPos.x < 240 then
            local dis = self._dPos.x / 50
            speedX = 3 + dis
        elseif self._dPos.x < -70 and self._dPos.x > -300 then
            local dis = self._dPos.x / 120
            speedX = -6.5 + dis
        end
    elseif self._id == 4 then
        if self._dPos.x >= -20 and self._dPos.x <= 0 then
            speedX = 0
        elseif self._dPos.x <= -140 and self._dPos.x >= -410 then
            speedX = -11.5 + self._dPos.x / 200
        end
    end
    return speedX
end

--重投
function M:heavilyPitching(speedY)
    self._canFly = false
    local rate = 30
    self._body:SetLinearVelocity(b2Vec2(self._dPos.x / rate, speedY * 2))
    self:scaleAction(0.6, 0.9)
    local pos = self:point()
    self._heavilyAction = A.cycle({
        { "delay", 1 / 60 },
        { "fn", function()
            if self:point().y < pos.y then
                self:stopAction(self._heavilyAction)
                self._maskBits    = 20
                self:setMaskBits(20)
                --飞到赛道
                self:flyOff()
            end
            pos = self:point()
        end },
    }):at(self)
end

--轻投
function M:gentlyPitching(speedY)
    local rate = 20
    self._body:SetLinearVelocity(b2Vec2(self._dPos.x / rate, (speedY - 1) * 1.7))
    local pos = self:point()
    self._gentlyAction = A.cycle({
        { "delay", 1 / 60 },
        { "fn", function()
            if self:point().y < pos.y and self:point().y < 300 then
                self._canChangeMaskBits = true
                self:z(15)
                self:stopAction(self._gentlyAction)
                if speedY > 20 then
                    self._maskBits    = 20
                    self:setMaskBits(20)
                else
                    self._maskBits    = 12
                    self:setMaskBits(12)
                end
                self:removeAllTailAction()
                self:resetBody()
                self:cycle({
                    { "fn", function()
                        local speedX = self._body:GetLinearVelocity().x / math.abs(self._body:GetLinearVelocity().x)
                        self._body:SetLinearVelocity(b2Vec2(self._body:GetLinearVelocity().x + speedX / 5, self._body:GetLinearVelocity().y))
                    end },
                    { "delay", 1 / 60 },
                })
            end
            pos = self:point()
        end },
    }):at(self)
    self:scaleAction(0.85, 0.3)
end

function M:scaleAction(scl, time, speedUp, speed)
    if self._action then
        self:stopAction(self._action)
        self._action = nil
    end
    local action = A.line({
        { "scaleTo", time, scl },
        { "fn", function()
            self._action = nil
        end },
    }):at(self)
    self._action = action
    if speedUp then
        speed = ifnil(speed, 1)
        self:cycle({
            { "fn", function()
                local speedX = self._body:GetLinearVelocity().x / math.abs(self._body:GetLinearVelocity().x) * speed
                self._body:SetLinearVelocity(b2Vec2(self._body:GetLinearVelocity().x + speedX / 5, self._body:GetLinearVelocity().y))
            end },
            { "delay", 1 / 30 },
        })
    end
end

--播放投射音效
function M:playPitchingEffect()
    local num = RD.num(1, 2)
    if num == 1 then
        soundEffect:playEffectsfx26222001()
    else
        soundEffect:playEffectsfx26222002()
    end
end

--接触地面音效
function M:playTouchLandEffectt()
    if self._tocuhLand == 1 then
        soundEffect:playEffectsfx26222004()
    elseif self._tocuhLand == 2 then
        soundEffect:playEffectsfx26222005()
    elseif self._tocuhLand == 3 then
        soundEffect:playEffectsfx26222006()
    end
    self._tocuhLand = self._tocuhLand + 1
end

--篮板音效
function M:playTouchBasketEffectt()
    local num = RD.num(1, 2)
    if num == 1 then
        soundEffect:playEffectsfx26222009()
    else
        soundEffect:playEffectsfx26222010()
    end
    self._touchBasket = false
    self._touchBasketOver = true
end
----------------------
-- 点击
----------------------
--[[--

判断是否点击到内容区域

### Example:

### Parameters:
-   string **event**    事件名称
-   number **x**        点击位置x(相对于屏幕)
-   number **y**        点击位置y(相对于屏幕)

]]
function M:onTouchBegan(x, y)
    if not self._canTouch then return end
    self:scale(1.1)
    return true
end

function M:onTouchMoved(x, y)
    local dPosition = cc.pSub(cc.p(x, y), self:worldpoint())
    self._dPos = dPosition
end

function M:onTouchEnded(x, y)
    if not self._dPos then
        self:scale(1)
        return
    end
    --加载刚体
    self:loadBody()
    self:z(22)
    for i, v in pairs(self._tailTable) do
        v:z(21)
    end
    self._shadow:hide()
    --根据拖动距离求Y的速度
    local speedY = self:calculateSpeed()
    self._body:SetGravityScale(1)
    self:unbindTouch()
    if x > self:px() then
        self._angleSpeed = -5
    else
        self._angleSpeed = 5
    end
    self._body:SetAngularVelocity(self._angleSpeed)
    --重投
    if speedY >= 28 then
        self._pitchID = 1
        self:heavilyPitching(speedY)
        --正常
    elseif speedY >= 24 then
        self._pitchID = 2
        self:normalPitching(speedY)
        --轻投
    else
        self._pitchID = 3
        self:gentlyPitching(speedY)
    end
    table.insert(self._main._needAddBallId, self._id)

    if self._main._guideBall then
        T.removeOrder(self._main._guideBall, self)
    end

    if self._main.removeGuide then
        self._main:removeGuide(self._id, self)
    end
    self:playPitchingEffect()
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M