--[[Copyright (c) 2012-2020 baby-bus.com

http://www.baby-bus.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]
--[[!--

篮球类

]]
----------------------
-- 类
----------------------
local M    = classSprite("Basket")
M.TAG    = "Basket"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return:
-   object              对象实例

]]
function M:ctor(params)
    --J.assert(M.TAG, params.images ~= nil and #params.images == 4,"images size must be 4")
    -- [超类调用]
    M.super.ctor(self, params)
    -- mainLayer
    self._main        = params.main
    --初始坐标
    self._pos        = params.pos
    self._world        = params.world
    --进一颗球的得分
    self._score        = 1
    --进球数量
    self._count        = 0
end




----------------------
-- 结点渲染
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    self:loadDisplay()
    --加载两个碰撞点
    self:loadNodeMask()
end


function M:onEnterTransitionFinish()
    --加载刚体
    self:loadBody()
    self:loadBasketMask()
    --得分计算
    self:statisticalScore()
end

--统计得分
function M:statisticalScore()
    --加载得分点
    local mask    = U.loadNodeMask({
        contentSize = CCSize(5, 5),
        color    = ccc4(0, 0, 0, 255),
    }):to(self._net):unbindTouch()
    :opacity(255):p(self._net:cw() / 2, 50):hide()
    mask:cycle({
        { "delay", 1 / 60 },
        { "fn", function()
            for i, v in pairs(self._main._balls) do
                if v._canScore then
                    local dis = ccpDistance(mask:worldpoint(), v:worldpoint())
                    if dis <= 30 then
                        v._canScore = false
                        self._main._score = self._main._score + self._score
                        self:checkDoubleBuff()
                        if self._main._progressBar then
                            self._main._progressBar:updateProgressbar(self._main._score / 50)
                        end
                        self:basketMaskShake(v:px() - mask:worldpoint().x, v)
                        if self._main.applause then
                            --鼓掌
                            self._main:applause()
                        end
                        if self._big then
                            soundEffect:playEffectsfx26222022()
                        elseif v._touchBasketOver then
                            soundEffect:playEffectsfx26222023()
                        else
                            soundEffect:playEffectsfx26222003()
                        end
                        v:scaleAction(0.85, 0.15, true)
                        for i, v in pairs(self._main._basket) do
                            if v ~= self then
                                v._count = 0
                            end
                        end
                    elseif v:py() < 345 then
                        self._count = 0
                        v._canScore = false
                        v:scaleAction(0.85, 0.15, true)
                    end
                end
            end
        end },
    })
end

-- 图片渲染
function M:loadDisplay()
    self:display("ball/basket.png")
    local basketshadow = D.img("ball/basketshadow.png")
    :to(self, -1):p(97, 22)
    local net = D.img("ball/shake1/10001.png")
    :to(self._main._emptyNode, 4):p(cc.pAdd(cc.p(0, 13), self._pos)):anchor(cc.p(0.5, 1))
    local emptyNode = U.loadNode():to(net, -1):p(net:cw() / 2, 80):scaleY(0.5):scaleX(1.35)
    local light = D.img("ball/light1.png")
    :anchor(cc.p(0.5, 0.5)):to(emptyNode):opacity(0)
    light:cycle({
        { "rotateBy", 1, 90 }
    })
    self._light = light
    self._net = net
    self._net._pos = self._net:point()
    local dazzle = D.img("ball/light/10001.png"):to(net, -1)
    :p(net:cw() / 2, 75):anchor(cc.p(0.5, 0)):opacity(0)
    self._dazzle = dazzle
end

--加载两个碰撞点
function M:loadNodeMask()
    local node1 = D.img("ball/bg.png"):to(self._net):scale(0.1)
    :p(self._net:cw() / 2 - 45, 72):opacity(255)
    node1._basket = true
    local node2 = D.img("ball/bg.png"):to(self._net):scale(0.1)
    :p(self._net:cw() / 2 + 45, 72):opacity(255)
    node2._basket = true
    self._node1 = node1
    self._node2 = node2
end

function M:loadBasketMask()
    local pos = self._net:point()
    local basketMask = D.img("ball/shake1/20001.png")
    :to(self._main._emptyNode, 10):p(pos):anchor(cc.p(0.5, 1))
    self._mask = basketMask
    self._mask._pos = self._mask:point()
end

-- 加载刚体
function M:loadBody()
    local points = {
        {
            ccp(-10, 10), ccp(0, 10),
            ccp(-5, -15), ccp(5, -15),
        },
        {
            ccp(10, 10), ccp(0, 10),
            ccp(5, -15), ccp(-5, -15),
        }
    }
    local nodes = {
        self._node1, self._node2
    }
    for i = 1, 2 do
        -- 定义刚体信息
        local bodyInfo        = {}
        -- 类型：静态(b2_staticBody),动态(b2_dynamicBody),平台(b2_kinematicBody)
        bodyInfo.type            = b2_staticBody
        -- 位置
        bodyInfo.pos                = nodes[i]:worldpoint()
        -- 用户数据：存储用户的数据，可以是任何类型的数据。一般要求存储的数据的类型是一致的
        bodyInfo.userData        = nodes[i]
        -- 密度：影响刚体的惯性、重量等
        bodyInfo.density        = 2
        --摩擦力
        bodyInfo.friction        = 0.5
        --弹性系数
        bodyInfo.restitution    = 0.6
        -- 形状
        bodyInfo.shape        = {
            {
                shapeType = EShape.POLYGON,
                points    = points[i]
            },
        }
        -- bodyInfo.groupIndex        = -1
        -- 类别标志位
        bodyInfo.categoryBits    = 4
        -- 遮罩标志位 
        bodyInfo.maskBits        = 6
        if i == 1 then
            self._body1 = bb.si.Box2D.createBody(bodyInfo)
            self._body1._followPic = true
        else
            self._body2 = bb.si.Box2D.createBody(bodyInfo)
            self._body2._followPic = true
        end
    end
end

-- 加载变大后的刚体
function M:loadBigBody()
    local points = {
        {
            ccp(-70, -25), ccp(-55, -25),
            ccp(-60, -50), ccp(-50, -50),
        },
        {
            ccp(70, -25), ccp(55, -25),
            ccp(60, -50), ccp(50, -50),
        }
    }
    for i = 1, 2 do
        -- 定义刚体信息
        local bodyInfo        = {}
        -- 类型：静态(b2_staticBody),动态(b2_dynamicBody),平台(b2_kinematicBody)
        bodyInfo.type            = b2_staticBody
        -- 位置
        bodyInfo.pos            = self._net:worldpoint()
        -- 用户数据：存储用户的数据，可以是任何类型的数据。一般要求存储的数据的类型是一致的
        bodyInfo.userData        = self._net
        -- 密度：影响刚体的惯性、重量等
        bodyInfo.density        = 2
        --摩擦力
        bodyInfo.friction        = 0
        --弹性系数
        bodyInfo.restitution    = 0.6
        -- 形状
        bodyInfo.shape        = {
            {
                shapeType = EShape.POLYGON,
                points    = points[i]
            },
        }
        -- bodyInfo.groupIndex        = -1
        -- 类别标志位
        bodyInfo.categoryBits    = 4
        -- 遮罩标志位 
        bodyInfo.maskBits        = 6
        if i == 1 then
            self._body1 = bb.si.Box2D.createBody(bodyInfo)
            self._body1._followPic = true
        else
            self._body2 = bb.si.Box2D.createBody(bodyInfo)
            self._body2._followPic = true
        end
    end
end

function M:destoryBody()
    if self._body1 then
        self._world:DestroyBody(self._body1)
    end
    if self._body2 then
        self._world:DestroyBody(self._body2)
    end
end

--球网等分效果抖动
function M:basketMaskShake(disX, ball)
    if self._net._shakeAction then
        self._net:stopAction(self._net._shakeAction)
        self._net._shakeAction = nil
    end
    if self._mask._shakeAction then
        self._mask:stopAction(self._mask._shakeAction)
        self._net._shakeAction = nil
    end
    if math.abs(disX) < 10 then
        if ball._body:GetLinearVelocity().x < 0 then
            self._net:flipX(true)
            self._mask:flipX(true)
        else
            self._net:flipX(false)
            self._mask:flipX(false)
        end
        local action = A.line({
            { "image", "ball/shake1/1000", 9, 0.1 },
            { "fn", function()
                self._net._shakeAction = nil
            end },
        }):at(self._net)
        self._net._shakeAction = action
        local action = A.line({
            { "image", "ball/shake1/2000", 9, 0.1 },
            { "fn", function()
                self._mask._shakeAction = nil
            end },
        }):at(self._mask)
        self._mask._shakeAction = action
    else
        if ball._body:GetLinearVelocity().x < 0 then
            self._net:flipX(true)
            self._mask:flipX(true)
        else
            self._net:flipX(false)
            self._mask:flipX(false)
        end
        local action = A.line({
            { "image", "ball/shake2/1000", 8, 0.1 },
            { "fn", function()
                self._net._shakeAction = nil
            end },
        }):at(self._net)
        self._net._shakeAction = action
        local action = A.line({
            { "image", "ball/shake2/2000", 8, 0.1 },
            { "fn", function()
                self._mask._shakeAction = nil
            end },
        }):at(self._mask)
        self._mask._shakeAction = action
    end
end

--变大
function M:bigger()
    soundEffect:playEffectsfx26222016()
    self._big = true
    local function nodeBigger(node)
        local size = node:scale()
        node:line({
            { "scaleTo", 0.2, size * 1.35 },
            { "scaleTo", 0.1, size * 1.15 },
            { "scaleTo", 0.1, size * 1.25 },
        })
    end
    nodeBigger(self._net)
    nodeBigger(self._mask)
    self._light:line({
        { "fadeTo", 1, 255 },
        { "fn", function()
            self._lightEffect = soundEffect:playEffectsfx26222017(true)
        end },
    })
    self._dazzle:stopAllActions()
    self._dazzle:line({
        { "fadeTo", 1, 255 },
    })
    self._dazzle:cycle({
        { "image", "ball/light/1000", 12, 0.1 }
    })
    self._score = 2

    self:line({
        { "delay", 8 },
        { "fn", function()
            self:smaller()
        end },
    })
end

--开始移动
function M:satrtMove(time)
    --篮筐开启移动
    soundEffect:playEffectsfx26222011()
    self._body1._followPic = true
    self._body2._followPic = true
    time = ifnil(time, 1)
    if self._move then return end
    soundEffect:playEffectsfx26222012()
    local action = A.cycle({
        { "moveBy", time, cc.p(-50, 0) },
        { "fn", function()
            soundEffect:playEffectsfx26222014()
        end },
        { "moveBy", 2 * time, cc.p(100, 0) },
        { "fn", function()
            soundEffect:playEffectsfx26222013()
        end },
        { "moveBy", time, cc.p(-50, 0) },
    }):at(self._net)
    self._moveAction1 = action
    local action = A.cycle({
        { "moveBy", time, cc.p(-50, 0) },
        { "moveBy", 2 * time, cc.p(100, 0) },
        { "moveBy", time, cc.p(-50, 0) },
    }):at(self._mask)
    self._moveAction2 = action
    self._move = true
end

--归位
function M:place()
    self._move = false
    self._net:stopAction(self._moveAction1)
    self._mask:stopAction(self._moveAction2)
    self._net:line({
        { "moveTo", 0.2, self._net._pos },
        { "fn", function()
            self._body1._followPic = true
            self._body2._followPic = true
        end },
    })
    self._mask:line({
        { "moveTo", 0.2, self._mask._pos },
    })
end

--检测双倍buff
function M:checkDoubleBuff()
    if self._big then return end
    if not self._main._progressBar then return end
    if self._main._gamerOver then return end
    if self._main._progressBar._starNum >= 1 then
        self._count        = self._count + 1
        if self._count > 2 then
            self._count = 0
            --变大奖励
            self:bigger()
        end
    end
end

--变小
function M:smaller()
    if not self._big then return end

    local function nodeSmaller(node)
        local size = node:scale()
        node:line({
            { "scaleTo", 0.1, 1.35 },
            { "scaleTo", 0.2, 1 },
        })
    end
    nodeSmaller(self._net)
    nodeSmaller(self._mask)
    self._big = false
    self._light:stopAllActions()
    self._light:line({
        { "fadeTo", 0.5, 0 },
    })
    self._dazzle:line({
        { "fadeTo", 0.5, 0 },
    })
    self._dazzle:line({
        { "fadeTo", 0.5, 0 },
    })
    self._score = 1

    if self._lightEffect then
        sound.stopSound(self._lightEffect)
        self._lightEffect = nil
    end
    soundEffect:playEffectsfx26222018()
end

function M:finishMove()
    self._net:stopAction(self._moveAction1)
    self._mask:stopAction(self._moveAction2)
    self:smaller()
    self._move = false
end
----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()
    SC.close(self._ropeScheduler)
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M