--[[
Copyright (c) 2012-2020 baby-bus.com

http://www.baby-bus.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类
----------------------
local M    = classArmatureTouch("FatKid")
M.TAG    = "FatKid"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 准备点 1:左,2:右
READY_POS = { ccp(250 + X_OFFSET, 200 + Y_OFFSET), ccp(700 + X_OFFSET, 200 + Y_OFFSET) }

GIRL = 1    -- 女孩
BOY = 2    -- 男孩
BLANK = 1    -- 黑皮
WHITE = 2    -- 白皮

-- 进入的时间，为保证动作节奏一致
INTOTIME = 1.34

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return:
-   object              对象实例

]]
function M:ctor(params)
    --J.assert(M.TAG, params.images ~= nil and #params.images == 4,"images size must be 4")
    -- [超类调用]
    M.super.ctor(self, params)
    -- mainLayer
    self._main                          = params.main
    -- 当前跳跃动作
    self._jumpName                      = "ph_ts"
    -- 小朋友类型
    self._type                          = params.type
    -- 小朋友的索引
    self._index                         = ifnil(params.index, 0)
    self._sex                           = params.sex

    -- 偏移量
    self._offsetX                       = params.offsetX
    self._pos                           = params.pos
end


----------------------
-- 结点渲染
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 换装
    self:changeClothesWithType()
end

-- 回调
function M:movementHandler(__evtType, __moveId)
    -- if __evtType == 1 then
        if self._type == 1 then
            -- 胖小孩回调处理方法
            self:fatChildrenMovement(__evtType, __moveId)
        end
    -- end
end

-- 根据小朋友类型换装
function M:changeClothesWithType()
    if self._type == 1 then
        self:fatClothesFactry()
    end
end

-- 随机换装(头发+眼镜)
function M:randomChangeClothes()
    local sex = self._sex
    if self._type == 2 then
        local type = sex == 1 and { "", 2, 3, 4 } or { 1, 5, 6, 7, 8 }
        local num = RD.number(1, 2)
        local id = T.random(type)
        if id == 8 then
            self:getBone("zch_tou"):change("zch_tou2")
        else
            if id == 3 then
                self:getBone("zch_htf"):show()
            elseif id == 2 then
                self:getBone("zch_lh"):show()
            elseif id == "" then
                self:getBone("zch_zbz"):show()
                self:getBone("zch_ybz"):show()
            end
            self:getBone("zch_tf"):change("zch_tf" .. id)
        end
        if num == 1 then
            self:getBone("zch_yanj"):hide()
        end
    elseif self._type == 1 then

    end
end

-- 胖小孩回调处理方法
function M:fatChildrenMovement(__evtType, __moveId)
    if __evtType == 1 then
        if __moveId == "ph_hh" then
            if self._main._gamerOver then
                self:play("ph_hh", 1.5)
            else
                -- 呼吸动作
                self:breathEvent()
            end
        elseif __moveId == "ph_pao" then
            -- 衔接动作-跑步
            self:connectAction("ph_pao1", "ph_pao2", 0.71)
        elseif __moveId == "ph_zou" then
            -- 衔接动作-走
            self:play("ph_zou1")
        elseif __moveId == "ph_gz" then
            -- 呼吸动作
            -- self:play("ph_dj1")
            self:breathEvent()
        elseif __moveId == "ph_dj1" then
            self:play("ph_zchx")
        elseif __moveId == "ph_dj2" then
            self:play("ph_zchx")    
        end
    elseif __evtType == 2 then
        if __moveId == "ph_zchx" then
            -- 呼吸动作
            self:breathEvent()
        end
    end
end

-- 呼吸动作
-- params
-- isTired 是不是累了
function M:breathEvent(isTired)
    isTired = ifnil(isTired, true)
    local breath = nil
    if self._type == 1 then
        -- 胖孩呼吸
        breath = self:fatBreath(isTired)
    end
    self:play(breath)
end

-- 胖孩呼吸
function M:fatBreath(isTired)
    return "ph_zchx"
end

--鼓掌
function M:cheer()
   if self._type == 1 then
        -- 胖孩呼吸
        self:play("ph_gz")
    end 
end

-- 跑起来方法
function M:runEvent()
    local run = nil
    if self._type == 1 then
        run = "ph_pao"
    end
    self:play(run)
end

-- 切换左腿骨骼
-- params
-- isChnage true：切换，false：恢复正常
function M:changeLeg(isChnage)
    isChnage = ifnil(isChnage, true)
    local leg = ifnil(leg, "")
    if self._sex == 2 then
        if isChnage then
            self:getBone("zch_zst"):change("zch_zst1_1")
            self:getBone("zch_zxt"):change("zch_zxt1_1")
            self:getBone("zch_zj"):change("zch_zj1_1")
        else
            self:getBone("zch_zst"):change("zch_zst1")
            self:getBone("zch_zst"):change("zch_zst1")
            self:getBone("zch_zj"):change("zch_zj1")
        end
    end
end

-- 走起来方法
function M:walkEvent()
    local run = nil
    if self._type == 1 then
        run = "ph_zou"
    elseif self._type == 2 then
        -- 换脚
        self:changeLeg()
        run = "zch_zou"
    end
    self:play(run)
end

-- 走路音效
function M:walkEffect()
    -- 停止走路音效
    self:stopWalkEffect()
    self._walkAction = A.cycle({
        { "fn", function()
            self._walkEffect = soundEffect:playEffectSfx26200004()
        end },
        { "delay", 0.4 },
    }):at(self)
end

-- 停止走路音效
function M:stopWalkEffect()
    if self._walkAction then
        self._walkAction = CommonFun:stopAction(self, self._walkAction)
        sound.stopSound(self._walkEffect)
    end
end

-- 激励方法
function M:maffickEvent()
    local run = nil
    if self._type == 1 then
        self:maffickClothesTbale()
        run = "ph_hh"
    elseif self._type == 2 then
        if self._sex == 2 then
            self:getBone("zch_yyz"):change("zch_yyz1_1")
            self:getBone("zch_zyz"):change("zch_zyz1_1")
            self:line({
                { "delay", 1.5 },
                { "fn", function()
                    self:getBone("zch_yyz"):change("zch_yyz1")
                    self:getBone("zch_zyz"):change("zch_zyz1")
                end }
            })
        end
        run = "zch_hh"
    end
    self:play(run)
end

-- 激励换装
function M:maffickClothesTbale()
    self:getBone("ph_zui"):change("ph_zui_2")
    self:getBone("ph_yyk"):show()
    self:getBone("ph_zyk"):show()
    self:getBone("ph_yyb"):show()
    self:getBone("ph_zyb"):show()
    if self._clothesType ~= 2 then
        self:getBone("ph_yyz"):change("ph_1k")
        self:getBone("ph_zyz"):change("ph_1k")
        self:getBone("ph_zyk"):change("ph_zyk_1")
        self:getBone("ph_yyk"):change("ph_yyk_1")
        self:getBone("ph_yyb"):change("ph_1k")
        self:getBone("ph_zyb"):change("ph_1k")
        self:line({
            { "delay", 1.5 },
            { "fn", function()
                self:eyeClothes()
            end }
        })
    end
end

-- 欢呼
-- params
-- isLast 最后一个
-- isEnd  是否结束了
function M:maffick(isLast, isEnd)
    isEnd = ifnil(isEnd, false)
    -- 延迟欢呼时间集合
    local delayTimes = { 0, 0.2, 0.4, 0.6 }
    local delayTime = T.random(delayTimes)
    self:line({
        { "delay", delayTime },
        { "fn", function()
            -- 欢呼
            self:maffickEvent()
            -- 最后一个欢呼小朋友，并且游戏没结束执行
            if isLast and not isEnd then
                -- 添加小朋友
                self._main:stageUpEvent()
            end
        end }
    })
end

-- 衔接动作
-- param
-- startAction 开始动作
-- 结束动作
-- 循环时间
function M:connectAction(startAction, endAction, time)
    -- 开始跑步循环
    self:play(startAction)
    self:line({
        { "delay", time },
        { "fn", function()
            -- 结束循环
            self:stop()
            -- 动作结束衔接
            self:play(endAction)
        end }
    })
end


-- 切换肤色方法
-- param
-- skirType 需要的皮肤类型 （1：黑人 2：白人）
-- sex 性别               (1：女孩 2：男孩)
function M:changeSkir(skirType, sex)
    -- 待替换的身体骨骼表
    local boneTable = {
        "ph_tou", "ph_ye", "ph_ze", "ph_boz", "ph_ysb", "ph_zsb", "ph_yxb", "ph_zxb", "ph_ys", "ph_zs",
        "ph_yst", "ph_zst", "ph_yxt", "ph_zxt"
    }
    -- 待隐藏的装饰表
    local hideClothesTable = {
        "ph_zhdj", "ph_yhdj", "ph_stf"
    }
    -- 根据男女的换装列表(1：女孩 2：男孩)
    local clothesBySexTable = {
        { "ph_zj1", "ph_zj" }, { "ph_yj1", "ph_yj" }, { "ph_ywz1", "ph_ywz" }, { "ph_zwz1", "ph_zwz" }, { "ph_st1", "ph_st" }
    }
    if skirType == BLANK then
        for k, v in pairs(boneTable) do
            self:getBone(v):change(v .. "1")
        end
    end
    for k, v in pairs(hideClothesTable) do
        self:getBone(v):hide()
    end
    for k, v in pairs(clothesBySexTable) do
        self:getBone(v[2]):change(v[sex])
    end
    self:eyeClothes()
    self._glasses = {true, false }
    if T.random(self._glasses) then
        self:getBone("ph_yanj"):hide()
    end
    self._sex = sex
    self._skir = skirType
end

-- 眼部换装
function M:eyeClothes()
    local skirType = self._skir
    if skirType == 1 then
        if self._sex == 1 then
            self:getBone("ph_yyk"):hide()
            self:getBone("ph_zyk"):hide()
            self:getBone("ph_yyb"):hide()
            self:getBone("ph_zyb"):hide()
            self:getBone("ph_yyz"):change("ph_yyz1")
            self:getBone("ph_zyz"):change("ph_zyz1")
        else
            self:getBone("ph_yyk"):change("ph_yyk2")
            self:getBone("ph_zyk"):change("ph_zyk2")
            self:getBone("ph_yyb"):change("ph_yyb2")
            self:getBone("ph_zyb"):change("ph_zyb2")
            self:getBone("ph_yyz"):change("ph_yyz2")
            self:getBone("ph_zyz"):change("ph_zyz2")
            self:getBone("ph_zgg"):hide()
            self:getBone("ph_ygg"):hide()
        end
    else
        self:getBone("ph_yyk"):change("ph_yyk")
        self:getBone("ph_zyk"):change("ph_zyk")
        self:getBone("ph_yyb"):change("ph_yyb")
        self:getBone("ph_zyb"):change("ph_zyb")
        self:getBone("ph_yyz"):change("ph_yyz")
        self:getBone("ph_zyz"):change("ph_zyz")
    end
end

-- 创建一个所有部件的列表
function M:getClothesTbale()
    -- 换装列表
    local clothesTable = {
        { "ph_tf", { "", 2, 3, 4, 8 }, { 1, 5, 6, 7 } }, { "ph_ymm", { "", 1, 2 } }, { "ph_zmm", { "", 1, 2 } },
        { "ph_zui", { "", 1, 2 } }, { "ph_bz", { "", 1, 2 } }
    }
    return clothesTable
end

-- 换装工厂方法
-- param
function M:fatClothesFactry(clothesType)
    self:changeSkir(self._main:getSkirAndSex())
    for k, v in pairs(self:getClothesTbale()) do
        -- 头发
        if k == 1 and self._sex == 1 then
            local hairId = T.random(v[3])
            if hairId == 1 then
                self:getBone("ph_zhdj"):show()
                self:getBone("ph_yhdj"):show()
                self:getBone("ph_stf"):show()
            elseif hairId == 6 then
                self:getBone("ph_htf"):change("ph_htf6")
            elseif hairId == 7 then
                self:getBone("ph_stf"):show()
            end
            self:getBone(v[1]):change(v[1] .. hairId)
            break
        elseif k == 1 and self._sex == 2 then
            local hairId = T.random(v[2])
            if hairId == 8 and self._skir == 1 then
                self:getBone("ph_tou"):change("ph_tou2")
                self:getBone("ph_tf"):hide()
            else
                self:getBone(v[1]):change(v[1] .. T.random({ "", 2, 3, 4,}))
            end
            break
        end
        self:getBone(v[1]):change(v[1] .. T.random(v[2]))
    end
end

-- 待机动作
function M:standAction()
    -- 如果不是待机状态结束返回true
    if not self._stand then return true end
    local stands = {}
    if self._type == 1 then
        stands = { "ph_dj1", "ph_dj2" }
    elseif self._type == 2 then
        stands = { "zch_dj1", "zch_dj2" }
    end
    local stand = T.random(stands)
    self:play(stand)
    return false
end


-- 进场
function M:enter()
    self:line({
        { "delay", 0.01 },
        { "fn", handler(self, self.walkEvent) },
        { "moveTo", self._offsetX / 200, self._pos },
        { "fn", function()
            self:play("ph_zou2")
            self:stopWalkEffect()
            -- 呼吸动作
            self:play("ph_zchx")
        end }
    })
end

----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()

end


----------------------
-- 点击
----------------------
--[[--

判断是否点击到内容区域

### Example:

### Parameters:
-   string **event**    事件名称
-   number **x**        点击位置x(相对于屏幕)
-   number **y**        点击位置y(相对于屏幕)

]]
function M:onTouchHandled(event, x, y)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M