--[[
Copyright (c) 2012-2020 baby-bus.com

http://www.baby-bus.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类
----------------------
local M    = classArmatureTouch("Kid")
M.TAG    = "Kid"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    --J.assert(M.TAG, params.images ~= nil and #params.images == 4,"images size must be 4")
    -- [超类调用]
    M.super.ctor(self, params)
    -- mainLayer
    self._main                  =    params.main
    -- 位置
    self._pos                   =    params.pos
    -- 编号
    self._id                    =    params.id
    -- 类型
    self._type                  =    params.type
    -- 头发编号
    self._hairId                =    params.hairId
    -- 偏移量
    self._offsetX               =    params.offsetX
    -- 人物大小
    self._scale                =    params.scale
    -- 进口顺序下标
    self._order                =    params.order
    -- 是否是领操人
    self._isLeader              =    params.isLeader
    -- 换装类型
    self._number                =    ifnil(params.number, 1)
    -- 节拍数
    self._beatNum               =    0    
    self._breathID              =    params.breathID
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 换装
    self:changeClothes()
end

-- 回调
function M:movementHandler(__evtType, __moveId)
    if __evtType == 1 then
        if __moveId == self._type .. "_lqgz" then
            self._cheer = false
            self:play("gh_lqhx", RD.number(8, 12) / 10)
        elseif __moveId == self._type .. "_lqgz1" then
            self._cheer = false
            self:play("gh_lqhx1", RD.number(8, 12) / 10)
        elseif __moveId == self._type .. "_lqhx" then
            -- 呼吸动作
            self:play("gh_lqhx", RD.number(8, 12) / 10)
        elseif __moveId == "gh_lqhh"..self._breathID then
            if self._main._gamerOver then
                self:cheer_1()
            else
                -- 呼吸动作
                self:await()
            end
        end
    elseif __evtType == 2 then
        -- if __moveId == self._type .. "_lqhx1" then
        --     -- 呼吸动作
        --     self:await(true)
        -- end
    end
end

-- 换装
function M:changeClothes()
    if self._type == "gh" then
        self:changeClothes_1()
    elseif self._type == "zch" then
        self:changeClothes_2()
    end
    self:randomChangeClothes()
end

-- 高个子换装
function M:changeClothes_1()
    self:hideSomeBones()
    self._isBoy = true
    if self._number == 2 then
        self._isBoy = false
        self:getBone("gh_tf"):change("gh_tf1")
        self:getBone("gh_ymm"):change("gh_ymm1")
        self:getBone("gh_zmm"):change("gh_zmm1")
        self:getBone("gh_yyx"):change("gh_yyx1")
        self:getBone("gh_zyy"):change("gh_zyy1")
        self:getBone("gh_bz"):change("gh_bz1")
        self:getBone("gh_st"):change("gh_st1")
        self:getBone("gh_yw"):change("gh_yw1")
        self:getBone("gh_zw"):change("gh_zw1")
        self:getBone("gh_yj"):change("gh_yj1")
        self:getBone("gh_zj"):change("gh_zj1")
        self:getBone("gh_zui"):change("gh_zui1")
    elseif self._number == 3 then
        self._isBoy = false
        self:getBone("gh_tf"):change("gh_tf2")
        self:getBone("gh_ymm"):change("gh_ymm1")
        self:getBone("gh_zmm"):change("gh_zmm1")
        self:getBone("gh_yyx"):change("gh_yyx2")
        self:getBone("gh_zyy"):change("gh_zyy2")
        self:getBone("gh_yyz"):change("gh_yyz2")
        self:getBone("gh_zyz"):change("gh_zyz2")
        self:getBone("gh_bz"):change("gh_bz1")
        self:getBone("gh_st"):change("gh_st1")
        self:getBone("gh_yw"):change("gh_yw1")
        self:getBone("gh_zw"):change("gh_zw1")
        self:getBone("gh_yj"):change("gh_yj1")
        self:getBone("gh_zj"):change("gh_zj1")
        self:getBone("gh_zui"):change("gh_zui2")
    elseif self._number == 4 then
        self:getBone("gh_ymm"):change("gh_ymm4")
        self:getBone("gh_zmm"):change("gh_zmm4")
        self:getBone("gh_yyx"):change("gh_yyx4")
        self:getBone("gh_zyy"):change("gh_zyy4")
        self:getBone("gh_yyz"):change("gh_yyz4")
        self:getBone("gh_zyz"):change("gh_zyz4")
        self:getBone("gh_bz"):change("gh_bz4")
        self:getBone("gh_zui"):change("gh_zui4")
        self:getBone("gh_ygg"):change("gh_ygg4")
        self:getBone("gh_zgg"):change("gh_zgg4")
        self:getBone("gh_yyb"):change("gh_yyb4")
        self:getBone("gh_zyb"):change("gh_zyb4")
        self:changeSkinColor()
    end
end

-- 隐藏一些骨骼
function M:hideSomeBones()
    if self._hairId ~= 0 then
        self:getBone("gh_dm"):hide()
    end
    if self._hairId ~= 1 and self._hairId ~= 4 then
        self:getBone("gh_zbz"):hide()
        self:getBone("gh_ybz"):hide()
    end
    if self._hairId ~= 2 then
        self:getBone("gh_htf"):hide()
    end
end

-- 正常小孩换装
function M:changeClothes_2()
    self:getBone("zch_htf"):hide()
    if self._number == 2 then  -- 白皮肤男孩
        self._isBoy = true
        self:getBone("zch_tf"):change("zch_tf1")
        self:getBone("zch_ymm"):change("zch_ymm1")
        self:getBone("zch_zmm"):change("zch_zmm1")
        self:getBone("zch_yyz"):change("zch_yyz1")
        self:getBone("zch_zyz"):change("zch_zyz1")
        self:getBone("zch_bz"):change("zch_bz1")
        self:getBone("zch_st"):change("zch_st1")
        self:getBone("zch_yj"):change("zch_yj1")
        self:getBone("zch_zj"):change("zch_zj1")
        self:getBone("zch_yxt"):change("zch_yxt1")
        self:getBone("zch_zxt"):change("zch_zxt1")
        self:getBone("zch_zbz"):hide()
        self:getBone("zch_ybz"):hide()
        self:getBone("zch_lh"):hide()
        self:changeSkinColor()
    end
end

-- 随机换装(头发+眼镜)
function M:randomChangeClothes()
    local num = RD.number(1, 2)
    if self._type == "zch" then
        if self._hairId == 8 then
            self:getBone("zch_tou"):change("zch_tou2")
        else
            if self._hairId == 3 then
                self:getBone("zch_htf"):show()
            end
            if self._hairId ~= 0 then
                self:getBone("zch_tf"):change("zch_tf" .. self._hairId)
            end
        end
        if num == 1 then
            self:getBone("zch_yanj"):hide()
        end
    elseif self._type == "gh" then
        if self._hairId == 4 then
            self:getBone("gh_zbz"):change("gh_zbz4")
            self:getBone("gh_ybz"):change("gh_ybz4")
        end
        if self._hairId ~= 0 then
            self:getBone("gh_tf"):change("gh_tf" .. self._hairId)
        end
        if num == 1 then
            self:getBone("gh_yanj1"):hide()
            self:getBone("gh_yanj"):hide()
        end
    end
end

-- 改变肤色
function M:changeSkinColor()
    if self._type == "gh" then
        self:changeSkinColor_1()
    elseif self._type == "zch" then
        self:changeSkinColor_2()
    end
end

-- 改变高个子肤色
function M:changeSkinColor_1()
    self:getBone("gh_tou"):change("gh_tou4")
    self:getBone("gh_ye"):change("gh_ye4")
    self:getBone("gh_ze"):change("gh_ze4")
    self:getBone("gh_boz"):change("gh_boz4")
    self:getBone("gh_ysb"):change("gh_ysb4")
    self:getBone("gh_zsb"):change("gh_zsb4")
    self:getBone("gh_yxb"):change("gh_yxb4")
    self:getBone("gh_zxb"):change("gh_zxb4")
    self:getBone("gh_ys"):change("gh_ys4")
    self:getBone("gh_zs"):change("gh_zs4")
    self:getBone("gh_yst"):change("gh_yst4")
    self:getBone("gh_zst"):change("gh_zst4")
    self:getBone("gh_yxt"):change("gh_yxt4")
    self:getBone("gh_zxt"):change("gh_zxt4")
end

-- 改变正常小孩肤色
function M:changeSkinColor_2()
    self:getBone("zch_tou"):change("zch_tou1")
    self:getBone("zch_ye"):change("zch_ye1")
    self:getBone("zch_ze"):change("zch_ze1")
    self:getBone("zch_ysh"):change("zch_ysh1")
    self:getBone("zch_zsh"):change("zch_zsh1")
    self:getBone("zch_boz"):change("zch_boz1")
    self:getBone("zch_ysb"):change("zch_ysb1")
    self:getBone("zch_zsb"):change("zch_zsb1")
    self:getBone("zch_yxb"):change("zch_yxb1")
    self:getBone("zch_zxb"):change("zch_zxb1")
    self:getBone("zch_ys"):change("zch_ys1")
    self:getBone("zch_zs"):change("zch_zs1")
    self:getBone("zch_yst"):change("zch_yst1")
    self:getBone("zch_zst"):change("zch_zst1")
    self:getBone("zch_scys"):change("zch_scys1")
    self:getBone("zch_ys_1"):change("zch_ys1_1")
    self:getBone("zch_ys_2"):change("zch_ys1_2")
    self:getBone("zch_zs_5"):change("zch_zs1_5")
    self:getBone("zch_zs_4"):change("zch_zs1_4")
    self:getBone("zch_zs_3"):change("zch_zs1_3")
    self:getBone("zch_zs_2"):change("zch_zs1_2")
    self:getBone("zch_zs_1"):change("zch_zs1_1")
end

-- 改变正常小孩左腿骨骼
function M:changeFootBone()
    if self._type == "zch" then
        if self._number == 1 then
            self:getBone("zch_zst"):change("zch_zst_1")
            self:getBone("zch_zxt"):change("zch_zxt_1")
            self:getBone("zch_zj"):change("zch_zj_1")
        elseif self._number == 2 then
            self:getBone("zch_zst"):change("zch_zst1_1")
            self:getBone("zch_zxt"):change("zch_zxt1_1")
            self:getBone("zch_zj"):change("zch_zj1_1")
        end
    end
end

-- 恢复正常小孩左腿骨骼
function M:restoreFootBone()
    if self._type == "zch" then
        if self._number == 1 then
            self:getBone("zch_zst"):change("zch_zst")
            self:getBone("zch_zxt"):change("zch_zxt")
            self:getBone("zch_zj"):change("zch_zj")
        elseif self._number == 2 then
            self:getBone("zch_zst"):change("zch_zst1")
            self:getBone("zch_zxt"):change("zch_zxt1")
            self:getBone("zch_zj"):change("zch_zj1")
        end
    end
end

-- 眼睛换装
function M:changeEyeBone(module)
    module = module or 1
    if self._type == "zch" and self._number == 1 then
        if module == 1 then
            self:getBone("zch_yyz"):change("zch_yyz1")
            self:getBone("zch_zyz"):change("zch_zyz1")
        elseif module == 2 then
            self:getBone("zch_yyz_1"):change("zch_yyz1_1")
            self:getBone("zch_zyz_1"):change("zch_zyz1_1")
        elseif module == 3 then
            self:getBone("zch_yyz_2"):change("zch_yyz1_2")
            self:getBone("zch_zyz_2"):change("zch_zyz1_2")
        elseif module == 4 then
            self:getBone("zch_zyz_3"):change("zch_zyz1_3")
        end
    end
end

-- 改变正常小孩手臂骨骼
function M:changeHandBone()
    if self._type == "zch" and self._number == 2 then
        self:getBone("zch_xczsb"):change("zch_xczsb1")
        self:getBone("zch_scysb"):change("zch_scysb1")
        self:getBone("zch_scyxb"):change("zch_scyxb1")
        self:getBone("zch_scys"):change("zch_scys1")
    end
end

-- 复原正常小孩手臂骨骼
function M:restoreHandBone()
    if self._type == "zch" and self._number == 2 then
        self:getBone("zch_xczsb"):change("zch_xczsb")
        self:getBone("zch_scysb"):change("zch_scysb")
        self:getBone("zch_scyxb"):change("zch_scyxb")
        self:getBone("zch_scys"):change("zch_scys")
    end
end


-- 走路音效
function M:walkEffect()
    if self._walkAction then return end
    self._walkAction = A.cycle({
        { "fn", function()
            self._walkEffect = soundEffect:playEffectSfx26200004()
        end },
        { "delay", 0.4 },
    })
    self:runAction(self._walkAction)
end

-- 停止走路音效
function M:stopWalkEffect()
    if self._walkAction then
        self:stopAction(self._walkAction)
        self._walkAction = nil
        sound.stopSound(self._walkEffect)
    end
end

-- 踏步
function M:markTime()
    self:changeFootBone()
    self:play(self._type .. "_ydtb")
end

-- 走路
function M:walk()
    self:changeFootBone()
    if self._type == "zch" then
        self:play(self._type .. "_zou_1")
    elseif self._type == "gh" then
        if self._number == 4 then
            self:play(self._type .. "_zl2")
        else
            self:play(self._type .. "_zl1")
        end
    end
end

-- 准备
function M:prepare()
    self:play(self._type .. "_yb", 0.5, { loop = 0 })
end

-- 待机
function M:await(flag)
    self:changeEyeBone(1)
    self:restoreFootBone()
    self:play("gh_lqhx"..self._breathID, RD.number(8, 12) / 10)
end

-- 生气
function M:anger()
    if self._main._isFinish or not self._main._isCanClick then return end
    self:angerEffect()
    if self._type == "zch" then
        self:restoreFootBone()
        self:play(self._type .. "_sq", nil, { loop = 0 })
    elseif self._type == "gh" then
        if self._number == 4 then
            self:play(self._type .. "_sq2", nil, { loop = 0 })
        else
            self:play(self._type .. "_sq1", nil, { loop = 0 })
        end
    end
end

-- 生气音效
function M:angerEffect()
    if self._isBoy then
        soundEffect:playEffectSfx26202018()
    else
        soundEffect:playEffectSfx26202019()
    end
end

-- 小激励
function M:happy()
    if self._type == "zch" then
        self:restoreFootBone()
        self:play(self._type .. "_jy")
    elseif self._type == "gh" then
        if self._number == 4 then
            self:play(self._type .. "_jy2")
        else
            self:play(self._type .. "_jy1")
        end
    end
end

--小激励
function M:cheer()
    self._cheer = true
    self:play("gh_lqgz"..self._breathID)
end

-- 欢呼开始
function M:cheer_1()
    self:changeFootBone()
    self:play("gh_lqhh"..self._breathID)
end



-- 进场
function M:enter()
    self:line({
        { "delay", 0.01 },
        { "fn", handler(self, self.walk) },
        { "moveTo", self._offsetX / 200, self._pos },
        { "fn", function()
            self:play("gh_lqgz")
            self:await()
        end}
    })
end

--------------------------
-- 触控
-------------------------- 

----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()

end


----------------------
-- 点击
----------------------
--[[--

判断是否点击到内容区域

### Example:

### Parameters:
-   string **event**    事件名称
-   number **x**        点击位置x(相对于屏幕)
-   number **y**        点击位置y(相对于屏幕)

]]
function M:onTouchHandled(event, x, y)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M