--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    
   Author:  Jeffers
   Date:    2021.09.01
   
   http://www.baby-bus.com/
   
]]


--------------------------
-- 引入命名空间
--------------------------


--[[
    类
--]]
local BaseGuideLayer = require("app.common.layer.BaseGuideLayer")
local M = class("Guide", BaseGuideLayer)


--------------------------
-- 构造函数
--------------------------
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- [本类调用]
    self._isGuide   = true
    -- 触控锁
    self._canTouch   = false
    -- 地图管理器
    self._mapManager = nil 
    -- 玩家
    self._player     = nil
    -- 角色列表
    self._roleList   = nil
    -- 点击的点
    self._touchPos   = cc.p(0, 0)
    -- 获得的爱心数量
    self._starNum    = 0
    -- 是否在打靶
    self._isShoot    = false
    -- 是否游戏结束
    self._isFinish   = false
    -- 游戏是否开始
    self._isGameStart = false
    -- 引导阶段
    self._guideState  = 0
end

--------------------------
-- 渲染
--------------------------
function M:onRender()
    M.super.onRender(self)

    -- 设置本地数据
	LData:getLocalData()
    collectgarbage("collect")
    local emptyNode = U.loadNode({ contentSize = cc.size(0.1, 0.1) }):to(self)
	self._bgNode    = emptyNode
    -- 加载地图
    self:loadMap()
    -- 初始化地图位置
    self._mapManager:judgeMap(self._player)
    -- 开启移动监听
    self:startSchedule()
    -- 加载拖尾
    self:loadMotionStreak()
    self:bindTouch()
    --------------------------------------------
    if Debug_Mode then
        -- 加载视图放大缩小按钮
        self:loadScaleSizeButton()
    end
end

--场景过渡完成
function M:onEnterTransitionFinish()
    -- 播放音效[sfx26230001]下雪环境音（全模块通用）
    soundEffect:playEffectsfx26230001()
end

-- 加载拖尾
function M:loadMotionStreak()
    self._trailing = CCMotionStreak:create(0.3, 2, 20, ccc3(222, 255, 255), "biathlon/trailing.png")
    self._trailing:setBlendFunc(GL_SRC_ALPHA, GL_ONE)
    self:addChild(self._trailing, 999)
end

-- 引导开始(子类重写)
function M:guideBegin()
    -- 引导阶段
    self._guideState  = self._guideState + 1
    if self._guideState == 1 then
        -- 游戏开始
        self:gameBegin()
        -- 引导
        self:guide()
    else
        -- 进入打靶
        self._mapManager:enterShoot()
    end
end

-- 引导
function M:guide(guideAction)
    if self._guideState ~= 1 then return end
    guideAction = ifnil(guideAction, self:guideAction())
    if not guideAction then return end

    local startPos = ccp(self._player:px() + 40, self._player:py() + 40)
    startPos = self._mapManager:convertToWorldSpace(startPos)
    -- 加载引导手
    local hand = D.img("common/guide/hand_1.png"):to(self):p(startPos)
    :scale(.6):z(V.h + 1):hide()
    self._handGuide = hand
    -- 开启引导调度器
    self._guideSch = guideAction
    hand:runAction(self._guideSch)
end

-- 引导动作
function M:guideAction()
    local endPos  = ccp(self._player:px() + 400, self._player:py()+40)
    local propbox = self._mapManager._propbox
    if self._player._curMaxSpeed == 0 then -- 开始引导吃道具
        if not tolua.isnull(propbox) then 
            endPos = ccp(self._player:px()+40, propbox:py())
        else
            return false
        end
    end
    endPos = self._mapManager:convertToWorldSpace(endPos)

    local guide = A.line({
        {"show"},
        {"image_range", "common/guide/hand_", 1, 3, .3},
        {"moveto", .9, endPos},
        {"fn", function()
            -- 继续引导判断
            self:continueGuideCheck()
        end},
        {"remove"},
    })
    return guide
end

-- 继续引导判断
function M:continueGuideCheck()
    if self._player._isShootOrPass then return end
	if self._player._body:GetLinearVelocity().x < 1 then
        -- 引导
        self:guide()
    else
        self:plan(function()
            -- 继续引导判断
            self:continueGuideCheck()
        end, 0.5)
    end
end

-- 游戏开始
function M:gameBegin()
    -- 游戏是否开始
    self._isGameStart = true
    -- 游戏开始
    self:gameStart()
end

-- 游戏开始
function M:gameStart()
    -- 开始游戏事件
    self._mapManager:gameStartEvent()
    self._canTouch = true    -- 开启触控
    for i,role in ipairs(self._roleList) do
        if not role._isPlayer then
            role:plan(function()
                role._fsm:doEvent("toSpeedup")-- 加速
            end, 0.05*N.random(2, 5))
        end
    end
end

-- 结束引导方法
function M:guideOver(score)
    CommonFun:switchScene("biathlon", 1, {layerId = 1})
end

-- 加载射击弹窗
function M:loadShootPopup(round)
    round = ifnil(round, 1)
	-- 射击弹窗
    local shootPopup = import("app.biathlon.node.ShootPopup").new({
        layer        = self,
        round        = round
    }):p(V.cw, V.ch)
    :to(self, 6000)

    if round == 2 then
        shootPopup:scale(0.5*Y_FACTOR):py(V.ch -35)
    end
end

-- 加载地图
function M:loadMap()
    local map = require("app.biathlon.node.MapManagerGuide").new({
        layer = self
    }):to(self):py(-626)
    
    self._mapManager   = map
    -- 获得玩家
    self._player       = map:getPlayer()
    -- 获得赛道管理器
    self._trackManager = map:getTrackManager()
    -- 获得角色列表
    self._roleList     = map:getRoleList()
end

-- 开启移动监听
function M:startSchedule()
    if self.mySchedule then return end
    self.mySchedule = scheduler.scheduleUpdateGlobal(handlerP(self, self.gameUpdate))
end

-- 关闭移动监听
function M:stopSchedule()
    if not self.mySchedule then
        return
    end
    SC.close(self.mySchedule)
    self.mySchedule = nil
end

-- 更新目标位置
function M:gameUpdate()
    self._mapManager:setTargetPosition()
end

-- 白屏过渡
function M:whiteScreen(cb1, endOpc)
    endOpc = ifnil(endOpc, 255)
    local time = 0.8
    local mask = self._mask
    if tolua.isnull(mask) then
        mask = U.loadNodeMask({contentSize = cc.size(V.w, V.h), color = ccc3(255, 255, 255),opacity = 0})
        :to(self, 990000):p(V.w_2, V.h_2)
        self._mask = mask
    end
    mask:line({
        {"fadeTo", time, endOpc},
        {"fn", function()
            if endOpc == 0 then
                self._mask = CommonFun.remove(self._mask)
            end
            if isfunction(cb1) then
                cb1()
            end
        end},
    })
end

-------------------------
-- 点击
-- ----------------------
function M:onTouchBegan(x, y, touches)
    if not self._canTouch then return end

    self._trailing:reset()
    if not tolua.isnull(self._handGuide) then
        self._handGuide:hide()
    end
    self._touchPos.x = x
    self._touchPos.y = y
    self._touchStartX = x
    return true 
end

function M:onTouchMoved(x, y, touches)
    if not self._canTouch then return end

    if not DRAGOPEN then
        self._trailing:setPosition(CCPointMake(x, y))
        if x - self._touchPos.x > 10 then
            local player     = self._player-- 玩家
            if player._curMaxSpeed ~= 0 then -- 没有被限制速度
                local disX       = x - self._touchPos.x
                self._touchPos.x = x
                local vel        = player._body:GetLinearVelocity() -- 线速度
                local result     = player:applySkiSpeed(disX/40)
                if player._extraSpeed <= 8 then
                    player._extraSpeed = player._extraSpeed + 0.08
                end

                if result then
                    player._fsm:doEvent("toSpeedup")-- 滑行
                end
            end
        end
        if self._player._curMaxSpeed == 0 and self._touchStartX and math.abs(self._touchPos.y - y) > 30 and math.abs(self._touchStartX - x) < 30 then
            local dir = -(self._touchPos.y - y)/math.abs(self._touchPos.y - y)
            -- 更换赛道
            self._trackManager:changeTrack(self._player, dir)
            self._touchStartX = nil -- 一次操作只能改一次赛道
        end
    elseif not self._isTouchChild then -- 调试用
        self._mapManager:px(self._mapManager:px() + x - self._touchPos.x)
        self._mapManager:py(self._mapManager:py() + y - self._touchPos.y)
        self._touchPos.x = x
        self._touchPos.y = y
    end
end

function M:onTouchEnded(x, y, touches)
    if DRAGOPEN then
        print(self._mapManager:px(), self._mapManager:py())
    end

    if ADDPOINT then
        local pos = self._mapManager:convertToNodeSpace(cc.p(x, y))
        
        local helpNode = D.img("biathlon/player_hint.png"):to(self._mapManager, 9999):p(pos):scale(0.2)
        -- 设置精灵对应地形边缘点的id
        self._mapManager:setSprEdgePosId(helpNode, pos.x)
        
        local Config = require("app.biathlon.Config")
        local posY = (Config.edgePoints[helpNode._edgePosId + 1].y - Config.edgePoints[helpNode._edgePosId].y)
                    * (pos.x - Config.edgePoints[helpNode._edgePosId].x)/(Config.edgePoints[helpNode._edgePosId + 1].x - Config.edgePoints[helpNode._edgePosId].x)
                    + Config.edgePoints[helpNode._edgePosId].y

        helpNode:py(posY)
        print("cc.p("..helpNode:px()..", "..helpNode:py()..")")
    end
end

--------------------------------------------
-- test
--------------------------------------------
-- 加载视图放大缩小按钮
function M:loadScaleSizeButton()
    self:loadButton("加点",cc.p(V.cw - 160, V.h - 80),function (node)
        ADDPOINT = not ADDPOINT
        if DRAGOPEN then
            node._label:color(cc.c3b(255, 0, 0))
        else
            node._label:color(cc.c3b(56, 128, 237))
        end
    end)
    self:loadButton("放大",cc.p(V.cw - 160, V.h - 40),function ()
        self._mapManager:scale(self._mapManager:scaleX() + 0.1)
    end)
    self:loadButton("缩小",cc.p(V.cw - 280, V.h - 40),function ()
        self._mapManager:scale(self._mapManager:scaleX()/1.5)
    end)
    self:loadButton("拖动",cc.p(V.cw - 220, V.h - 40),function (node)
        DRAGOPEN = not DRAGOPEN
        if DRAGOPEN then
            node._label:color(cc.c3b(255, 0, 0))
            -- 关闭移动监听
            self:stopSchedule()
        else
            node._label:color(cc.c3b(56, 128, 237))
            -- 开启移动监听
            self:startSchedule()
        end
    end)
end

-- 加载按钮
function M:loadButton(labelName, pos, callback)
    local node = CCNodeExtend.extend(display.newRoundedRect(cc.size(46, 30), 10, {
        fillColor = cc.c4f(0.22,0.22,0.22,0.47),
        borderColor = cc.c4f(0.784,0.784,0.784,0.47),
        borderWidth = 2
    })):p(pos):to(self,10000)
    node._label = self:loadText(labelName, node, cc.p(23,15))
    node:bindTouch()
    node.callback = callback
    function node:onTouchBegan(x, y, touches)
        self:scale(1.1)
        return SIGN_TOUCH_BEGAN_SWALLOWS 
    end
    function node:onTouchEnded(x, y, touches)
        self:scale(1)
        if self.callback then
            self.callback(node)
        end
    end
    return node
end

-- 加载文字
function M:loadText(text, partent, position, dimensions)
    local fontSize =  16
    local color = ccc3(44, 131, 252)
    -- 文字
    local lb = U.loadLabel({ 
        text        = text,
        fontSize    = fontSize, 
        color       = color,
        position    = position,
        dimensions  = dimensions
    }):to(partent)
    return lb
end
----------------------
-- 辅助功能
----------------------

--置空全部参数
function M:emptyAllNode()
	self._bg = nil
	collectgarbage("collect")
end

----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
	M.super.onDestructor(self)
	self:emptyAllNode()
end

function M:onExitTransitionStart()
    -- 关闭移动监听
    self:stopSchedule()
	GLOBAL_TOUCH_LOCK = true
	--停止所有动作
	ND.stopActionCascade(self)
end







return M






