--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    
   Author:  Neighbour Mr.Wang
   Date:    2021.07.30
   
   http://www.baby-bus.com/
   
]]
----------------------
-- 类
----------------------
local M = classLayer("Main")


----------------------
-- 公共参数
----------------------
-- 总的有效滑动次数
local sumSlideTime = 0 
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
	M.super.ctor(self, params)
	-- 触控锁
    self._canTouch   = false
    -- 地图管理器
    self._mapManager = nil 
    -- 玩家
    self._player     = nil
    -- 角色列表
    self._roleList   = nil
    -- 点击的点
    self._touchPos   = cc.p(0, 0)
    -- 获得的爱心数量
    self._starNum    = 0
    -- 是否在打靶
    self._isShoot    = false
    -- 是否游戏结束
    self._isFinish   = false
    -- 游戏是否开始
    self._isGameStart = false
    -- 是否需要预览
    self._isNeedPreView = false
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- 设置本地数据
	LData:getLocalData()
    -- M.super.onRender(self)
    -- 首次进入语音,更新本地数据
    CommonFun:playFirstGuideVoice(self)
    collectgarbage("collect")
    self:bindTouch()
    local emptyNode = U.loadNode({ contentSize = cc.size(0.1, 0.1) }):to(self)
	self._bgNode    = emptyNode
    -- 加载地图
    self:loadMap()
    if not self._isNeedPreView then
        -- 初始化地图位置
        self._mapManager:judgeMap(self._player)
        -- 开启移动监听
        self:startSchedule()
    else
        local map = self._mapManager
        local sc  = V.h/(map._mapSize.height*2.8)
        map:scale(sc):p(0, 0)
    end
    -- 加载拖尾
    self:loadMotionStreak()
    --------------------------------------------
    -- debug
    --------------------------------------------
    if Debug_Mode then
        -- 加载视图放大缩小按钮
        self:loadScaleSizeButton()
    end
end

-- 场景过渡完成
function M:onEnterTransitionFinish()
    if not self._isNeedPreView then
        -- 倒计时准备开始游戏
        self:countDownToStartGame()
    else
        local map = self._mapManager
        local sc  = V.h/(map._mapSize.height*2.8)
        map:scale(sc):p(0, 0)
        -- -- 预览
        A.line({
            {"easing","sinein", {"moveBy", 3, cc.p(-map._mapSize.width*3, 0)}},
            {"place", cc.p(-(map._row-2)*map._mapSize.width*sc + V.w + 200 + map._mapSize.width*1, 0)},
            {"easing","sineout", {"moveBy", 1, cc.p(-map._mapSize.width*1, 0)}},
            {"fn", function()
                -- 白屏过渡
                self:whiteScreen(function ()
                    if tolua.isnull(map) then return end
                    map:scale(1)
                    -- 开启移动监听
                    self:startSchedule()
                    -- 白屏过渡
                    self:whiteScreen(function ()
                        -- 倒计时准备开始游戏
                        self:countDownToStartGame()
                    end, 0) 
                end) 
            end},
        }):at(map)
    end

    -- 播放音效[sfx26230001]下雪环境音（全模块通用）
    soundEffect:playEffectsfx26230001()
end

-- 加载拖尾
function M:loadMotionStreak()
    self._trailing = CCMotionStreak:create(0.3, 2, 20, ccc3(222, 255, 255), "biathlon/trailing.png")
    self._trailing:setBlendFunc(GL_SRC_ALPHA, GL_ONE)
    self:addChild(self._trailing, 999)
end

-- 倒计时准备开始游戏
function M:countDownToStartGame()
    self._bgNode:rotate(-0.2):px(-60)

	local countDown = CommonFun:loadCountDown(self)          -- 加载倒计时 
    A.line({
        {"delay", 3.5},
        {"fn", function()
            for i,role in ipairs(self._roleList) do
                -- 准备
                role:playReady()
            end
        end},
    }):at(self)
end

-- 开始游戏的倒计时事件
function M:startCountEvent(coutName)
    if #coutName > 1 then return end
	A.union({
        {"moveBy", 0.2, cc.p(20, 0)},
        {"rotateTo", 0.2, 0},
    }):at(self._bgNode)
end

-- 游戏开始
function M:gameBegin()
    -- 初始化地图位置
    self._mapManager:judgeMap(self._player)
    -- 开启移动监听
    self:startSchedule()
    -- 游戏是否开始
    self._isGameStart = true
    -- 游戏开始
    self:gameStart()
end

-- 游戏开始
function M:gameStart()
    -- 开始游戏事件
    self._mapManager:gameStartEvent()
    -- 引导
    self:guide()
    self._canTouch = true    -- 开启触控
    -- self._progressBar:openSC()
    for i,role in ipairs(self._roleList) do
        if not role._isPlayer then
            role:plan(function()
                role._fsm:doEvent("toSpeedup")-- 加速
            end, 0.06*N.random(1, 8))
            -- 出发
            -- role:playSetOut()
        end
    end

    -- 播放音效[sfx26233012]比赛开始时，其他三个选手起跑（看作整体）
    soundEffect:playEffectsfx26233012()
end

-- 引导
function M:guide(guideAction, delayTime)
    delayTime      = ifnil(delayTime, 0)
    guideAction    = ifnil(guideAction, self:guideAction(delayTime))
    if tolua.isnull(self._handGuide) then
        -- 加载引导手
        local hand = D.img("common/guide/hand_1.png"):to(self)
        :scale(.6):z(V.h + 1):hide()
        self._handGuide = hand
    end
    -- 开启引导调度器
    self._guideSch = guideAction
    self._handGuide:runAction(self._guideSch)
end

-- 引导动作
function M:guideAction(delayTime)
    if self._guideSch then return end

    local player = self._player
    local endPos = cc.p(player:px() + 400, player:py()+40)
    endPos       = self._mapManager:convertToWorldSpace(endPos)

    local guide = A.line({
        {"delay", delayTime},
        {"fn", function()
            local vX = player._body:GetLinearVelocity().x
            if (not self._canTouch) or player._isShootOrPass or player._floorDis > 5 or vX > 2 or vX < -2  then -- 在打靶子, 速度过大时, 或者在空中时不引导
                self._handGuide:stop()
                self._handGuide:hide()
                self._guideSch = nil
            else
                local startPos = cc.p(player:px() + 40, player:py()+40)
                startPos       = self._mapManager:convertToWorldSpace(startPos)
                self._handGuide:p(startPos)
            end
        end},
        {"delay", 0.2},
        {"show"},
        {"image_range", "common/guide/hand_", 1, 3, .3},
        {"moveto", .9, endPos},
        {"fn", function()
            self._guideSch = nil
            -- 引导
            self:guide(nil, 5)
        end},
        {"hide"},
    })
    return guide
end

-- 加载射击弹窗
function M:loadShootPopup(round)
    round = ifnil(round, 1)
	-- 射击弹窗
    local shootPopup = import("app.biathlon.node.ShootPopup").new({
        layer        = self,
        round        = round
    }):p(V.cw, V.ch)
    :to(self, 6000)

    if round == 2 then
        shootPopup:scale(0.5*Y_FACTOR):py(V.ch -35)
    end
end

-- 加载倒计时
function M:loadCountDown()
    if self._isFinish then return end
    if self._isShoot or self._player._isShootOrPass then -- 停止进度条更新 then 
        self:plan(function()
            -- 加载倒计时
            self:loadCountDown()
        end, 1)
        return
    end -- 正在打靶则延后加载

	local sprite      = require("app.biathlon.node.CountDown")
    local countDown = sprite.new({
          layer   = self,
          pos     = cc.p(65.5 + X_OFFSET, 242.7 + Y_OFFSET),
          time    = self._totalTime,
          percent = {25, 60, 95},
    }):to(self, 50):scale(.9)
    self._countDown = countDown 
    countDown:enter()
end

-- 显示倒计时
function M:showCountDown()
	if not tolua.isnull(self._countDown) then
        self._countDown:show()
    end
end

-- 隐藏倒计时
function M:hideCountDown()
    if not tolua.isnull(self._countDown) then
        self._countDown:hide()
    end
end

-- 游戏结束
function M:gameFinish(rank)
    if self._isFinish then return end
    self._isFinish   = true
    -- 停止玩家滑雪音效
    self._player:stopPlayerSkiSnd()
    -- 停止正常滑雪音效(无操作)
    self._player:stopSkiSnd()
    if not tolua.isnull(self._countDown) then
        -- 停止闹钟动作
        self._countDown:stopClockAct()
    end

    print("排名为:"..rank)
    if rank <= 3 then
        -- 结束激励
        CommonFun:finishParticle(self)
    end
    self:line({
        {"delay", 2},
        {"fn", function()
            local score = 4-rank
            -- 加载结算页面
            CommonFun:loadSettleAccount(self, score)
        end}
    })
end

-- 加载地图
function M:loadMap()
    local map = require("app.biathlon.node.MapManager").new({
        layer = self
    }):to(self._bgNode):py(-626)
    
    self._mapManager   = map
    -- 获得玩家
    self._player       = map:getPlayer()
    -- 获得赛道管理器
    self._trackManager = map:getTrackManager()
    -- 获得角色列表
    self._roleList     = map:getRoleList()
end

-- 开启移动监听
function M:startSchedule()
    if self.mySchedule then return end
    self.mySchedule = scheduler.scheduleUpdateGlobal(handlerP(self, self.gameUpdate))
end

-- 关闭移动监听
function M:stopSchedule()
    if not self.mySchedule then
        return
    end
    SC.close(self.mySchedule)
    self.mySchedule = nil
end

-- 更新目标位置
function M:gameUpdate()
    self._mapManager:setTargetPosition()
end

-- 白屏过渡
function M:whiteScreen(cb1, endOpc)
    endOpc = ifnil(endOpc, 255)
    local time = 0.8
    local mask = self._mask
    if tolua.isnull(mask) then
        mask = U.loadNodeMask({contentSize = cc.size(V.w, V.h), color = ccc3(255, 255, 255),opacity = 0})
        :to(self, 990000):p(V.w_2, V.h_2)
        self._mask = mask
    end
    mask:line({
        {"fadeTo", time, endOpc},
        {"fn", function()
            if endOpc == 0 then
                self._mask = CommonFun.remove(self._mask)
            end
            if isfunction(cb1) then
                cb1()
            end
        end},
    })
end

-- 施加额外的速度
function M:applyExtraSpeed()
    sumSlideTime = sumSlideTime + 1
    -- 增加额外的速度
    if sumSlideTime % 2 == 1 then -- 每有效滑动3次, 提供满额加速
        local player     = self._player
        local extraSpeed = player._extraSpeed
        local addSpeed   = 12 -- 需要添加的速度
        local maxExSpeed   = player._extraMaxSpeed
        extraSpeed         = extraSpeed + addSpeed
        extraSpeed         = extraSpeed <= maxExSpeed and extraSpeed or maxExSpeed
        player._extraSpeed = extraSpeed
        -- print("❤️", player._extraSpeed, addSpeed)
        local result = player:applySkiSpeed(50)
        if result then
            -- 角色缩放动作
            self:roleDoScaleAct(extraSpeed/200, player)
        else
            -- 角色缩放动作
            self:roleDoScaleAct(0, player)
        end
    end
end

-- 角色缩放动作
function M:roleDoScaleAct(exSc, player)
    if player._scAct then
        player:stopAction(player._scAct)
        player._scAct = nil
    end
    player._scAct = A.line({
        {"scaleTo", 0.2, 1+exSc},
        {"delay", 0.2},
        {"scaleTo", 1, 1},
        {"fn", function()
            player._scAct = nil
        end},
    }):at(player)
end

-- 播放角色开心声音
function M:playRoleHappySnd(role)
    if self._happySnd then return end
    if self._isFinish then return end
	if role._isPlayer then
        -- 播放音效开心
        self._happySnd = soundEffect["playEffectsfx2623000"..N.random(3, 4)](soundEffect)
    elseif role:isInScreen() then
        if role._name == "hx1_" then
            -- 播放音效男路人开心
            self._happySnd = soundEffect["playEffectsfx2623302"..N.random(6, 8)](soundEffect)
        else
            -- 播放音效女路人开心
            self._happySnd = soundEffect["playEffectsfx262330"..N.random(29, 31)](soundEffect)
        end
    end
    if self._happySnd then
        sound.setFinishCallback(self._happySnd, function ()
            self._happySnd = nil 
        end)
    end
end

----------------------
-- 点击
-- ----------------------
function M:onTouchBegan(x, y, touches)
    if not self._canTouch then return end

    if not tolua.isnull(self._handGuide) then
        self._handGuide:stop()
        self._handGuide:hide()
        self._guideSch = nil
    end
    self._trailing:reset()
    self._startX  = x
    self._touchPos.x  = x
    self._touchPos.y  = y
    self._touchStartX = x
    self._canChangeTrack = true -- 能够换赛道
    -- 自动加速动作
    self._player:autoApplySpeedAct()

    return true 
end

function M:onTouchMoved(x, y, touches)
    if not self._canTouch then return end

    if not DRAGOPEN then
        self._trailing:setPosition(CCPointMake(x, y))
        if x - self._touchPos.x > 10 then
            local player     = self._player-- 玩家
            local disX       = x - self._touchPos.x
            self._touchPos.x = x
            local vel        = player._body:GetLinearVelocity() -- 线速度
            local result     = player:applySkiSpeed(disX/10)

            if result then
                player._fsm:doEvent("toSpeedup")-- 滑行
            end
        end
        if self._canChangeTrack and math.abs(self._touchPos.y - y) > 30 and math.abs(self._touchStartX - x) < 30 then
            local dir = -(self._touchPos.y - y)/math.abs(self._touchPos.y - y)
            -- 更换赛道
            self._trackManager:changeTrack(self._player, dir)
            self._canChangeTrack = false -- 一次操作只能改一次赛道
        end
    elseif not self._isTouchChild then -- 调试用
        self._mapManager:px(self._mapManager:px() + x - self._touchPos.x)
        self._mapManager:py(self._mapManager:py() + y - self._touchPos.y)
        self._touchPos.x = x
        self._touchPos.y = y
    end
end

function M:onTouchEnded(x, y, touches)
    -- 停止自动加速动作
    self._player:stopAutoApplySpeedAct()
    if self._startX < x then -- 避免反向操作加速
        -- 施加额外的速度
        self:applyExtraSpeed()
    end
    -- 引导
    self:guide(nil, 5)

    if DRAGOPEN then
        print(self._mapManager:px(), self._mapManager:py())
    end

    if ADDPOINT then
        local pos = self._mapManager:convertToNodeSpace(cc.p(x, y))
        
        local helpNode = D.img("biathlon/player_hint.png"):to(self._mapManager, 9999):p(pos):scale(0.2)
        -- 设置精灵对应地形边缘点的id
        self._mapManager:setSprEdgePosId(helpNode, pos.x)
        
        local Config = require("app.biathlon.Config")
        local posY = (Config.edgePoints[helpNode._edgePosId + 1].y - Config.edgePoints[helpNode._edgePosId].y)
                    * (pos.x - Config.edgePoints[helpNode._edgePosId].x)/(Config.edgePoints[helpNode._edgePosId + 1].x - Config.edgePoints[helpNode._edgePosId].x)
                    + Config.edgePoints[helpNode._edgePosId].y

        helpNode:py(posY)
        print("cc.p("..helpNode:px()..", "..helpNode:py()..")")
    end
end

--------------------------------------------
-- test
--------------------------------------------
-- 加载视图放大缩小按钮
function M:loadScaleSizeButton()
    self:loadButton("加点",cc.p(V.cw - 160, V.h - 80),function (node)
        ADDPOINT = not ADDPOINT
        if DRAGOPEN then
            node._label:color(cc.c3b(255, 0, 0))
        else
            node._label:color(cc.c3b(56, 128, 237))
        end
    end)
    self:loadButton("放大",cc.p(V.cw - 160, V.h - 40),function ()
        self._mapManager:scale(self._mapManager:scaleX() + 0.1)
    end)
    self:loadButton("缩小",cc.p(V.cw - 280, V.h - 40),function ()
        self._mapManager:scale(self._mapManager:scaleX()/1.5)
    end)
    self:loadButton("拖动",cc.p(V.cw - 220, V.h - 40),function (node)
        DRAGOPEN = not DRAGOPEN
        if DRAGOPEN then
            node._label:color(cc.c3b(255, 0, 0))
            -- 关闭移动监听
            self:stopSchedule()
        else
            node._label:color(cc.c3b(56, 128, 237))
            -- 开启移动监听
            self:startSchedule()
        end
    end)
end

-- 加载按钮
function M:loadButton(labelName, pos, callback)
    local node = CCNodeExtend.extend(display.newRoundedRect(cc.size(46, 30), 10, {
        fillColor = cc.c4f(0.22,0.22,0.22,0.47),
        borderColor = cc.c4f(0.784,0.784,0.784,0.47),
        borderWidth = 2
    })):p(pos):to(self,10000)
    node._label = self:loadText(labelName, node, cc.p(23,15))
    node:bindTouch()
    node.callback = callback
    function node:onTouchBegan(x, y, touches)
        self:scale(1.1)
        return SIGN_TOUCH_BEGAN_SWALLOWS 
    end
    function node:onTouchEnded(x, y, touches)
        self:scale(1)
        if self.callback then
            self.callback(node)
        end
    end
    return node
end

-- 加载文字
function M:loadText(text, partent, position, dimensions)
    local fontSize =  16
    local color = ccc3(44, 131, 252)
    -- 文字
    local lb = U.loadLabel({ 
        text        = text,
        fontSize    = fontSize, 
        color       = color,
        position    = position,
        dimensions  = dimensions
    }):to(partent)
    return lb
end
----------------------
-- 辅助功能
----------------------

--置空全部参数
function M:emptyAllNode()
	self._bg = nil
	collectgarbage("collect")
end

----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
	M.super.onDestructor(self)
	self:emptyAllNode()
end

function M:onExitTransitionStart()
    -- 关闭移动监听
    self:stopSchedule()
	GLOBAL_TOUCH_LOCK = true
	--停止所有动作
	ND.stopActionCascade(self)
	if not self._isNotClear then
        R.removeAllTextures()
    end
end



return M