--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    障碍物(弹簧, 冰柱, 雪坑, 冰面)
   Author:  Neighbour Mr.Wang
   Date:    2021.08.06
   
   http://www.baby-bus.com/
   
]]----------------------
-- 类
----------------------

local M     = classSprite("Barrier")
M.TAG       = "Barrier"

----------------------
-- 公共参数
----------------------
local Config = require("app.biathlon.Config")

-- [常量]
-- 障碍物名字
local barrierName = {
    "icicles",  -- 冰柱(撞晕停止3s)
    "bellows",  -- 弹簧
    "snowpit",  -- 雪坑
    "iceblock",  -- 冰块
}
-- 障碍物信息(锚点, 缩放)
local barrierInfo = {
    {cc.p(0.5, 0.03), 1},  -- 冰柱
    {cc.p(0.5, 0.17), 0.8},  -- 弹簧
    {cc.p(0.5, 0.5), 0.7},  -- 雪坑
    {cc.p(0.5, 0.5), 0.5},  -- 冰块
}

-- 高度限制表(在指定高度内才能触发碰到)
local highLimitList = {
    100, 15, 15, 20
}

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]

function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    -- 障碍物类型
    self._types      = params.types
    self._isBarrier  = self._types ~= 2 -- 弹簧不算障碍物
    -- 障碍物名字
    self._name       = barrierName[self._types]
    -- y偏移量的id
    self._offYId     = params.offYId
    -- y偏移量
    self._offY       = params.offY
    -- 地图管理器
    self._mapManager = params.mapManager
    -- 高度限制(在指定高度内才能触发碰到)
    self._highLimit  = highLimitList[self._types]
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    
    -- 图片渲染
    self:loadDisplay()
    table.insert(self._mapManager._propBarrierList, self)
end

-- 图片渲染
function M:loadDisplay()
    if self._name == "snowpit" then -- 雪坑
        self:display("biathlon/barrier/"..self._name.."/bottom/1.png"):scale(1.44)
    else
        self:display("biathlon/barrier/"..self._name.."/1.png")
    end

    if self._name == "icicles" then -- 冰柱
        self:z(4995 - self._offY*10)
    else
        self:z(100)
    end

    self:anchor(barrierInfo[self._types][1]):scale(barrierInfo[self._types][2])
end

-- 被碰撞事件
function M:beCrashEvent(role)
    if (role:px() - self:px()) > 60 then return end -- 靠右减少判断距离
    T.removeOrder(self._mapManager._propBarrierList, self)

    if self._name == "icicles" then -- 冰柱
       -- 冰柱被撞击事件
       self:iciclesBeCrashEvent(role)
    elseif self._name == "iceblock" then
        -- 冰块被撞击事件
        self:iceblockBeCrashEvent(role)
    elseif self._name == "bellows" then
        -- 弹簧被撞击事件
        self:bellowsBeCrashEvent(role)
    elseif self._name == "snowpit" then
        -- 雪坑被撞击事件
        self:snowpitBeCrashEvent(role)
    end
end

-- 同步刚体y轴偏移量(处理在换赛道时碰到障碍物)
function M:synRelBodyOffsetY(role)
	if role._relBodyOffsetY ~= self._offY then
        local trackManager = self._mapManager:getTrackManager() -- 赛道管理器
        -- 强行设置角色y偏移量
        trackManager:forceSetRoleOffsetY(role, self._offYId)
    end
end

-- 雪坑被撞击事件
function M:snowpitBeCrashEvent(role)
    role._defaultAngle = self:rotate() -- 使在异常状态下角色角度不出问题

    role._fsm:doEvent("toUnusual")-- 异常状态
    role._body:SetLinearVelocity(b2Vec2(0, 0))
    role._body:SetLinearDamping(1000)
    -- 重置额外的速度
    role:resetExSpees()
    -- 转化为地图管理器上的坐标
    local pos = self:convertPosToMapManager(cc.p(self:cw()/2, self:ch()/4*3))
    -- 雪坑顶部
    local topSnowpit = D.img("biathlon/barrier/snowpit/top/3.png"):p(self:point()):to(self._mapManager, 5000-self._offY*10-10):scale(self:scaleX())
    A.line({
        {"jumpTo", 0.2, pos, 10, 1},
        {"fn", function()
            -- 雪坑被撞击事件持续状态
            self:snowpitBeCrashEventing(role, topSnowpit)
        end},
    }):at(role)
end

-- 雪坑被撞击事件持续状态
function M:snowpitBeCrashEventing(role, topSnowpit)
	-- 掉进雪坑-准备
    role:playSnowHoleBegin()
    A.line({
        {"image", "biathlon/barrier/snowpit/bottom/", 3, 0.1},
        {"fn", function()
            role._ignoreSynPhyZ = true -- 忽略层级同步
            role:z(5000-self._offY*10-30)
            A.line({
                {"imagerange", "biathlon/barrier/snowpit/top/", 3, 5, 0.1},
            }):at(topSnowpit)
        end},
        {"imagerange", "biathlon/barrier/snowpit/bottom/", 3, 5, 0.1},
        {"delay", 0.5},
        {"fn", function()
            A.line({
                {"fadeOut", 0.3},
                {"remove"},
            }):at(topSnowpit)
            -- 雪坑被撞击事件结束
            self:snowpitBeCrashEventEnd(role)
        end},
        {"fadeOut", 0.3},
        {"remove"},
    }):at(self)
end

-- 转化为地图管理器上的坐标
function M:convertPosToMapManager(pos)
    pos = self:convertToWorldSpace(pos)
    pos = self._mapManager:convertToNodeSpace(pos)

    return pos
end

-- 雪坑被撞击事件结束
function M:snowpitBeCrashEventEnd(role)
    -- 同步刚体y轴偏移量(处理在换赛道时碰到障碍物)
    self:synRelBodyOffsetY(role)
    role._body:SetLinearDamping(0)

    local x, y = role:px(), role:py()
    ZBox2D.setBodyPosition(role._body, cc.p(x, y-role._relBodyOffsetY))
    role._body:SetLinearVelocity(b2Vec2(1, 1))

    -- 转化为地图管理器上的坐标
    local pos = self:convertPosToMapManager(cc.p(self:cw()/2, self:ch()/2))
    A.one({"jumpTo", 0.2, pos, 10, 1}):at(role)
    -- 掉进雪坑-结束
    role:playSnowHoleEnd()
end

-- 弹簧被撞击事件
function M:bellowsBeCrashEvent(role)
    role._fsm:doEvent("toUnusual")-- 异常状态

    -- 转化为地图管理器上的坐标
    local pos = self:convertPosToMapManager(cc.p(80, 30))
    A.line({
        {"jumpTo", 0.16, pos, 10, 1},
        {"fn", function()
            if role._isPlayer then
                -- 播放音效[sfx26233021]弹簧弹起
                soundEffect:playEffectsfx26233021()
                -- 播放配音[v262331]弹簧弹起的时候
                soundVoice:playEffectv262331()
            end
            self:z(4995 - self._offY*10)
            A.line({
                {"image", "biathlon/barrier/bellows/", 16, 0.04},
                A.reverse({"image", "biathlon/barrier/bellows/", 16, 0.04}),
                {"fadeOut", 0.1},
                {"remove"},
            }):at(self)
        end},
        {"delay", 0.03},
        {"moveBy", 0.084, cc.p(10, 60)},
        {"fn", function()
           	 -- 同步刚体y轴偏移量(处理在换赛道时碰到障碍物)
            self:synRelBodyOffsetY(role)
            ZBox2D.setBodyPosition(role._body, ccp(role:px(), role:py()-role._relBodyOffsetY+20))
            role._body:SetLinearVelocity(Config.bellowSpeed)
            -- 弹簧弹飞效果
            role:flyWithBellowsEffect()
            role._fsm:doEvent("toNormal")-- 正常状态
            role._fsm:doEvent("toFly")-- 飞起状态
        end},
    }):at(role)
end

-- 冰块被撞击事件
function M:iceblockBeCrashEvent(role)
    role._fsm:doEvent("toUnusual")-- 异常状态
    role._body:SetLinearVelocity(b2Vec2(0, 0))
    role._body:SetLinearDamping(3)
    if role._isPlayer then
        -- 播放音效失落
        soundEffect["playEffectsfx2623000"..N.random(5, 6)](soundEffect)
        -- 播放音效[sfx26233017]滑到冰块处
        soundEffect:playEffectsfx26233017()
    end
    -- 转化为地图管理器上的坐标
    local pos = self:convertPosToMapManager(cc.p(120, 50))
    A.line({
        {"easing","sineout",{"moveTo", 0.3, pos}},
        {"fn", function()
            -- 同步刚体y轴偏移量(处理在换赛道时碰到障碍物)
            self:synRelBodyOffsetY(role)
            ZBox2D.setBodyPosition(role._body, ccp(role:px(), role:py()-role._relBodyOffsetY+22))
            role._fsm:doEvent("toNormal")-- 正常状态
            role._body:SetLinearVelocity(b2Vec2(-10, -2))
            -- 重置额外的速度
            role:resetExSpees()

            role._fsm:doEvent("toSkid")-- 打滑
            A.line({
                {"delay", 0.4},
                {"fn", function()
                    if not tolua.isnull(role) then
                        -- 道具碎裂
                        self:barrierFracture("biathlon/barrier/iceblock/1.png")
                        role._body:SetLinearDamping(0)
                    end
                end},
                {"fadeOut", 0.1},
                {"remove"},
            }):at(self)
        end},
        {"delay", 1},
        {"fn", function()
            role._body:SetLinearVelocity(b2Vec2(0, 0))
            role._fsm:doEvent("toStopSkid")-- 打滑结束
        end},
    }):at(role)
end

-- 冰柱被撞击事件
function M:iciclesBeCrashEvent(role)
    role._body:SetLinearDamping(1000)

    role._body:SetLinearVelocity(b2Vec2(0, 0))
    -- 重置额外的速度
    role:resetExSpees()
    local dir = role:px() > self:px() and 1 or -1
    if role._isPlayer then
        -- 屏幕晃动
        self._mapManager:screenShake(10*dir, 50*dir)
    end
    -- 被撞击事件
    role:beCrashEvent(-0.5)
    if role._isPlayer then
        soundEffect:playEffectsfx26233015()
        -- 播放音效失落
        soundEffect["playEffectsfx2623000"..N.random(5, 6)](soundEffect)
    end
    -- 粒子
    local particle1 = P.newParticle("particle/biathlon/zhuangbingzhu.plist", 140, 110):to(role, 100)
    A.line({
        {"rotateBy", 0.05, -8},
        {"rotateBy", 0.05, 8},
        {"rotateBy", 0.04, -6},
        {"rotateBy", 0.04, 6},
        {"rotateBy", 0.03, -4},
        {"rotateBy", 0.03, 4},
        {"rotateBy", 0.02, -2},
        {"rotateBy", 0.02, 2},
        {"fn", function()
            if not tolua.isnull(role) then
                -- 道具碎裂
                self:barrierFracture("biathlon/barrier/icicles/1.png")
                role._body:SetLinearDamping(0)
            end
        end},
        {"fadeOut", 0.1},
        {"remove"},
    }):at(self)
end

-- 道具碎裂
function M:barrierFracture(imgPath)
    local node = CshatterSprite()
    node:init(imgPath)
    node:setPosition(self:cw()/2, self:ch()/2)
    self:addChild(node, 1000)
    node:setOpacity(0)
    local action = CshatterAction:create(10)
    node:runAction(action)
    
    CCNodeExtend.extend(node)
    node:plan(function()
        CommonFun.remove(node)
    end, 1)
end

----------------------
-- 结点析构
----------------------

function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]

function M:onExitTransitionStart()

end

----------------------
-- 验证
----------------------

-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end

return M
