--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    缆车
   Author:  Neighbour Mr.Wang
   Date:    2021.08.18
   
   http://www.baby-bus.com/
   
]]
local M     = classSprite("CableCar")


local wayPoints = {
    cc.p(2352, 103), cc.p(2385, 125), cc.p(2407, 156),
    cc.p(2407, 325), cc.p(2388, 362), cc.p(2342, 380), cc.p(2272, 380), 
    cc.p(128, 380), cc.p(78, 380), cc.p(39, 365), cc.p(15, 335), 
    cc.p(15, 147), cc.p(41, 116), cc.p(82, 103), 
}

----------------------
-- 构造方法
----------------------
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    self._cableCarList = {}
end




--------------------------------------------
-- 结点渲染 
--------------------------------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

    -- 图片渲染
    self:loadDisplay()
    -- 加载缆车
    self:loadCableCar()
end

-- 图片渲染
function M:loadDisplay()
    -- 底座(分前后层)
    local baseBack  = D.img("biathlon/decorate/cablecar/base1back.png"):p(0, 0):to(self, -100):anchor(cc.p(0, 0))
    local base1     = D.img("biathlon/decorate/cablecar/base2.png"):p(1022, 0):to(self, 1):anchor(cc.p(0, 0))
    local base2     = D.img("biathlon/decorate/cablecar/base3.png"):p(2046, 0):to(self, 1):anchor(cc.p(0, 0))
    local base2Back = D.img("biathlon/decorate/cablecar/base3back.png"):p(2046, 0):to(self, -100):anchor(cc.p(0, 0))
end

-- 加载缆车
function M:loadCableCar()
	local posList = {
        cc.p(708, 99), cc.p(1354, 96), cc.p(2060, 96), cc.p(2058, 376), cc.p(1352, 376), 
        cc.p(714, 371), 
    }

    for i,pos in ipairs(posList) do
        -- 缆车
        local cableCar = D.img("biathlon/decorate/cablecar/"..i..".png"):p(pos):to(self, 50):anchor(cc.p(0.5, 1))
        local topHook  = D.img("biathlon/decorate/cablecar/hook.png"):p(cableCar:cw()/2, cableCar:ch()):to(cableCar, 50):anchor(cc.p(0.5, 0.1))
        cableCar._topHook = topHook

        if i > 3 then
            cableCar._nextPosId = 8
        else
            cableCar._nextPosId = 1
        end
        cableCar._topNode = D.img("biathlon/decorate/cablecar/hook.png"):p(wayPoints[cableCar._nextPosId]):to(self, 50):anchor(cc.p(0.5, 0.1)):hide()

        table.insert(self._cableCarList, cableCar)
    end
    -- 缆车动作
    self:doCableCarAct()
end

-- 更新下一目标点Id
function M:updateCableCarNextPosId(car, x, y)
	local nextPosId = car._nextPosId + 1
    car._nextPosId = nextPosId <= #wayPoints and nextPosId or 1
    if car._nextPosId == 2 or car._nextPosId == 9 then -- 两个需要处理层级问题的拐点
        car._topNode:p(x, y):show() -- 处理层级问题
        car._topHook:hide()
        car:z(-1)
    elseif car._nextPosId == 8 or car._nextPosId == 14 then -- 两个需要处理层级问题的拐点
        car._topHook:show()
        car._topNode:hide()
        car:z(50)
    end
end

-- 缆车动作
function M:doCableCarAct()
    if self._isPlayAct then return end
    self._isPlayAct = true
    A.cycle({
		{"delay", 1/30},
		{"fn", function()
            if not self:getParent():isVisible() then return end
			for i,car in ipairs(self._cableCarList) do
                local x, y = car:px(), car:py()     -- 当前x,y坐标
                local subX = wayPoints[car._nextPosId].x - x -- 与下一目标点的x差值
                local subY = wayPoints[car._nextPosId].y - y
                local absSubX, absSubY = math.abs(subX), math.abs(subY)
                if absSubX <= 2 and absSubY <= 2 then -- 需要移动的距离很小的情况下, 直接移到目标点并开始移向下一个点
                    -- 更新下一目标点Id
                    self:updateCableCarNextPosId(car, x, y)
                end

                local mulX, mulY = 2, 1 -- 移动速率倍数
                if (absSubX >= 2 and absSubY >= 1) then--在拐角处移动. y轴移动速度减慢
                    local k = math.atan(absSubX/absSubY)
                    mulX, mulY = k, 0.5/k
                end
                if absSubX >= 2 then 
                    x = x + subX/absSubX*mulX
                    car:px(x)
                    if car._topNode:isVisible() then
                        car._topNode:px(x)
                    end
                end
                if absSubY >= 1 then
                    y = y + subY/absSubY*mulY
                    car:py(y)
                    if car._topNode:isVisible() then
                        car._topNode:py(y)
                    end
                end
            end
		end},
	}):at(self)
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end








return M