--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    倒计时
   Author:  Neighbour Mr.Wang
   Date:    2021.08.23
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------
local M     = classSprite("CountDown")
M.TAG       = "CountDown"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- mainLayer
    self._layer     = params.layer
    -- 位置
    self._pos       = params.pos
    self._totalTime = 20
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
    -- 加载数字框
    self:loadNumberFrame()
    -- 加载闹钟
    self:loadClock()
    -- 加载倒计时数字
    self:loadNumber()
end

-- 图片渲染
function M:loadDisplay()
    self:setContentSize(cc.size(0.1, 0.1))
    self:p(-100, 245 + Y_OFFSET)
    self._pos = cc.p(65.5 + X_OFFSET, 245 + Y_OFFSET)
end

-- 进场
function M:enter(callback)
    self._time = self._totalTime               -- 剩余时间
    self:showClock()
    -- 开启调度器
    self:openSC()
    self:line({
        {"easing", "BackOut", {"moveTo", .5, self._pos}},
        {"fn", function()
            if callback then
                callback()
            end
        end},
    })
end

-- 离开
function M:leave()
    self:line({
        {"moveTo", .3, cc.p(-100, self._pos.y)},
    })
end

-- 加载数字框
function M:loadNumberFrame()
    if self._isGather then
        local frame = D.img("gather/number.png"):to(self, 5):p(0, self._starPercent[1] * self:ch()/100):anchor(cc.p(0, .5))
        self._frame = frame
    end
end

-- 数字框提示
function M:frameTip()
    if not tolua.isnull(self._frame) then
        self._frame:line({
            A.cycle({
                {"scaleBy", .2, 1.2},
                {"scaleBy", .2, 1/1.2},
            }, 2)
        })
    end
end

-- 数字框上移
function M:frameMoveUp(percent, num)
    if tolua.isnull(self._frame) then return end
    self._frame:stopAllActions()
    self._frame:line({
        {"moveTo", .5, cc.p(0, percent * self:ch())},
        {"fn", function()
            self._frame._number:setValue(num)       
        end},
    })
end

-- 加载闹钟
function M:loadClock()
    local clock = D.img("common/progressbar/clock.png"):to(self._layer, 999):pc():hide()
    self._clock = clock
end

-- 展示闹钟
function M:showClock()
    self._isShow = true
    self._clock:line({
        {"show"},
        {"scaleBy", .5, 1.1},
        {"scaleBy", .5, 1/1.1},
        {"fn", function()
            if not tolua.isnull(self) and not tolua.isnull(self._clock) then
                CommonFun:resetParent(self._clock, self, 50)
                self._clock:line({
                    {"jumpTo", .5, cc.p(self:cw()/2, -25), 100, 1},
                    {"scaleBy", .15, .95},
                    {"scaleBy", .15, 1/.95},
                })
            end
        end},
    })
end

-- 加载倒计时数字
function M:loadNumber()
    local number = U.loadNumLinear({
           value = self._totalTime,
          imgPre = "common/ui/number/",
          margin = 0,
       itemWidth = 46/2,
      itemHeight = 50/2
    }):p(self._clock:cw()/2 - 2, 40):to(self._clock, 5):scale(.85):color(ccc3(109, 29, 28))
    self._number = number
end

-- 更新进度条
function M:updateProgressBars()
    if self._isFinish then return end
    if self._layer._isShootOrPass or self._layer._player._isShootOrPass then return end -- 停止进度条更新

    self._time = self._time - 1
    self:updateNumber(self._time)
    if self._time <= 15 and not self._isLast then       -- 最后15s红框闪烁提示
        self._isLast = true
        self:warning()
        self:clockShakeAction()
        if self._layer["lastTime"] then
            self._layer:lastTime()
        end
    elseif self._time <= 12 and not self._isWarning then
        self._isWarning = true
        soundVoice:stopNowSound()
        soundVoice:playEffectV2622009()
    elseif self._time < 12 and math.fmod(self._time, 3) == 0 then
        if self._time ~= 0 then
            self:numberScaleAction() 
        end
        
        if self._time <= 0 and not self._isFinish then       -- 游戏结束
            -- 进度条结束事件
            self:countDownEndEvent()
        end
    end
end

-- 进度条结束事件
function M:countDownEndEvent()
	local layer     = self._layer -- 层
    self._isFinish = true
    -- 停止闹钟动作
    self:stopClockAct()
    self:updateNumber(0)
    local player    = layer._player 
    layer._canTouch = false    -- 关闭触控
    -- 角色失败事件
    self:roleFailEvent()
end

-- 角色失败事件
function M:roleFailEvent()
    local layer     = self._layer -- 层
    local player    = layer._player 
    local state = player._fsm:getState() -- 状态机状态
    if state == "fly" or state == "special" then
        self:plan(function()
            -- 角色失败事件
            self:roleFailEvent()
        end, 0.5)
    return end
    -- 强行停止
    player:forceStop()
    player._body:SetLinearDamping(10000)
    player:playSad()
    if layer and isfunction(layer.gameFinish) then
        layer:gameFinish(4)
    end
end

-- 停止闹钟动作
function M:stopClockAct()
    self:closeSC()
    self:stopWarningAction()
    soundVoice:stopNowSound()
    soundVoice:playEffectV2622011()
    A.line({
        {"moveBy", 0.4, cc.p(-100, 0)},
        {"hide"}
        -- {"remove"},
    }):at(self)
end

-- 更新倒计时数字
function M:updateNumber(time)
    if not tolua.isnull(self._number) then
        self._number:setValue(time) 
    end
end

-- 数字缩放动作
function M:numberScaleAction()
    if tolua.isnull(self._number) then return end
    self._number:line({
        A.cycle({
            {"scaleBy", .2, 1.4},
            {"scaleBy", .2, 1/1.4},
        }, 2)
    })
end

-- 闹钟晃动动作
function M:clockShakeAction()
    if tolua.isnull(self._clock) or self._shakeAction or self._effectAction or self._isFinish then return end
    local interval = .04
    self._shakeAction = A.cycle({
        {"rotateTo", interval, -7},
        {"rotateTo", interval, 0},
        {"rotateTo", interval, 7},
        {"rotateTo", interval, 0},
    }):at(self._clock)
    self._effectAction = A.cycle({
        {"fn", function()
            local layer = self._layer -- 层
            if not layer._player._isShootOrPass then
                self._clock._effect = soundEffect:playEffectSfx26200032() 
            end
        end},
        {"delay", 2.5},
    }):at(self._clock)
end

-- 停止闹钟晃动动作
function M:stopShakeAction()
    if not tolua.isnull(self._clock) then
        self._shakeAction  = CommonFun:stopAction(self._clock, self._shakeAction)
        self._effectAction = CommonFun:stopAction(self._clock, self._effectAction)
        self._clock:rotate(0)
        sound.stopSound(self._clock._effect) 
    end
end

-- 倒计时15s红色闪烁
function M:warning()
    if self._warningAction then return end
    self._warningEffect = soundEffect:playEffectsfx26236009()
    local warning = D.img("common/warning.png"):to(self._layer, 100):pc():opacity(0)
    warning:scaleY(V.h / warning:ch() * 1.1)
    warning:scaleX(V.w / warning:cw() * 1.1)
    self._warning = warning
    self._warningAction = A.cycle({
        {"fadeTo", 0.5, 255},
        {"fadeTo", 0.5, 50},
    })
    self._warning:runAction(self._warningAction)
end

-- 停止倒计时闪烁
function M:stopWarningAction()
    if self._warningAction then
        sound.stopSound(self._warningEffect)
        self._warning:stopAction(self._warningAction)
        self._warningAction = nil
        self._warning:line({
            {"fadeTo", .5, 0},
            {"remove"},
        })
    end
end

-- 开启调度器
function M:openSC()
    self:closeSC()
    if self._time <= 15 then
       self:warning()
       self:clockShakeAction()
    end
    self.scUpdate = SC.open(handler(self, self.updateProgressBars), 1) 
 end
 
 -- 关闭调度器
 function M:closeSC()
    self:stopWarningAction()
    self:stopShakeAction()
    if self.scUpdate then 
        SC.close(self.scUpdate)
        self.scUpdate = nil 
    end
 end

----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()
    self:closeSC()
end


----------------------
-- 点击
----------------------
--[[--

判断是否点击到内容区域

### Example:

### Parameters:
-   string **event**    事件名称
-   number **x**        点击位置x(相对于屏幕)
-   number **y**        点击位置y(相对于屏幕)

]]
function M:onTouchHandled(event, x, y)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end


return M