--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    冰人
   Author:  Neighbour Mr.Wang
   Date:    2021.08.06
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------

local M     = classNode("IceMan")
M.TAG       = "IceMan"

----------------------
-- 公共参数
----------------------

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]

function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    -- 冰人类型
    self._iceManType = params.iceManType
    -- 位置类型（1起点2终点）
    self._posType    = params.posType
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]

function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    
    -- 图片渲染
    self:loadDisplay()
end

------------------------------------------------------------------------------------------------
-- 功能函数
------------------------------------------------------------------------------------------------
-- 图片渲染
function M:loadDisplay()
    self:setContentSize(cc.size(50, 50))

    local offsetY = self._iceManType == 1 and 0 or -4
    -- 头
    local head = D.img("biathlon/iceman/"..self._iceManType.."/head.png"):p(25, 52 + offsetY):to(self, 2):anchor(cc.p(0.5, 0))
    -- 脖子
    local neck = D.img("biathlon/iceman/neck.png"):p(24, 42 + offsetY):to(self, 0):anchor(cc.p(0.5, 0))
    -- 身体
    local body = D.img("biathlon/iceman/"..self._iceManType.."/body.png"):p(25, 0):to(self, -1):anchor(cc.p(0.5, 0))
    -- 加载左手
    self:loadLeftHand()
    -- 加载右手
    self:loadRightHand()

    A.cycle({
        {"fn", function()
            A.union({
                {"jumpBy", 0.3, cc.p(0, 0), 14, 1},
                {"scaleTo", 0.3, 0.85, 1.1},
            }):at(neck)
        end},
        {"jumpBy", 0.3, cc.p(0, 0), 25, 1},
        {"scaleTo", 0.1, 1.1, 0.9},
        {"fn", function()
            A.union({
                {"jumpBy", 0.3, cc.p(0, 0), 8, 1},
                {"scaleTo", 0.3, 1},
            }):at(neck)
        end},
        {"union",{
            {"scaleTo", 0.1, 0.9, 1.1},
            {"jumpBy", 0.3, cc.p(0, 0), 20, 1},
        }},
        {"scaleTo", 0.1, 1},
        {"delay", 0.7},
    }):at(head)
end

-- 加载左手
function M:loadLeftHand()
    -- 手信息（位置， 锚点）
    local handInfo = {
        {{cc.p(-6, 41), cc.p(1, 0)}, {cc.p(-7, 41), cc.p(1, 0)}},
        {{cc.p(13, 51), cc.p(1, 1)}, {cc.p(9, 78), cc.p(1, 0)}},
    }
    local info = handInfo[self._posType][self._iceManType]
    -- 左手
    local leftHand = D.img("biathlon/iceman/"..self._iceManType.."/lefthand"..self._posType..".png")
    :p(info[1]):to(self, 0):anchor(info[2])
    if self._posType == 1 then
        A.cycle({
            {"rotateBy", 0.3, -10},
            {"rotateBy", 0.3, 10},
        }):at(leftHand)

        local flagInfo = {
            {cc.p(21, 57), -23},
            {cc.p(31, 57), -10},
        }
        -- 加载旗子
        self:loadFlag(leftHand, flagInfo[self._iceManType][1], flagInfo[self._iceManType][2])
    end
end

-- 加载右手
function M:loadRightHand()
    local rightPos = self._iceManType == 1 and cc.p(54, 48) or cc.p(53, 45)
    -- 右手
    local righthand = D.img("biathlon/iceman/"..self._iceManType.."/righthand.png")
    :p(rightPos):to(self, 0):anchor(cc.p(0, 0))
    A.cycle({
        {"rotateBy", 0.3, 10},
        {"rotateBy", 0.3, -10},
    }):at(righthand)

    local flagInfo = {
        {cc.p(50, 46), 22},
        {cc.p(28, 56), 0},
    }
    -- 加载旗子
    self:loadFlag(righthand, flagInfo[self._iceManType][1], flagInfo[self._iceManType][2])
end

-- 加载旗子
function M:loadFlag(parent, pos, angle)
   -- 旗子
   local flag = D.img("biathlon/iceman/flag/1.png"):to(parent):p(pos):rotate(angle)
   A.cycle({
        {"image", "biathlon/iceman/flag/", 4, 0.05},
    }):at(flag)
end
----------------------
-- 结点析构
----------------------

function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]

function M:onExitTransitionStart()

end

----------------------
-- 验证
----------------------

-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end

return M
