--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    引导用地图管理类
   Author:  Neighbour Mr.Wang
   Date:    2021.08.04
   
   http://www.baby-bus.com/
   
]]
----------------------
-- 类
----------------------
local M     = class("MapManagerGuide", require("app.biathlon.node.MapManager"))
M.TAG       = "MapManagerGuide"

----------------------
-- 公共参数
----------------------
-- [常量]

-- 需要控制的一圈地图相对标识
local ControlMaps = {
    cc.p(0, 1),
    cc.p(1, 1),
    cc.p(1, 0),
    cc.p(1, -1),
    cc.p(0, -1),
    cc.p(-1, -1),
    cc.p(-1, 0),
    cc.p(-1, 1), 
}

local Config = require("app.biathlon.Config")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    self:setContentSize(self._mapSizeTotal)
    -- 加载地图
    self:loadMaps()
    -- 加载物理世界
    self:loadPhysicsWorld()
    -- 加载角色
    self:loadRole()
    -- 初始化地图位置
    self:judgeMap(self._player)
    -- 加载道路背景
    self:loadWayBg()
    -- 加载背景装饰
    self:loadBgDecorate()
    -- 引导流程
    self:guideFlow()
end

-- 引导流程
function M:guideFlow()
    -- 加载起点
    self:loadStartPoint()
    -- 加载打靶地点
    self:loadShootSite()
    -- 加载路障
    self:loadRoadblock()
    -- 加载赛道管理器(障碍物, 道具的加载)
    self:loadTrackManager()
    -- 加载道具
    self:loadProp()
    -- 加载加速节点
    self:loadSpeedNode()

    -- 道路背景
    local wayBg = D.img("biathlon/ways/guide.png"):p(cc.p(22744, 472.8)):to(self, 100):anchor(cc.p(0, 0)):scale(1.5492)--:color(cc.p(255, 0, 0))
    local fenceList = {
        cc.p(23440, 552.47), cc.p(23140.44, 549.79), cc.p(23121.42, 1077.97), cc.p(23424.05, 1076.34)
    }
    for _,pos in ipairs(fenceList) do
        local fence = D.img("biathlon/decorate/fence/1.png"):p(pos):to(self, 120)
    end
end

-- 加载背景装饰
function M:loadBgDecorate()
    -- 加载栅栏1
    self:loadFences(Config.ordinaryFencePosList, 1)
    -- 加载栅栏3
    self:loadFences(Config.specialFencePosList, 3)
    -- 加载树
    self:loadTrees()
    -- 加载冰山
    self:loadMountains()
    -- 加载靶场
    self:loadRange()
    -- 加载帐篷
    self:loadTent()
end

-- 加载道具
function M:loadProp()
    local pos  = cc.p(23885, 611)
    local offY = 60

    local realPos = cc.p(pos.x, pos.y + offY) -- 实际的位置

    -- 道具盒子
    local propbox = import("app.biathlon.node.PropBox").new({
        enterPos   = cc.p(500 + X_OFFSET, 50),
        offY       = offY,
        mapManager = self,
        types      = 3, --  1香蕉皮 2 红烧章鱼 3彩虹 4火箭
        allowTypes = {2, 3}     -- 允许出现的道具类型
    }):p(realPos.x, realPos.y-20)
    :to(self)

    -- 获得离地面高度及地面斜率
    local dis,k = self:getHighAndSlope(self, realPos.x, realPos.y)
    local angle = -math.deg(math.atan(k))
    propbox:rotate(angle)

    self._propbox = propbox
end

-- 加载起点
function M:loadStartPoint()
    -- 加载冰人
    self:loadIceMan(1, 1, cc.p(23600, 1022.47), self)
    self:loadIceMan(2, 1, cc.p(23600, 533), self)
end

-- 加载冰人
function M:loadIceMan(iceManType, posType, pos, parent)
    -- 冰人
    local iceMan = import("app.biathlon.node.IceMan").new({
        iceManType = iceManType,
        posType    = posType,
    }):to(parent, 1000):p(pos)
end

-- 第二次打靶开始
function M:secondShootStart()
    local layer = self._layer -- 层
    layer._guideState = layer._guideState + 1
    layer._handGuide = CommonFun.remove(layer._handGuide)
    -- 加载裁判
    layer:loadReferee()
end

-- 进入打靶
function M:enterShoot()
    local pos   = self._shootNodeList[2]._worldPos
    CommonFun:changeAnchorP(self._shootSiteList[2], cc.p(0.5, 1))
    -- 打靶提示弹动
    self:shootHintSpring(self._shootSiteList[2])

    A.line({ --  镜头移到靶场
        {"delay", 0.3},
        {"easing","sineout",{"moveTo", 1.2, cc.p(-pos.x + V.cw, -pos.y + V.ch)}},
        {"fn", function()
            -- 过渡并加载打靶弹窗
            self:transitionAndLoadShootPop(1)
        end},
    }):at(self)
end

--------------------------------------------
-- 碰撞
--------------------------------------------
-- 碰撞判断
function M:crashCheck()
    -- 靶场检测
    self:shootSiteCheck()
    -- 道具障碍碰撞检测
    self:propBarrierCrashCheck()
    -- 引导道具箱检测
    self:guidePropBoxCheck()
end

-- 引导道具箱检测
function M:guidePropBoxCheck()
    -- 角色
    local role = self._player
    if not tolua.isnull(self._propbox) then
        if role:px() >= self._propbox:px() then
            if not self._hasPropGuideSnd then
                self._hasPropGuideSnd = true
                -- 播放配音[v262330]是道具箱，看看是什么道具吧
                soundVoice:playEffectv262330()
            end
            role._body:SetLinearVelocity(b2Vec2(0, 0))
            role._curMaxSpeed = 0 -- 不让向前移动
        end
    elseif not tolua.isnull(self._layer._handGuide) then
        self._layer._handGuide:hide()
    end
end

----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()

end

return M