--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    地图上生成的道具
   Author:  Neighbour Mr.Wang
   Date:    2021.08.06
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------

local M     = classSprite("MapProp")
M.TAG       = "MapProp"

----------------------
-- 公共参数
----------------------
-- [常量]

local Config = require("app.biathlon.Config")
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]

function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
  
    -- y偏移量
    self._offY       = params.offY
    -- 道具名称
    self._propName   = nil-- params.propName
    -- 地图管理器
    self._mapManager = params.mapManager
    -- 角色
    self._role       = params.role
    -- 高度限制(在指定高度内才能触发碰到)
    self._highLimit  = 50
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]

function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

end

------------------------------------------------------------------------------------------------
-- 功能函数
------------------------------------------------------------------------------------------------
-- 加载光
function M:loadLight()
	-- 光
    local light = D.img("biathlon/props/light.png"):p(self:cw()/2, self:ch()/2):to(self, -5)
    A.cycle({
        {"rotateBy", 1, 360},
    }):at(light)

    self._light = light
end

-- 移除光
function M:removeLight()
    if tolua.isnull(self._light) then return end
	A.line({
        {"union",{
            {"scaleTo", 0.3, 1.3},
            {"fadeTo", 0.3, 100},
		}},
        {"remove"},
    }):at(self._light)
end

-- 出现动作
function M:enter(cb)
    A.line({
        {"union",{
            {"jumpBy", 0.15, cc.p(0, 100), 50, 1},
            {"scaleTo", 0.15, 1},
		}},
        {"fn", function()
            A.line({
                {"easing","sineout",{"scaleTo", 0.05, 1.3}},
                {"easing","sinein",{"scaleTo", 0.05, 1}},
            }):at(self._light)
        end},
        {"delay", 0.1},
        {"fn", function()
            -- 道具效果
            self:propEffect()
            if isfunction(cb) then
                cb()
            end
        end},
    }):at(self)
end

-- 道具效果
function M:propEffect()
	if self._propName == "rainbow" or self._propName == "rocket" then -- 彩虹/火箭
        CommonFun:resetParent(self, self._role, 1000)
        A.line({
            {"union",{
                {"scaleTo", 0.2, 1.3},
                {"fadeTo", 0.2, 50},
                {"moveBy", 0.2, cc.p(-20, -20)},
            }},
            {"fn", function()
                local role = self._role
                if self._propName == "rocket" then
                    -- 火箭效果
                    self:rocketEffect(role)
                    if role._isPlayer then
                        -- 播放配音[v262332]坐火箭的时候
                        soundVoice:playEffectv262332()
                        -- 播放音效坐火箭时角色的笑声
                        soundEffect["playEffectsfx2623302"..N.random(4, 5)](soundEffect)
                        -- 停止玩家滑雪音效
                        role:stopPlayerSkiSnd()
                        -- 停止正常滑雪音效(无操作)
                        role:stopSkiSnd()
                        -- 播放音效[sfx26233014]坐火箭
                        soundEffect:playEffectsfx26233014()
                    end
                elseif self._propName == "rainbow" then
                    -- 彩虹效果
                    self:rainbowEffect(role)
                end
            end},
            {"remove"},
        }):at(self)
    else
        -- 减益道具效果
        self:debuffPropEffect(self._propName)
    end
end

-- 减益道具效果
function M:debuffPropEffect()
	local offsetY = self._role._relBodyOffsetY
    CommonFun:resetParent(self, self._mapManager, 4990 - offsetY*10)
    local endX = self:px()-120
    -- 获得离地面高度及地面斜率
    local dis, k  = self._mapManager:getHighAndSlope(self, endX, self:py())
    local config   = require("app.biathlon.Config")
    local nextPos = config.edgePoints[self._edgePosId+1]
    local endPos  = cc.pSub(nextPos, cc.p(nextPos.x - endX, (nextPos.x - endX)*k))
    -- 移除光
    self:removeLight()
    A.line({
        {"union",{
            {"scaleTo", 0.35, 1},
            {"jumpTo", 0.35, cc.p(endPos.x, endPos.y + offsetY), 80, 1},
        }},
        {"fn", function()
            table.insert(self._mapManager._propBarrierList, self)
        end},
    }):at(self)
end

-- 被碰撞事件
function M:beCrashEvent(role)
    T.removeOrder(self._mapManager._propBarrierList, self)
    if role._isPlayer and self._mapManager._layer._isFinish then return end -- 主角失败导致游戏结束时不走

    if self._propName == "bananapeel" then  -- 香蕉皮
        if role._isPlayer or role:canPlaySnd() then
            -- 播放音效[sfx26233020]撞到香蕉皮
            soundEffect:playEffectsfx26233020()
        end
        role._fsm:doEvent("toSpecial2")-- 获得特殊状态
        role._body:SetLinearDamping(6)

        role._body:SetLinearVelocity(b2Vec2(0, 0))
        -- 踩到香蕉皮-准备
        role:playBananaBegin()
        role:plan(function()
            if role._fsm:getState() == "special2" then
                -- 踩到香蕉皮-结束
                role:playBananaEnd()
            end
            role._body:SetLinearDamping(0)
        end, 1.2)
    elseif self._propName == "octopus" then -- 红烧章鱼
        if role._isPlayer or role:canPlaySnd() then
            -- 播放音效[sfx26233018]撞到章鱼
            soundEffect:playEffectsfx26233018()
        end
        -- 红烧章鱼效果
        self:octopusEffect(role)
    end
    CommonFun.remove(self)
end

-- 红烧章鱼效果
function M:octopusEffect(role)
	role._fsm:doEvent("toSpecial2")-- 获得特殊状态
    -- 玩具遮脸-准备
    role:playToyBegin()
    local vec = role._body:GetLinearVelocity()
    role._body:SetLinearVelocity(b2Vec2(vec.x/3, vec.y/3))
    local mapManager = self._mapManager
    mapManager:getTrackManager():changeTrack(role, N.randomExcept(-1, 1, 0)) -- 随机换赛道
    if role._isPlayer then
        -- 播放音效失落
        soundEffect["playEffectsfx2623000"..N.random(5, 6)](soundEffect)
    end
    A.line({
        {"delay", 0.7},
        {"fn", function()
            mapManager:getTrackManager():changeTrack(role, N.randomExcept(-1, 1, 0)) -- 随机换赛道
        end},
        {"delay", 0.7},
        {"fn", function()
            if role._fsm:getState() == "special2" then
                -- 玩具遮脸-结束
                role:playToyEnd()
            end
        end},
    }):at(role)
end

-- 火箭效果
function M:rocketEffect(role)
    role._fsm:doEvent("toSpecial")-- 获得特殊状态
    -- 火箭滑板-准备
    role:playRocketBegin()
    role._ignoreControl = true -- 忽略控制

    if role._rocketAct then -- 停止火箭动作
        role:stopAction(role._rocketAct)
    end
    -- 火箭动作效果
    self:rocketEffectAct(role)
    if role._isPlayer then
        A.line({
            {"easing","sinein",{"scaleTo", 3, 1.1}},--镜头推近
        }):at(self._mapManager)
    end
end

-- 火箭动作效果
function M:rocketEffectAct(role)
    local vX = 40               -- x方向速度
    local vY = 60 + (13 - role:py()/50)-- y方向速度
    -- 粒子
    local particle = P.newParticle("particle/biathlon/huojiantuowei.plist", 0, 50):to(role, -2)
    role._rocketAct = A.cycle({
        {"fn", function()
            role._body:SetLinearVelocity(b2Vec2(vX, vY))
            vX = vX - 1
            if vY > 0 then
                vY = vY - 10
                vY = vY <= 0 and 0 or vY
            end
            -- 空中上下浮动
            role:floatUpDownWithFly(vX)
            if vX <= role._maxSpeed+role._extraMaxSpeed then -- 加速结束
                particle:stopSystem()
                particle:plan(function()
                    particle:remove()
                end, 0.3)
                if role._isPlayer then
                    A.line({
                        {"easing","sineout",{"scaleTo", 1, 1}}, -- 镜头拉远
                    }):at(role._mapManager)
                end
                role:stopAction(role._rocketAct)
                role._rocketAct  = nil
                role._extraSpeed = role._extraMaxSpeed  -- 设置额外速度到最大
                role._floorDis   = 0 -- 让他可以飞
                role._fsm:doEvent("toNormal")-- 正常状态
                -- 道具引起的飞行状态
                role._propFlyState  = 1 
            end
        end},
        {"delay", 1/10},
    }):at(role)
end

-- 彩虹效果
function M:rainbowEffect(role)
    role._fsm:doEvent("toSpecial")-- 获得特殊状态(可能在空中等, 需要等一会再进入彩虹效果)
    role:playRainbowTrail()
    -- 播放正常滑雪音效(无操作)
    role:playSkiSnd()
    if role._fsm:getState() == "fly" then
        local vec = role._body:GetLinearVelocity()
        role._body:SetLinearVelocity(b2Vec2(vec.x, vec.y-6))
    end

    A.line({
        {"delay", 0.12},
        {"fn", function()
            role._fsm:doEvent("toGetrainbow")-- 获得彩虹状态
            -- 播放正常滑雪音效(无操作)
            role:playSkiSnd()
            local vec = role._body:GetLinearVelocity()
            role._body:SetLinearVelocity(b2Vec2(vec.x + 8, vec.y*2))
        end},
        {"delay", 1.8},
        {"fn", function()
            if role._fsm:getState() == "getrainbow" then
                role._fsm:doEvent("toSki")-- 滑行
            end
        end},
    }):at(role)
end

----------------------
-- 结点析构
----------------------

function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]

function M:onExitTransitionStart()

end

----------------------
-- 验证
----------------------

-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end

return M
