--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    道具箱
   Author:  Neighbour Mr.Wang
   Date:    2021.08.06
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------

local M     = classSprite("PropBox")
M.TAG       = "PropBox"

----------------------
-- 公共参数
----------------------
-- [常量]
-- 道具名字
local propName = {
    "bananapeel",  -- 香蕉皮
    "octopus",  -- 红烧章鱼
    "rainbow",  -- 彩虹
    "rocket",  -- 火箭
}


----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]

function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    -- y偏移量
    self._offY       = params.offY
    -- 地图管理器
    self._mapManager = params.mapManager
    -- 高度限制(在指定高度内才能触发碰到)
    self._highLimit  = 100
    -- 道具类型
    self._types      = params.types
    self._allowTypes = params.allowTypes     -- 允许出现的道具类型
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]

function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

    -- 图片渲染
    self:loadDisplay()
    table.insert(self._mapManager._propBarrierList, self)
end

------------------------------------------------------------------------------------------------
-- 功能函数
------------------------------------------------------------------------------------------------
-- 图片渲染
function M:loadDisplay()
    self:display("biathlon/props/box/close.png"):z(4999 - self._offY*10):anchor(cc.p(0.5, 0.2)):scale(0.9)--:opacity(0)
end

-- 被碰撞事件
function M:beCrashEvent(role)
    if role._isPlayer or role:isInScreen() then
        -- 播放音效[sfx26233007]撞到道具箱子
        soundEffect:playEffectsfx26233007()
    end

    T.removeOrder(self._mapManager._propBarrierList, self)
    local particle = P.newParticle("particle/biathlon/chidaoju.plist", 120, 120):to(role, 1)
    -- 获得的道具去重
    if not role._lastPropType then -- 角色最后一次吃到的道具类型
        role._lastPropType = self._types
    elseif role._lastPropType == self._types then
        for i,types in pairs(self._allowTypes) do
            if types ~= self._types then
                self._types = types
                role._lastPropType = types
                break
            end
        end
    end
    -- 碰到道具反馈
    role:playPropEffect()
    self:hide():opacity(0)
    -- 加载角色身上的箱子
    self:loadRoleBox(role)
end

-- 加载角色身上的箱子
function M:loadRoleBox(role)
	local sc  = 0.7
    -- 箱子
    local box = D.img("biathlon/props/box/close.png"):p(150, 75):to(role, 50):scale(sc):anchor(cc.p(0.5, 0))
    -- 箱子打开
    self:boxOpen(box, role)
    A.line({
        {"union", {
            {"scaleTo", 0.12, sc+0.1, sc-0.05},
            {"moveBy", 0.12, cc.p(0, -1)},
        }},
        {"union", {
            {"scaleTo", 0.12, sc-0.05, sc+0.1},
            {"moveBy", 0.12, cc.p(0, 1)},
        }},
        {"scaleTo", 0.12, sc},
    }):at(box)
end

-- 箱子打开
function M:boxOpen(box, role)
    if self._hasOpen then return end
    self._hasOpen = true
    
    box:display("biathlon/props/box/open_bottom.png")
    -- 展示道具
    self:showProp(box, role)
    -- 光
    local light = D.img("biathlon/props/box/light.png"):p(44, 20):to(box, 50):anchor(cc.p(0.5, 0)):opacity(100):scale(1.03)
    A.one({"fadeIn", 0.3}):at(light)
    local front = D.img("biathlon/props/box/open_front.png"):p(46, 1):to(box, 50):anchor(cc.p(0.5, 0))
    -- 光粒子
    local lightPar = D.img("biathlon/props/box/light_frames/1.png"):p(49, 45):to(box, 50):anchor(cc.p(0.5, 0))
    A.line({
        {"image", "biathlon/props/box/light_frames/", 5, 0.08},
        {"fadeOut", 0.1},
        {"remove"},
    }):at(lightPar)
end

-- 展示道具
function M:showProp(box, role)
    local propName = propName[self._types]
	-- 道具
    local prop = import("app.biathlon.node.MapProp").new({
        role       = role,
        mapManager = self._mapManager,
        offY       = self._offY,
        imgName    = "biathlon/props/"..propName..".png"
    }):p(49, -20):to(box, 50):scale(0.2):anchor(cc.p(0.5, 0.1))

    prop._propName = propName
    -- 加载光
    prop:loadLight()

    -- 出现动作
    prop:enter(function ()
        A.line({
            {"union",{
                {"fadeTo", 0.15, 120},
                {"scaleTo", 0.15, 0.1},
                {"moveBy", 0.15, cc.p(-40, 60)},
            }},
            {"fn", function()
                CommonFun.remove(self)
            end},
            {"remove"},
        }):at(box)
    end)
end

----------------------
-- 结点析构
----------------------

function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]

function M:onExitTransitionStart()

end

----------------------
-- 验证
----------------------

-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end

return M
