--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    ai角色
   Author:  Neighbour Mr.Wang
   Date:    2021.08.05
   
   http://www.baby-bus.com/
   
]]
----------------------
-- 类
----------------------
local M     = class("RoleAI", require("app.biathlon.node.RoleBaseWithFsm"))


----------------------
-- 公共参数
----------------------



----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    -- 是否正在打靶
    self._isShootOrPass = false
    -- 是ai
    self._isAI          = true
    -- 当前速度比率
    self._curSpeedRatio = 1 
    -- 最大速度
    self._maxSpeed      = (self._maxSpeed + self._extraMaxSpeed/2)*N.random(90, 100)*0.01 -- 速度不一样
    -- 当前最大速度
    self._curMaxSpeed   = self._maxSpeed
    -- 打靶需要消耗的时间
    self._shootTime     = ST.getNumber("BIATHLON_SHOOT_ELAPSEDTIME", 10)  -- 获得角色打靶时间
end


----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

    -- 加载位置提示
    self:loadPosTip()
end

-- 加载位置提示
function M:loadPosTip()
    -- 位置提示
    local posTip  = D.img("biathlon/pos_tips/"..self._id..".png"):to(self._layer, 50):hide():opacity(0)
    -- 小箭头
    local arrow   = D.img("biathlon/pos_tips/"..self._id.."_1.png"):p(posTip:cw()/2, posTip:ch()/2):to(posTip, 50):anchor(cc.p(0.5, 0.35))
    -- 人物小头像
    local tipRole = D.img("biathlon/pos_tips/"..self._id..".png"):to(arrow, 50):p(posTip:cw()/2, posTip:ch()/2)

    posTip._arrow = arrow
    self._posTip  = posTip
    self._tipRole = tipRole
end

-- 角色位置提示
function M:rolePosTips()
    -- 是否在屏幕里
    local inScreen, pos = self:isInScreen()
    local player = self._layer._player

    if inScreen or player._isShootOrPass then
        if self._posTip then
            self._posTip:hide()
        end
        return
    end

    pos         = cc.p(CommonFun:limitRect(pos.x, pos.y, cc.rect(80, 80, V.w - 160, V.h - 160)))
    local angle = CommonFun.getAngle(cc.p(V.cw, V.ch), pos)

    -- 位置提示
    self._posTip:p(pos):show()
    self._posTip._arrow:rotate(angle)
    self._tipRole:rotate(-angle)

    if not self._guideAct and (player._fsm:getState() == "stop" or player._fsm:getState() == "ready")  then -- 角色停止时引导
       -- 执行引导动作
       self:doGuideAct()
    end
end

-- 执行引导动作
function M:doGuideAct()
    self._guideAct = A.line({
        {"fn", function()
            A.one({"scaleTo", 0.3, 0.95}):at(self._posTip._arrow)
        end},
        {"easing","sinein",{"moveBy", 0.15, cc.p(0, -30)}},
        {"easing","sineout",{"moveBy", 0.15, cc.p(0, 30)}},
        {"fn", function()
            A.one({"scaleTo", 0.26, 1}):at(self._posTip._arrow)
        end},
        {"easing","sinein",{"moveBy", 0.13, cc.p(0, -20)}},
        {"easing","sineout",{"moveBy", 0.13, cc.p(0, 20)}},
        -- {"easing","sinein",{"moveBy", 0.07, cc.p(0, -10)}},
        -- {"easing","sineout",{"moveBy", 0.07, cc.p(0, 10)}},
        {"delay", 2},
        {"fn", function()
            self._guideAct = nil
        end},
    }):at(self._tipRole)
end

-- 执行滑行逻辑
function M:doSkiingLogic()
    if self._hasDoSkiLogic then return end
    self._hasDoSkiLogic = true
    -- 滑行逻辑
    self:skiingLogic()
end

-- 改变速度倍率
function M:changeSpeedRatio()
    local disX = self._layer._player:px() - self:px() -- 与玩家的距离
    
    if disX > 100 then
        local ratio = disX/2000 + N.random(10, 11)*0.1
        ratio       = ratio >= 1.5 and 1.5 or ratio
        self._curSpeedRatio = ratio
    elseif disX < 50 then
        local ratio         = 1 - N.random(-250, 0)*0.001 -- 随机减速
        self._curSpeedRatio = ratio
    else
        self._curSpeedRatio = 1
    end
end

-- 滑行逻辑
function M:skiingLogic()
    if self._isShootOrPass then return end

    local randomN = math.random(self._curMaxSpeed/1.5, self._curMaxSpeed)
    -- 改变速度倍率
    self:changeSpeedRatio()
    self._curMaxSpeed = self._curSpeedRatio*self._maxSpeed
    -- 施加滑行速度
    local result = self:applySkiSpeed(randomN*self._curSpeedRatio)
 
    if result then
        self._fsm:doEvent("toSpeedup")-- 加速
    end

    self._skiAct = A.line({
        {"delay", randomN*0.05},
        {"fn", function()
            self:skiingLogic()
            self._skiAct = nil
        end},
    }):at(self)
end

-- 尝试结束打靶(发现玩家早已经打完了, 提前一点结束打靶)
function M:tryShootEnd()
    local layer = self._layer -- 层
    if tolua.isnull(layer) then return end

    local player = layer._player
	if player._gameStage >= self._gameStage and not layer._isShoot then
        self:stopAction(self._shootAct)
        -- 打靶结束事件
        self:shootEndEvent()
    end
end

-- 打靶结束事件
function M:shootEndEvent()
    self._isShootOrPass = false
    
    local result     = self:applySkiSpeed(5)
    if result then
        self._fsm:doEvent("toSpeedup")-- 滑行
    end
    -- 滑行逻辑
    self:skiingLogic()
    -- 打靶成功事件
    self:shootSuccessEvent()
end

-- 打靶计时
function M:shootTimeCount()
    local shootTime = self._shootTime + N.random(-5, 5) + 5 -- 增加5秒适应角色进靶场的镜头切换及白屏时间
    shootTime = shootTime < 10 and 10 or shootTime
    shootTime = shootTime > 50 and 50 or shootTime
	self._shootAct = A.line({
        {"delay", shootTime/2},
        {"fn", function()
            -- 尝试结束打靶
            self:tryShootEnd()
        end},
        {"delay", shootTime/2},
        {"fn", function()
            -- 打靶结束事件
            self:shootEndEvent()
        end},
    }):at(self)
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end








return M