--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    角色基类
   Author:  Neighbour Mr.Wang
   Date:    2021.08.05
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------
local M     = class("RoleBase", require("app.common.nodebase.ArmatureBase"))


----------------------
-- 公共参数
----------------------

local Config = require("app.biathlon.Config")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 传参数[层]
    self._layer          = params.layer
    self._id             = params.id or 0
    -- 旋转偏移量
    self._rotateOffset   = 0
    self._name           = params.name
    -- 角色类型
    self._roleType       = params.roleType or 1
    -- 名次
    self._rank           = 1
end


----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

    self:initChange()
    self._armature:rotate(self._rotateOffset)
end

-- 初始换装
function M:initChange()
    local info = Config.animalChangeInfo1
    if self._name == "hx1_" then
        info = Config.animalChangeInfo2

        if self._roleType ~= 1 then
            -- 骨骼帧动画换装
            self:changeBonesWithFile()
        end
    end
    for i, v in ipairs(info) do
        local f = v["f"..self._roleType]
        if f == "" then
            self:hideBone(v.b)
        elseif f then
            self:change(v.b, f)
        end
    end
end

-- 骨骼帧动画换装
function M:changeBonesWithFile()
    -- 换装信息(骨骼名, 纹理集合)
    local changesInfo = {
        ["hx1_zui"]   = {"hx1_zui1.png", "hx1_zui1_1.png", "hx1_zui1_2.png", "hx1_zui1_3.png", "hx1_zui1_4.png"}, -- 嘴
        ["hx1_zyk"]   = {"hx1_zyk.png", "hx1_zyk1_1.png", "hx1_zyk1_2.png", "hx1_zyk1_3.png", "hx1_zyk1_4.png"},  -- 左眼眶
        ["hx1_yyk"]   = {"hx1_yyk.png", "hx1_yyk1_1.png", "hx1_yyk1_2.png", "hx1_yyk1_3.png", "hx1_yyk1_4.png"},  -- 右眼眶
        ["hx1_zmm"]   = {"hx1_ymm1_1.png", "hx1_zmm1.png"},  -- 左眉毛
        ["hx1_ymm"]   = {"hx1_ymm1.png", "hx1_ymm1_1.png"},  -- 右眉毛
    }

    for boneName,textureList in pairs(changesInfo) do
        for i, v in ipairs(textureList) do
            self:changeBoneIndexWithFile(self:getBone(boneName), v, i-1)
        end
    end
end

--------------------------------------------
-- 骨骼动作
--------------------------------------------
--. 站着呼吸
function M:playBreath1()
	return self:play(self._name.."zhx1")
end

-- 准备
function M:playReady()
	return self:play(self._name.."zb1")
end

--. 出发前呼吸
function M:playBreath2()
	return self:play(self._name.."hx1")
end

-- 开心1
function M:playHappy1()
    if self._name == "hx2_" then
        return self:play(self._name.."kx1")
    end
	return self:play(self._name.."kx1_1")
end

-- 开心1
function M:playHappy1ing()
	return self:play(self._name.."kx1_2")
end

-- 开心2
function M:playHappy2()
    if self._name == "hx2_" then
        return self:play(self._name.."kx2")
    end
	return self:play(self._name.."kx2_1")
end

-- 开心2
function M:playHappy2ing()
	return self:play(self._name.."kx2_2")
end

-- 播放开心动画
function M:palyHappy()
    -- 播放角色开心声音
    self._layer:playRoleHappySnd(self)
    -- 开心
    -- self["playHappy"..N.random(1, 2)](self)
    self["playHappy2"](self)
end

-- 失落
function M:playSad()
    if self._isPlayer then
        -- 播放音效失落
        soundEffect["playEffectsfx2623000"..N.random(5, 6)](soundEffect)
    elseif self:isInScreen() then
        if self._name == "hx1_" then
            -- 播放音效男路人难过
            soundEffect["playEffectsfx2623303"..N.random(2, 3)](soundEffect)
        else
            -- 播放音效女路人难过
            soundEffect["playEffectsfx2623303"..N.random(4, 5)](soundEffect)
        end
    end
    if self._name == "hx1_" then
	    return self:play(self._name.."sl1_1")
    else
	    return self:play(self._name.."sl1", nil, {loop = 1})
    end
end

-- 失落
function M:playSading()
	return self:play(self._name.."sl1_2")
end

-- 出发
function M:playSetOut()
	return self:play(self._name.."cf1", nil, {durationTween = 0.2})
end

--. 滑雪
function M:playSki()
	return self:play(self._name.."hua1")
end

--. 彩虹拖尾
function M:playRainbowTrail()
	return self:play(self._name.."ch1")
end

-- 冲向终点线
function M:playSprint()
	return self:play(self._name.."cx1")
end

-- 刹车
function M:playBrake()
	return self:play(self._name.."sc1")
end

-- 打滑-准备
function M:playSkidBegin()
	return self:play(self._name.."dh1_1")
end

--. 打滑-循环
function M:playSkiding()
	return self:play(self._name.."dh1_2")
end

-- 打滑-结束
function M:playSkidEnd()
	return self:play(self._name.."dh1_3")
end

-- 火箭滑板-准备
function M:playRocketBegin()
	return self:play(self._name.."hjx1_1")
end

--. 火箭滑板-循环
function M:playRocketing()
	return self:play(self._name.."hjx1_2")
end

-- 火箭滑板-结束
function M:playRocketEnd()
	return self:play(self._name.."hjx1_3")
end

-- 碰到道具反馈
function M:playPropEffect()
	return self:play(self._name.."zdj1")
end

-- 踩到香蕉皮-准备
function M:playBananaBegin()
	return self:play(self._name.."xjp1_1")
end

--. 踩到香蕉皮-循环
function M:playBananaing()
	return self:play(self._name.."xjp1_2")
end

-- 踩到香蕉皮-结束
function M:playBananaEnd()
	return self:play(self._name.."xjp1_3")
end

-- 掉进雪坑-准备
function M:playSnowHoleBegin()
	return self:play(self._name.."dxk1_1")
end

--. 掉进雪坑-循环
function M:playSnowHoleing()
	return self:play(self._name.."dxk1_2")
end

-- 掉进雪坑-结束
function M:playSnowHoleEnd()
	return self:play(self._name.."dxk1_3")
end

-- 玩具遮脸-准备
function M:playToyBegin()
	return self:play(self._name.."wjtt1_1")
end

--. 玩具遮脸-循环
function M:playToying()
	return self:play(self._name.."wjtt1_2")
end

-- 玩具遮脸-结束
function M:playToyEnd()
	return self:play(self._name.."wjtt1_3")
end

-- 被砸到雪球-准备
function M:playSnowBallBegin()
	return self:play(self._name.."xq1_1")
end

--. 被砸到雪球-循环
function M:playSnowBalling()
	return self:play(self._name.."xq1_2")
end

-- 被砸到雪球-结束
function M:playSnowBallEnd()
	return self:play(self._name.."xq1_3")
end

-- 碰撞到障碍物-准备
function M:playBarrierBegin()
	return self:play(self._name.."pz1_1")
end

-- 碰撞到障碍物2
function M:playBarrier2()
    -- if self._name 
	return self:play(self._name.."pz2")
end

--. 碰撞到障碍物-循环
function M:playBarriering()
    local time = self:play(self._name.."pz1_2")
    if self._name == "hx2_" then
        if self._roleType == 1 then
            self:getBone("hx2_mhxz1"):hide()
        else
            self:getBone("hx2_mhxz"):hide()
        end
    else
        if self._roleType ~= 1 and not self._hasChangeBarrierTexture then
            self._hasChangeBarrierTexture = true
            -- 骨骼外部换装(骨骼，图片路径， 索引)
            self:changeBoneIndexWithFile(self:getBone("hx1_tx"), "hx1_mhxz1.png", 0)
            self:changeBoneIndexWithFile(self:getBone("hx1_tx"), "hx1_mhxz1_1.png", 1)
            self:changeBoneIndexWithFile(self:getBone("hx1_tx"), "hx1_mhxz1_2.png", 2)
        end
    end
	return time
end

-- 碰撞到障碍物-恢复
function M:playBarrierEnd()
	return self:play(self._name.."pz1_3")
end

-- 跳跃-准备
function M:playJumpBegin()
	return self:play(self._name.."tq1_1")
end

--. 跳跃-空中
function M:playJumping()
	return self:play(self._name.."tq1_2")
end

-- 跳跃-落地
function M:playJumpEnd()
	return self:play(self._name.."tq1_3")
end

-- 飞起-准备
function M:playFlyBegin()
	return self:play(self._name.."fq1_1")
end

--. 飞起-空中
function M:playFlying()
	return self:play(self._name.."fq1_2")
end

-- 飞起-落地
function M:playFlyEnd()
	return self:play(self._name.."fq1_3")
end




----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end





----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end




return M