--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    角色基类
   Author:  Neighbour Mr.Wang
   Date:    2021.08.05
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------
local M     = class("RoleBaseWithFsm", require("app.biathlon.node.RoleBase"))


----------------------
-- 公共参数
----------------------
local Config = require("app.biathlon.Config")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    -- 物理世界
    self._physicsWorld        = params.physicsWorld
    self._mapManager          = params.mapManager
    self._canTouch            = true    -- 开启触控
    -- 最大速度
    self._maxSpeed            = Config.roleMaxSpeed
    -- 可以施加的额外速度
    self._extraMaxSpeed       = Config.extraMaxSpeed
    -- 额外的速度(拖动触发)
    self._extraSpeed          = 0 
    -- 加速带额外的速度(加速带触发)
    self._speedNodeExSpeed      = 0 
    -- 当前最大速度
    self._curMaxSpeed         = self._maxSpeed
    -- 彩虹拖尾时的最大速度
    self._rainbowSpeed        = Config.rainbowSpeed
    -- 相对刚体的Y轴偏移量
    self._relBodyOffsetY      = 0
    -- 是否隐藏影子
    self._hideShadow          = false
    self._specialOffsetY      = 0 -- 特殊情况偏移量(飞行时)
    -- 当前赛道
    self._currentTrack        = 1
    -- 初始赛道
    self._initTrack           = 1
    -- 处于边缘位置段的id(对应刚体世界edge上的点索引)
    self._edgePosId           = 1
    -- 游戏阶段(1.前往靶场1, 2.前往靶场2, 3.前往终点)
    self._gameStage           = 1
    -- 是否处于碰撞中(碰撞时也不可控制)
    self._isCrash             = false
    -- 是否忽略控制
    self._ignoreControl       = false
    -- 是否忽略同步物理世界位置
    self._ignoreSynPhyPos     = false
    -- 是否忽略层级设置
    self._ignoreSynPhyZ       = false
    -- 是否已经通过游戏
    self._hasPassGame         = false
    -- 默认角度
    self._defaultAngle        = 0
    -- 道具引起的飞行状态(0:未吃到飞行道具, 1:吃到飞行道具, 2:依赖飞行道具落地)
    self._propFlyState       = 0 
    -- 离地时间
    self._leaveFloorTime           = 0
end


----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

    -- 设置状态机 
    self:setFsm()
end

-- 设置状态机
function M:setFsm()
    --初始化
    self._fsm = {}
    cc.GameObject.extend(self._fsm):addComponent("components.behavior.StateMachine"):exportMethods()
    self._fsm:getComponent("components.behavior.StateMachine").onError_ = function (event, error, message)
        -- printf("%s [StateMachine] ERROR: error %s, event %s, from %s to %s", tostring(self.target_), tostring(error), event.name, event.from, event.to)
        -- printError(message)
    end
    --状态机设置
    self:setFSM()
end

--设置状态机
function M:setFSM()
    self._fsm:setupState({
		initial = "ready",
 
		events = self:getFsmEvent(), -- 获得状态机事件
 
		callbacks = {
            onstate        = self:getEnterStatefun(),-- 进入新状态通用方法
            onready        = self:getEnterReadyfun(),-- 进入准备状态方法
            onski          = self:getEnterSkifun(),-- 进入滑雪状态方法
            onleaveski     = self:getLeaveSkifun(), -- 离开滑雪状态方法 
            onspeedup      = self:getEnterSpeedupfun(), -- 进入加速状态方法
            onstop         = self:getEnterStopfun(), -- 进入加速状态方法
            onfly          = self:getEnterFlyfun(), -- 进入飞行状态方法
            onleavefly     = self:getLeaveFlyfun(), -- 离开飞行状态方法 
            onstopfly      = self:getEnterStopflyfun(), -- 进入停止飞行状态方法 
            oncrash        = self:getEnterCrashfun(), -- 进入碰撞状态方法 
            onstopcrash    = self:getEnterStopcrashfun(), -- 进入停止碰撞状态方法 
            onskid         = self:getEnterSkidfun(), -- 进入打滑状态方法 
            onstopskid     = self:getEnterStopskidfun(), -- 进入停止打滑状态方法 
            onunusual      = self:getEnterUnusualfun(), -- 进入异常状态方法 
            onleaveunusual = self:getLeaveUnusualfun(), -- 离开异常状态方法 
            onrest         = self:getEnterRestfun(), -- 进入休息状态方法 
            onleaverest    = self:getLeaveRestfun(), -- 离开休息状态方法
            ongetrainbow    = self:getEnterGetRainbowfun(),-- 进入获得彩虹状态方法
            onleavegetrainbow    = self:getLeaveRainbowfun(),-- 进入离开彩虹状态方法
		},
    })
end

-- 获得状态机事件
function M:getFsmEvent()
	return  {
        {   name = "toReady",  to = "ready"},    -- 准备
        {   name = "toSpeedup",  to = "speedup", -- 加速
            from = {"speedup","rest", "normal", "ready", "ski", "stop", "stopfly", "stopcrash", "stopskid"}
        },
        {   name = "toSki", to = "ski",  -- 滑行
            from = { "getrainbow", "special2", "special", "normal", "speedup", "stop", "stopfly", "stopcrash", "stopskid"}, 
        },
        {   name = "toStop", to = "stop",-- 停止
            from = {"ski", "stopfly", "stopcrash", "stopskid"}, 
        },         
        {   name = "toFly", to = "fly",-- 飞起
            from = {"getrainbow", "normal", "speedup", "stop", "ski", "fly", "stopcrash", "stopskid"}
        },   
        {   name = "toStopFly",  to = "stopfly",-- 停止飞起
            from = {"fly"}
        },   
        {   name = "toCrash", to = "crash",-- 碰撞状态
            from = {"getrainbow", "stop", "special2", "fly", "normal", "stopfly", "stop", "stopskid", "ski", "speedup", "stopcrash", "skid", "crash"}
        },   
        {   name = "toStopCrash", to = "stopcrash", -- 停止碰撞状态
            from = {"normal", "crash"}
        },  
        {   name = "toSkid",  to = "skid", -- 打滑
            from = {"getrainbow", "normal", "stopfly", "stop", "ski", "speedup", "stopcrash"}
        },  
        {   name = "toStopSkid",  to   = "stopskid",-- 停止打滑
            from = {"normal", "skid"}
        }, 
        {   name = "toUnusual", to = "unusual" }, -- 异常状态(忽略位置同步)
        {   name = "toRest",    to = "rest"}, -- 休眠状态
        {   name = "toNormal",  to = "normal", -- 正常状态
            from = {"unusual", "special"}, 
        }, 
        -- {   name = "toGetrocket",  to = "specital"},-- 获得火箭状态
        {   name = "toSpecial",  to = "special"},-- 特殊状态(无视碰撞等)
        {   name = "toSpecial2",  to = "special2"},-- 特殊状态2(可以碰撞等)
        {   name = "toGetrainbow",  to = "getrainbow"},-- 获得彩虹状态
    }
end

-- 能否切换赛道
function M:canChangeTrack()
    local cantList = {"skid", "special", "skid", "crash", "fly"}
    if T.find(cantList, self._fsm:getState()) or self._isShootOrPass then
        return false
    end

    return true
end

-- 播放玩家滑雪音效
function M:playPlayerSkiSnd()
	if not self._isPlayer then return end
    if self._sndNode then
        self._sndNode:stopAction(self._stopSkiSndAct)
        self._stopSkiSndAct = nil
    end
    if not self._playerSkiSound then
        -- 播放音效[sfx26233008]滑雪（玩家双手滑动）
        self._playerSkiSound = soundEffect:playEffectsfx26233008()
    else
        sound.stopVoiceFade(self._playerSkiSound)
        sound.setVolume(self._playerSkiSound, 1)
    end
end

-- 停止玩家滑雪音效
function M:stopPlayerSkiSnd()
	if not self._playerSkiSound then return end
    if tolua.isnull(self._sndNode) then
		local sndNode = U.loadNode({ contentSize = cc.size(0.1, 0.1) }):to(self, 55)
		self._sndNode = sndNode
	end
    if self._stopSkiSndAct then return end

    self._stopSkiSndAct = A.line({
        {"delay", 0.4},
        {"fn", function()
            self._stopSkiSndAct = nil
            sound.voiceFadeOut(self._playerSkiSound, 0.3, sound.fadeLinear, function ()
                sound.stopSound(self._playerSkiSound)
                self._playerSkiSound = nil
            end)
        end},
    }):at(self._sndNode)
end

-- 播放正常滑雪音效(无操作)
function M:playSkiSnd()
	if not self._isPlayer then return end
    if not self._skiSound then
        -- 播放音效[sfx26233013]滑雪（玩家双手不滑动）
        self._skiSound = soundEffect:playEffectsfx26233013()
    else
        sound.stopVoiceFade(self._skiSound)
        sound.setVolume(self._skiSound, 1)
    end
end

-- 停止正常滑雪音效(无操作)
function M:stopSkiSnd()
	if self._skiSound then
        sound.voiceFadeOut(self._skiSound, 0.3, sound.fadeLinear, function ()
            sound.stopSound(self._skiSound)
            self._skiSound = nil
        end)
    end
end

-- 播放飞起音效
function M:playFlyWorkSnd()
    if not self._isPlayer then return end
    -- if tolua.isnull(self._sndNode) then
	-- 	local sndNode = U.loadNode({ contentSize = cc.size(0.1, 0.1) }):to(self, 55)
	-- 	self._sndNode = sndNode
	-- end
    if not self._workSound then
        -- 播放音效[sfx26233001]腾空飞起的风声
        self._workSound = soundEffect:playEffectsfx26233001()
    end
end

-- 停止飞起音效
function M:stopFlyPlayWorkSnd()
    if not self._isPlayer then return end
    if not self._workSound then return end

    sound.stopSound(self._workSound)
    -- self._sndNode:stopAction(self._sdAct)
    self._workSound = nil
end

-- 是否在屏幕里
function M:isInScreen()
    local worldX = self:worldx()
    local worldY = self:worldy()
    if worldX > - self:ch() and worldX < V.w+self:ch() and worldY > -self:ch() and worldY < V.h+self:ch() then -- 在屏幕里
        return true, cc.p(worldX, worldY)
    end

    return false, cc.p(worldX, worldY)
end

-- 能否发出声音
function M:canPlaySnd()
    return false  -- ai角色声音隐藏
	-- return (self:isInScreen() and not self._layer._player._isShootOrPass)
end

--------------------------------------------
-- 状态机状态方法
--------------------------------------------
-- 进入获得彩虹状态方法
function M:getEnterGetRainbowfun()
    return function()
        if self._isPlayer then
            -- 播放音效[sfx26233023]彩虹拖尾音效
            self._rainbowSnd = soundEffect:playEffectsfx26233023()
        end
        --. 彩虹拖尾
        self:playRainbowTrail()
    end
end

-- 进入离开彩虹状态方法
function M:getLeaveRainbowfun()
    return function()
        if self._rainbowSnd then
            sound.stopSound(self._rainbowSnd)
            self._rainbowSnd = nil
        end
        -- -- 停止正常滑雪音效(无操作)
        -- sound.stopSound(self._skiSound)
        -- self._skiSound = nil
    end
end

-- 进入停止飞行状态方法
function M:getEnterStopflyfun()
	return function()
        self._ignoreControl = false -- 取消忽略控制
        if self:getAnimation():getCurrentMovementID() == self._name.."tq1_2" then 
            -- 跳跃-落地
            self:playJumpEnd()
        else
            -- 飞起-落地
            self:playFlyEnd()
        end
    end
end

-- 道具飞行落地
function M:propFlyToGround()
    if self._propFlyState == 2 and self._floorDis <= 7 then -- 依赖道具飞起
        self._propFlyState = 0
        local particle1 = P.newParticle("particle/biathlon/diaoluo.plist", 70, 120):to(self, 1)
        particle1:setPositionType(kCCPositionTypeGrouped)
        local particle2 = P.newParticle("particle/biathlon/diaoluo2.plist", 80, 120):to(self, 1)
        particle1:setPositionType(kCCPositionTypeGrouped)
    end
end

-- 进入碰撞状态方法
function M:getEnterCrashfun()
    return function()
        self._isCrash = true

        if self._isCrashFront then
            -- 碰撞到障碍物2
            self:playBarrier2()
        else
            -- 碰撞到障碍物-准备
            self:playBarrierBegin()
        end
    end
end

-- 进入停止碰撞状态方法 
function M:getEnterStopcrashfun()
    return function()
        if self._isCrashFront then
            self._isCrashFront = false
        end
        if self._isAI then
            local dir = self._currentTrack == 4 and -1 or N.randomExcept(-1, 1, 0)
            dir = self._currentTrack == 1 and 1 or dir
            -- 更换赛道
            self._mapManager:getTrackManager():changeTrack(self, dir)
        end

        -- 碰撞到障碍物-恢复
        self:playBarrierEnd()
    end
end

-- 进入打滑状态方法
function M:getEnterSkidfun()
    return function()
        self._isCrash = true
        -- 打滑-准备
        self:playSkidBegin()
    end
end

-- 离开休息状态方法
function M:getLeaveRestfun()
    return function()
        self:show()
        self._ignoreSynPhyZ = false   -- 取消忽略层级同步
    end
end

-- 进入休息状态方法
function M:getEnterRestfun()
    return function()
        self:stop():hide()
        self._ignoreSynPhyZ = true   -- 忽略层级同步
    end
end

-- 离开异常状态方法
function M:getLeaveUnusualfun()
    return function()
        self._ignoreSynPhyPos = false -- 取消忽略位置同步
        self._ignoreSynPhyZ = false   -- 取消忽略层级同步
    end
end

-- 进入异常状态方法
function M:getEnterUnusualfun()
    return function()
        self._ignoreSynPhyPos = true -- 忽略位置同步
    end
end

-- 进入停止打滑状态方法
function M:getEnterStopskidfun()
    return function()
        -- 打滑-结束
        self:playSkidEnd()
    end
end

-- 离开飞行状态方法 
function M:getLeaveFlyfun()
    return function()
        if self._propFlyState == 1 then
            self._propFlyState = 2
        end
        -- 停止飞起音效
        self:stopFlyPlayWorkSnd()

        if self._isPlayer then
            -- 播放音效[sfx26233002]离地后落地
            soundEffect:playEffectsfx26233002()
        end
    	self._ignoreControl = false -- 取消忽略控制
    end
end

-- 进入飞行状态方法
function M:getEnterFlyfun()
    return function()
        -- 停止玩家滑雪音效
        self:stopPlayerSkiSnd()
        -- 停止正常滑雪音效(无操作)
        self:stopSkiSnd()
        self._ignoreControl = true -- 忽略控制
        local randN = N.random(1, 3)  -- 随机数
        if randN == 1 then
            -- 跳跃-准备
            self:playJumpBegin()
        else
            -- 飞起-准备
            self:playFlyBegin()
        end
        if self._isPlayer and isfunction(self._layer.playRoleHappySnd) then
            -- 播放角色开心声音
            self._layer:playRoleHappySnd(self)
        end
        -- 播放飞起音效
        self:playFlyWorkSnd()
    end
end

-- 进入加速状态方法
function M:getEnterStopfun()
    return function()
        -- 刹车
        self:playBrake()
        -- 停止玩家滑雪音效
        self:stopPlayerSkiSnd()
        -- 停止正常滑雪音效(无操作)
        self:stopSkiSnd()
    end
end

-- 获得进入准备状态方法
function M:getEnterReadyfun()
    return function()
        --. 站着呼吸
        self:playBreath1()
    end
end

-- 获得进入滑雪状态方法
function M:getEnterSkifun()
    return function()
        -- 停止玩家滑雪音效
        self:stopPlayerSkiSnd()
        -- 播放正常滑雪音效(无操作)
        self:playSkiSnd()
        --. 滑雪
        self:playSki()
    end
end

-- 离开滑雪状态方法
function M:getLeaveSkifun()
    return function()
        -- 停止正常滑雪音效(无操作)
        self:stopSkiSnd()
    end
end

-- 获得进入加速状态方法
function M:getEnterSpeedupfun()
    return function()
        if self:getAnimation():getCurrentMovementID() == self._name.."cf1" then return end
        -- if self._isPlayer then
            -- 播放音效[sfx26233008]滑雪（玩家双手滑动）
            -- soundEffect:playEffectsfx26233008()
        -- end
        -- 停止正常滑雪音效(无操作)
        self:stopSkiSnd()
        -- 播放玩家滑雪音效
        self:playPlayerSkiSnd()
        -- 出发
        self:playSetOut()
        if isfunction(self.doSkiingLogic) then
            -- 执行滑行逻辑
            self:doSkiingLogic()
        end
    end
end

-- 获得进入新状态通用方法
function M:getEnterStatefun()
    return function(event)
        self._isCrash = false
        -- if self._isPlayer then
        --     print("❤️", event.to)
        -- end
    end
end

----------------------
-- 骨骼回调
----------------------
function M:movementHandler(__evtType, __moveId)
    local name = self._name

    if __evtType == 2 then
        if __moveId == name.."pz1_2" then  -- 碰撞
            self._fsm:doEvent("toStopCrash")-- 停止碰撞状态
        elseif __moveId == name.."kx2_2" then  -- 开心持续
            -- 播放角色开心声音
            self._layer:playRoleHappySnd(self)
        end
    elseif __evtType == 1 then   
        if __moveId == name.."cf1" then  -- 出发
            self._fsm:doEvent("toSki")-- 滑行
        elseif __moveId == name.."zb1" then  -- 准备1
            --. 出发前呼吸
            self:playBreath2()
        elseif __moveId == name.."pz2" then  -- 碰撞2
            self._fsm:doEvent("toStopCrash")-- 停止碰撞状态
        elseif __moveId == name.."pz1_2" then  -- 碰撞
            self._fsm:doEvent("toStopCrash")-- 停止碰撞状态
        elseif __moveId == name.."pz1_1" then
            --. 碰撞到障碍物-循环
            self:playBarriering()
        elseif __moveId == name.."kx1_1" then -- 开心1
            -- 开心1
            self:playHappy1ing()
        elseif __moveId == name.."kx2_1" then -- 开心2
            -- 开心2
            self:playHappy2ing()
        elseif __moveId == name.."sl1_1" then -- 失落
            -- 失落
            self:playSading()
        elseif __moveId == name.."cx1" then -- 冲刺
            if self._rank <= 3 then
                -- 播放开心动画
                self:palyHappy()
            else
                -- 失落
                self:playSad()
            end
            if self._isPlayer then
                -- 游戏结束
                self._layer:gameFinish(self._rank)
            end
        elseif __moveId == name.."wjtt1_3" then
            self._fsm:doEvent("toSki")-- 滑行
        elseif __moveId == name.."wjtt1_1" then
            --. 玩具遮脸-循环
            self:playToying()
        elseif __moveId == name.."xjp1_1" then-- 踩到香蕉皮-准备
            --. 踩到香蕉皮-循环
            self:playBananaing()
        elseif __moveId == name.."xjp1_3" then-- 踩到香蕉皮-结束
            self._fsm:doEvent("toSki")-- 滑行
        elseif __moveId == name.."hjx1_1" then
            --. 火箭滑板-循环
            self:playRocketing()
        elseif __moveId == name.."pz1_3" then
            self._fsm:doEvent("toSki")-- 滑行
        elseif __moveId == name.."tq1_1" then
            --. 跳跃-空中
            self:playJumping()
        elseif __moveId == name.."tq1_3" then
            self._fsm:doEvent("toSki")-- 滑行
        elseif __moveId == name.."fq1_1" then
            --. 飞起-空中
            self:playFlying()
        elseif __moveId == name.."fq1_3" then -- 飞起落地
            self._fsm:doEvent("toSki")-- 滑行
        elseif __moveId == name.."sc1" then
            --. 站着呼吸
            self:playBreath1()
        elseif __moveId == name.."dxk1_1" then
            --. 掉进雪坑-循环
            self:playSnowHoleing()
        elseif __moveId == name.."dh1_1" then
            --. 打滑-循环
            self:playSkiding()
        elseif __moveId == name.."zdj1" then -- 撞道具
            self._fsm:doEvent("toNormal")-- 正常状态
            if self._body:GetLinearVelocity().x > 1 then
                -- --. 滑雪
                -- self:playSki()
                self._fsm:doEvent("toStopCrash")-- 停止碰撞状态
                self._fsm:doEvent("toSki")-- 滑行
            else
                self._fsm:doEvent("toStop")-- 停下
            end
        elseif __moveId == name.."dxk1_3" then -- 掉雪坑结束
            --. 站着呼吸
            self:playBreath1()
            self._fsm:doEvent("toNormal")-- 正常状态
            self._defaultAngle = 0 -- 恢复默认旋转角度
            self._floorDis     = nil
        elseif __moveId == name.."tq1_1" then
            --. 跳跃-空中
            self:playJumping()
        end
    end 
end

--------------------------------------------
-- 功能方法
--------------------------------------------
-- 施加滑行速度
function M:applySkiSpeed(ratio)
    if (not self._ignoreControl) and (not self._isCrash) and (not self._ignoreSynPhyPos) and (not self._isShootOrPass) then
        local maxSpeed = self._fsm:getState() == "getrainbow" and self._rainbowSpeed or self._curMaxSpeed
        local body     = self._body
        -- 获得线速度
        local vel    = body:GetLinearVelocity()
        local angle  = self:rotate()
   
        local ratioX = math.cos(math.rad(- angle))
        local ratioY = math.sin(math.rad(- angle))
        if angle > 4 then-- 下坡时速度调小
            ratioX = ratioX*0.8
            ratioY = ratioY*0.8
        elseif angle < 4 then-- 上坡更容易
            ratioX = ratioX*1.7
            ratioY = ratioY*1.7
        end
        local vecX   = ratioX*ratio
        local vecY   = ratioY*ratio 
        vecX = math.abs(vecX) + vel.x
        vecY = math.abs(vecY) + vel.y
       
        vecY = vecY <= maxSpeed*ratioY and vecY or maxSpeed*ratioY
        vecX = vecX <= maxSpeed*ratioX and vecX or maxSpeed*ratioX
        vecX = vecX + self._extraSpeed*ratioX + self._speedNodeExSpeed*ratioX
        if vecX <= vel.x then
            vecX = vecX <= vel.x and vel.x or vecX -- 速度已经超过最大速度时, 保留原速度
        end
        vecY = vecY + self._extraSpeed*ratioY + self._speedNodeExSpeed*ratioY
        -- 再判断一次
        if (not self._ignoreControl) and (not self._isCrash) and (not self._ignoreSynPhyPos) and (not self._isShootOrPass) then
            body:SetLinearVelocity(b2Vec2(vecX, vecY))
            return true
        end
    end

    return false
end

-- 重置额外的速度
function M:resetExSpees()
	self._extraSpeed       = 0
    self._speedNodeExSpeed = 0
end

-- 开始打靶事件
function M:startShootEvent()
    self._gameStage   = self._gameStage + 1 -- 准备进入下一阶段
    if self._gameStage >= 3 then   -- 有人物尝试冲刺终点了!!!
        self._mapManager._openLineCheck = true -- 开启冲线检测
    end
    self._body:SetLinearVelocity(b2Vec2(0, self._body:GetLinearVelocity().y))
    self._curMaxSpeed   = self._maxSpeed -- 恢复最大速度

    -- 强行停止
    self:forceStop()
end

-- 打靶成功事件
function M:shootSuccessEvent()
    local shootSitId     = self._gameStage - 1 -- 处于的靶场的id
    local blockId        = self._shootTrack > 2 and 2 or 1       -- 靶场对应的路障id
    local roadBlockIndex = 2*(shootSitId - 1) + blockId  -- 路障对应索引

    local passId = 2 -- 通过栅栏的id(1上面那根, 2下面那根)
    if self._shootTrack == 2 or self._shootTrack == 4 then
        passId = 1
    end
    -- 允许通行
    self._mapManager._roadBlockList[roadBlockIndex]:allowPass(passId)
    if self._isPlayer then
        -- 播放音效[sfx26233016]闸门打开
        soundEffect:playEffectsfx26233016()
    end
end

-- 角色上下移动
function M:roleMoveY(dir, offsetX)
    if self._moveYAct then return end

    self._moveYAct = A.cycle({
        {"fn", function()
            self._specialOffsetY = self._specialOffsetY + dir

            if math.abs(self._specialOffsetY - offsetX) < math.abs(dir+1) then
                self:stopAction(self._moveYAct)
                self._moveYAct = nil

                if math.abs(offsetX) <= 1 then
                    self._specialOffsetY = 0
                else
                    -- 角色上下移动
                    self:roleMoveY(-dir, 0)
                end
            end
        end},
        {"delay", 1/60},
    }):at(self)
end

-- 角色恢复特殊偏移量动作(同时恢复锚点)
function M:resetSpecialOffsetYAct()
    if self._moveYAct then return end

    local dir = - self._specialOffsetY/math.abs(self._specialOffsetY)*5
    local anrYDir = (0.35 - self:anchor().y)/math.abs(self._specialOffsetY)/5

    self._moveYAct = A.cycle({
        {"fn", function()
            self._specialOffsetY = self._specialOffsetY + dir
            self:anchor(cc.p(0.5, self:anchor().y + anrYDir))

            if math.abs(self._specialOffsetY) <= 5 then -- 特殊偏移量恢复完毕
                self._specialOffsetY = 0
                self:anchor(cc.p(0.5, 0.35))
                self:stopAction(self._moveYAct)
                self._moveYAct = nil
            end
        end},
        {"delay", 1/60},
    }):at(self)
end

-- 弹簧弹飞效果
function M:flyWithBellowsEffect()
    self._hideShadow = true
    if self._isPlayer then
        -- 玩家提示消失
        self:palyerHintDieOut()
    end
    -- 粒子
    local particle = P.newParticle("particle/biathlon/tanqituowei.plist", 50, 50):to(self, 2)
    particle:setPositionType(kCCPositionTypeFree)
    self._bellowPar = particle -- 弹簧粒子
    CommonFun:changeAnchorP(self, cc.p(0.5, 0.5))
    self._specialOffsetY = self:ch()/2 -- 避免位置突然偏移

    self._bellowAct = A.line({
        {"fn", function()
            if self._speedParticle1 then
                self._speedParticle1:one({"easing","sineout",{"rotateBy", 3.5, -360*5}})
            end
        end},
        {"easing","sineout",{"rotateBy", 3.5, 360*5}},
        {"fn", function()
            self._bellowAct = nil
            -- 弹簧弹飞效果结束
            self:flyWithBellowsEffectEnd()
        end},
    }):at(self) 

    if self._isPlayer then
        -- 播放音效[sfx26233022]弹簧弹起后，角色空中旋转的音效
        self._bellowsSnd = soundEffect:playEffectsfx26233022()
    end
end

-- 弹簧弹飞效果结束
function M:flyWithBellowsEffectEnd()
    self._propFlyState = 2 -- 道具引起的飞行
    self._hideShadow = false

    if self._bellowsSnd then
        sound.voiceFadeOut(self._bellowsSnd, 0.2, sound.fadeLinear, function ()
            sound.stopSound(self._bellowsSnd)
            self._bellowsSnd = nil
        end)
    end

    if not tolua.isnull(self._bellowPar) then
        self._bellowPar:stopSystem()
        self._bellowPar:plan(function()
            self._bellowPar:remove()
        end, 0.5)
    end
    if self._speedParticle1 then
        self._speedParticle1:stop()
        self._speedParticle1:rotation(90)
    end
    if self._isPlayer then
        -- 玩家提示出现
        self:palyerHintAppear()
    end
    -- self._specialOffsetY = 0 -- 避免位置突然偏移
    -- CommonFun:changeAnchorP(self, cc.p(0.5, 0.35))
	-- 角色恢复特殊偏移量动作
    self:resetSpecialOffsetYAct(true)
end

-- 被撞击事件
function M:beCrashEvent(vecX)
    self._fsm:doEvent("toCrash")-- 碰撞状态
    local body = self._body
    -- 获得线速度
    local vel = body:GetLinearVelocity()
    -- vecX      = vecX + vel.x

    body:SetLinearVelocity(b2Vec2(vecX, vel.y))
end

-- 创建body
function M:createBody(pos)
	-- 创建人物刚体
    self._physicsWorld:createRoleBody(self, pos)
end

-- 空中上下浮动
function M:floatUpDownWithFly(vX)
    local moveInfo = {-- 空中上下浮动信息表({开始改变的时间点, 改变的偏移量})
        {39, -185}, {31, 120}, {22, -130}
    }
    for _,info in ipairs(moveInfo) do -- 空中上下浮动
        if vX == info[1] and info[1] > self._maxSpeed then
            -- 角色上下移动
            self:roleMoveY(info[2]/20, info[2])
            break
        end
    end
end

-- 强行停止
function M:forceStop()
    self._fsm:doEvent("toStopCrash")-- 停止其他状态
    self._fsm:doEvent("toStopFly")-- 停止其他状态
    self._fsm:doEvent("toStop")-- 状态
end
----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end





----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end




return M