--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    玩家角色
   Author:  Neighbour Mr.Wang
   Date:    2021.08.05
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------
local M     = class("RolePlayer", require("app.biathlon.node.RoleBaseWithFsm"))


----------------------
-- 公共参数
----------------------

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 相对刚体的Y轴偏移量
    self._relBodyOffsetY = 0

    self._isPlayer       = true
end



----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

    -- 加载玩家提示
    self:loadPlayerHint()
    -- 加速1粒子
    self:loadSpeedParticle1()
end

-- 加载玩家提示(小箭头)
function M:loadPlayerHint()
    -- 玩家提示
    local playerHint = D.img("biathlon/player_hint.png"):p(102, 208):to(self, 50):anchor(cc.p(0.5, 0))
    self._playerHint = playerHint
    -- 玩家提示呼吸
    self:palyerHintBreath()
end

-- 玩家提示呼吸
function M:palyerHintBreath()
	A.cycle({
        {"union",{
            {"fadeTo", 0.8, 180},
            {"easing","sinein",{"moveBy", 0.8, cc.p(0, -5)}},
		}},
        {"union",{
            {"fadeTo", 0.8, 255},
            {"moveBy", 0.8, cc.p(0, 5)},
		}},
        {"delay", 1.5},
        {"moveBy", 0.5, cc.p(0, 9)},
        {"union",{
            {"scaleTo", 0.1, 1.1, 0.8},
            {"easing","sinein",{"moveBy", 0.1, cc.p(0, -20)}},
		}},
        {"union",{
            {"scaleTo", 0.1, 0.85, 1.15},
            {"easing","sineout",{"moveBy", 0.1, cc.p(0, 11)}},
		}},
        {"union",{
            {"scaleTo", 0.07, 1.05, 0.9},
            {"easing","sinein",{"moveBy", 0.07, cc.p(0, -5)}},
		}},
        {"union",{
            {"scaleTo", 0.07, 0.92, 1.08},
            {"easing","sineout", {"moveBy", 0.07, cc.p(0, 5)}},
		}},
        {"scaleTo", 0.07, 1.0},
    }):at(self._playerHint)
end

-- 玩家提示消失
function M:palyerHintDieOut()
    self._playerHint:stop()
	A.one({"fadeOut", 0.3}):at(self._playerHint)
end

-- 玩家提示出现
function M:palyerHintAppear()
    self._playerHint:stop()

    A.line({
        {"fadeIn", 0.3},
        {"fn", function()
            -- 玩家提示呼吸
            self:palyerHintBreath()
        end},
    }):at(self._playerHint)
end

-- 加速1粒子
function M:loadSpeedParticle1()
    if self._speedParticle1 then return end
    local particle = P.newParticle("particle/biathlon/jiasu.plist", 30, 120):to(self, 2):rotation(90):scaleX(0.8)
    self._speedParticle1 = particle

    particle:stopSystem()
end

-- 设置加速线条速度
function M:setSpeedParSpeed(vX, vY)
    if not self._speedParticle1 then
        -- 加速1粒子
        self:loadSpeedParticle1()
    end
    local par = self._speedParticle1
    if vX < 4 and par._isWork then
        par:stopSystem()
        par._isWork = false
    elseif vX > 6 and not par._isWork then
        par._isWork = true
        par:resetSystem()
    elseif par._isWork then 
        local speed = math.sqrt(vX*vX + vY*vY)
        self._parSpeed = 80 + speed*5
        self._parSpeed = self._parSpeed > 300 and 300 or self._parSpeed 
        
        par:setSpeed(self._parSpeed)
    end
end

-- 重置所有状态
function M:resetAllState()
    self._isShootOrPass   = false -- 防止可能出现的bug, 恢复所有状态
    self._ignoreControl   = false
    self._ignoreSynPhyPos = false
    self._isChangeTrack   = false
end

-- 关闭加速粒子1
function M:closeSpeedParticle1()
    if self._speedParticle1 then
        self._speedParticle1:remove()
        self._speedParticle1 = nil
    end
end

-- 自动加速动作
function M:autoApplySpeedAct()
    if self._autoSpeedupAct then return end
	self._autoSpeedupAct = A.line({
        {"delay", 1.4},
        {"fn", function()
            -- 施加滑行速度
            local result = self:applySkiSpeed(50)
            -- 施加额外的速度
            self._layer:applyExtraSpeed()
            self._layer:applyExtraSpeed() -- 增加两次, 直接喷射前进
            if result then
                self._fsm:doEvent("toSpeedup")-- 加速
            end

            self._autoSpeedupAct = nil
        end},
    }):at(self)
end

-- 停止自动加速动作
function M:stopAutoApplySpeedAct()
    if not self._autoSpeedupAct then return end

    self:stopAction(self._autoSpeedupAct)
    self._autoSpeedupAct = nil
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end
----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end








return M