--[[
   Copyright (c) 2012-2021 baby-bus.com

   TODO:    
   Author:  Neighbour Mr.Wang
   Date:    2021.08.06
   
   http://www.baby-bus.com/
   
]]

----------------------
-- 类
----------------------

local M     = classSpriteTouch("ShootPopup")
M.TAG       = "ShootPopup"

----------------------
-- 公共参数
----------------------
local Config = require("app.biathlon.Config")
-- 准心速度(对应等级)
local reticleSpeeds = Config.reticleSpeeds


----------------------
-- 构造方法
----------------------
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
  

    self._layer     = params.layer
    -- 打靶的轮数
    self._round      = params.round or 1
    -- 靶子列表
    self._targetList = {}
    self._canTouch   = false    -- 关闭触控
    self._successNum = 0    -- 打中数量
    self._startTime  = 0 -- 打靶开始时时间
    self._endTime    = 0 -- 打靶结束时时间
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]

function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    
    -- 数据初始化
    self:initData()
    -- 图片渲染
    self:loadDisplay()
    -- 进入
    self:enter()
end

------------------------------------------------------------------------------------------------
-- 功能函数
------------------------------------------------------------------------------------------------
-- 数据初始化
function M:initData()
    -- 准心位置
    local reticlePos = {
        {cc.p(V.w - 400, V.h), cc.p(V.w + 400, V.h)},
        {cc.p(V.w, V.h+360), cc.p(V.w, V.h-360)},
    }
    -- 准心移动的两个位置
	self._reticlePos1 = reticlePos[self._round][1]
	self._reticlePos2 = reticlePos[self._round][2]
    -- 准心移动的下一个位置
    self._reticleNextPos = self._reticlePos1
    -- 准心速度
    self._reticleSpeed = reticleSpeeds[1]
end

-- 图片渲染
function M:loadDisplay()
    self:setContentSize(cc.size(V.w*2, V.h*2))
    -- 加载打靶底
    self:loadShootBottom()
    -- 加载靶子
    self:loadTarget()
    -- 加载准心
    self:loadReticle()
end

-- 加载打靶底
function M:loadShootBottom()
    -- 靶子底部
    local bottom    = D.img("biathlon/shoot/bar.png"):p(V.w, V.h):to(self, -1000)--:bindTouch()
    local bottomTop = D.img("biathlon/shoot/bartop"..self._round..".png"):p(bottom:cw()/2, bottom:ch()/2):to(bottom, 2000)--:bindTouch()
    self._bottom    = bottom

    if self._round == 2 then
        bottom:rotate(90)
    end
end

-- 加载靶子
function M:loadTarget()
    local posList = {
        cc.p(107, 101), cc.p(281, 101), cc.p(452, 101), cc.p(628, 101), cc.p(810, 101)
    }

    local targetSc = self._round == 1 and 0.68 or 1
    for i,pos in ipairs(posList) do
        -- 靶子
        local target       = D.img("biathlon/shoot/target/1.png"):to(self._bottom, 100):p(pos):scale(targetSc)
        -- 光
        local light        = D.img("biathlon/shoot/target/light.png"):to(self._bottom, 2000):p(pos):scale(targetSc):opacity(0)
        target._light      = light
        -- 靶子背面
        local targetBack   = D.img("biathlon/shoot/target/2.png"):to(self._bottom, 99):p(pos):scale(targetSc)
        target._targetBack = targetBack
        -- 靶子列表
        table.insert(self._targetList, target)
    end
end

-- 靶子提示动作
function M:targetHintAct()
	A.cycle({
		{"delay", 0.1},
		{"fn", function()
            local pos = self._hole:point() -- 打击点
            local endPosWorld = self:convertToWorldSpace(pos)
            -- 获得打中的靶子
            local target      = self:getShootTarget(endPosWorld)
            if target and target._light then
                if not tolua.isnull(target) and not target._fadeAct then
                    -- 播放音效[sfx26216003]准星移动到靶子上音效
                    soundEffect:playEffectsfx26216003()
                    target._fadeAct = A.line({
                        {"fadeTo", 0.15, 230},
                        {"fadeTo", 0.1, 170},
                        {"fadeTo", 0.15, 220},
                        {"fadeTo", 0.1, 170},
                        {"fn", function()
                            target._fadeAct = nil 
                        end},
                    }):at(target._light)
                end
            end
            for i,v in pairs(self._targetList) do
                if v ~= target and v._light then
                    if v._fadeAct then -- 发光引导动作
                        v._light:stopAction(v._fadeAct)
                        A.one({"fadeOut", 0.1}):at(v._light)
                        v._fadeAct = nil
                    else
                        v._light:opacity(0)
                    end
                end
            end
		end},
	}):at(self)
end

-- 底部受击效果
function M:bottomBeShootEffect()
    -- 放入物品反馈
    CommonFun.feedback(self._bottom, -0.2, 1, 0.1, 1, 1)
end

-- 靶子被打中
function M:targetBeShoot(target, endPos)
    -- 底部受击效果
    self:bottomBeShootEffect()
    if tolua.isnull(target) then return end
    target._light   = CommonFun.remove(target._light)
    local targetPos = target:convertToNodeSpace(endPos)
    local bottomPos = self._bottom:convertToNodeSpace(endPos)
    -- 打中效果1
    local effectA1 = D.img("biathlon/shoot/effect1.png"):p(targetPos):to(target, 50)
    local effectB1 = D.img("biathlon/shoot/effect1.png"):p(targetPos):to(target._targetBack, 50)
    -- 打中效果2
    local effect2 = D.img("biathlon/shoot/effect2.png"):p(bottomPos):to(self._bottom, 3000):scale(0.3):rotate(N.random(0, 360))
    A.line({
        --出场动作
        {"easing","ELASTICOUT",{"scaleTo", 0.2, 1}, 3},
        {"fadeOut", 0.3},
    }):at(effect2)
end

-- 靶子翻转
function M:targetFlip(target, endPos)
    if tolua.isnull(target) then
        -- 继续打靶
        self:continueShoot()
        return
    end
    local pos1      = self._bottom:convertToNodeSpace(endPos) -- 在底上的位置
    local disX      = pos1.x - target:px()
    local dir       = disX > 0 and 1 or -1
    soundEffect:playEffectsfx26233005()
    -- 翻转
    A.line({
        {"fn", function()
            A.line({
                {"OrbitCamera", 0.05, 1, 0, 0, 90*dir, 0, 0},
                {"hide"},
            }):at(target) 
        end},
        {"OrbitCamera", 0.1, 1, 0, 0, 180*dir, 0, 0},
        {"fn", function()
            A.line({
                {"show"},
                {"OrbitCamera", 0.16, 1, 0, 0, 180*dir, 0, 0},
            }):at(target) 
        end},
        {"OrbitCamera", 0.08, 1, 0, 0, 90*dir, 0, 0},
        {"fn", function()
            A.line({
                {"OrbitCamera", 0.1, 1, 0, 0, 90*dir, 0, 0},
                {"remove"}
            }):at(target) 
        end},
        {"OrbitCamera", 0.2, 1, 0, 0, 180*dir, 0, 0},
        {"fn", function()
           -- 靶子被打中结束
           self:targetBeShootEnd(target)
        end},
    }):at(target._targetBack)
end

-- 靶子被打中结束
function M:targetBeShootEnd(target)
    target._beShoot = true -- 被打中
    -- 难度提升
    self:difficultyPromote()

    if #self._targetList == 0  then
        if self._layer._isGuide then
            self._layer:guideOver()-- 引导结束
            return
        end
        -- 打靶结束
        self:shootEnd()
    else
        -- 继续打靶
        self:continueShoot()
    end
end

-- 打靶结束
function M:shootEnd()
    local layer = self._layer -- 层
    -- 白屏过度
    layer:whiteScreen(function ()
        self._endTime     = os.time()
        local elapsedTime = self._endTime - self._startTime -- 总耗时
        ST.setNumber("BIATHLON_SHOOT_ELAPSEDTIME", elapsedTime)-- 记录人物打靶子耗时

        self = CommonFun.remove(self)
        -- 打靶完成
        layer._mapManager:shootFinish()
    end, 255)
end

-- 难度提升
function M:difficultyPromote()
    self._successNum = self._successNum +1
    -- 准心速度改变
	self._reticleSpeed = reticleSpeeds[self._successNum+1]
end

-- 继续打靶
function M:continueShoot()
    -- 准心移动
    self:reticleMove()
    self._canTouch = true    -- 开启触控
end

-- 加载准心
function M:loadReticle()
    -- -- 初始位置
    local pos  = cc.p((self._reticlePos1.x+self._reticlePos2.x)/2, (self._reticlePos1.y+self._reticlePos2.y)/2)
    local hole = U.loadNode({ contentSize = cc.size(0.01, 0.01) }):p(pos)
    self._hole = hole

    hole:to(self, -1):opacity(200)
    -- 准心V.ch
    local reticle = D.img("biathlon/shoot/reticle/5.png"):to(hole, 1000):p(hole:cw()/2, hole:ch()/2)
    self._reticle = reticle
end

-- 准心移动
function M:reticleMove()
    if self._reticleAct then return end
    -- 单次移动时长
    local time = PT.distance(self._reticleNextPos, self._hole:point())/self._reticleSpeed
	self._reticleAct = A.line({
		{"moveTo", time, self._reticleNextPos},
		{"fn", function()
            self._reticleAct = nil
            -- 准心移动的下一个位置
            self._reticleNextPos = self._reticleNextPos == self._reticlePos1 and self._reticlePos2 or self._reticlePos1
            -- 准心移动
            self:reticleMove()
        end},
	}):at(self._hole)
end

-- 进入
function M:enter(cb)
    self._startTime = os.time()

    local pos = cc.p(0, 0)
    -- 白屏过度
    self._layer:whiteScreen(function ()
        if isfunction(cb) then
            cb()
        end
        -- 靶子提示动作
        self:targetHintAct()
        -- 准心移动
        self:reticleMove()
        if self._layer._isGuide then
            -- 播放配音[v262316]准星到靶子的时候，就可以丢出雪球啦
            self._guideSnd = soundVoice:playEffectv262316()
            A.line({
                {"delay", soundVoice:getVoiceTime("v262315", 7)},
                {"fn", function()
                    self._guideSnd = nil
                end},
            }):at(self)
        end
        self._canTouch = true    -- 开启触控
    end, 0)
end

-- 后坐力效果
function M:recoilForceEffect()
	A.line({
        {"delay", 0.07},
        {"easing","sinein",{"scaleTo", 0.07, 0.98},},
        {"easing","sinein",{"scaleTo", 0.07, 1.02},},
        {"easing","sineout",{"scaleTo", 0.07, 1},},
    }):at(self._layer)
end

-- 丢雪球
function M:throwSnowBall()
    -- 播放音效[sfx26233004]射击
    soundEffect:playEffectsfx26233004()
    self._canTouch = false    -- 关闭触控
    A.one({"image", "biathlon/shoot/reticle/", 5, 0.1}):at(self._reticle)

    self._hole:stopAction(self._reticleAct)
    self._reticleAct = nil
	-- 雪球
    local snowBall    = D.img("biathlon/shoot/ball.png"):p(V.cw, -1500):to(self, 50):scale(0.8)
    local endPos      = self._hole:point() -- 打击点
    local endPosWorld = self:convertToWorldSpace(endPos)
    -- 获得打中的靶子
    local target      = self:getShootTarget(endPosWorld)

    if not target then
        -- 未打中引导
        self:shootMissGuide()
    end
    -- 后坐力效果
    self:recoilForceEffect()
    A.line({
        {"union",{
            {"moveTo", 0.22, endPos},
            {"scaleTo", 0.22, 1},
		}},
        {"fn", function()
           -- 靶子被打中
           self:targetBeShoot(target, endPosWorld)
        end},
        {"image", "biathlon/shoot/ballimgs/", 4, 0.1},
        {"fn", function()
            -- 靶子翻转
            self:targetFlip(target, endPosWorld) 
            T.removeOrder(self._targetList, target)
        end},
        {"delay", 0.04},
        {"remove"},
    }):at(snowBall)
end

-- 获得打中的靶子
function M:getShootTarget(endPos)
	local target = nil
    local pos = self._bottom:convertToNodeSpace(endPos)

    local successDis = self._round == 1 and 50 or 82
    for k,v in pairs(self._targetList) do
        if not tolua.isnull(v) and not v._beShoot and PT.distance(v:point(), pos) <= successDis then
            target = v
            break
        end
    end

    return  target
end

-- 未打中引导
function M:shootMissGuide()
	if self._guideSnd then
        return
    end
    -- 播放配音[v262317]瞄准靶子，丢出雪球吧
    self._guideSnd = soundVoice:playEffectv262317()
    A.line({
        {"delay", soundVoice:getVoiceTime("v262317", 7)},
        {"fn", function()
            self._guideSnd = nil
        end},
    }):at(self)
end
--------------------------------------------------------------------
-- 触控
--------------------------------------------------------------------
function M:onTouchBegan(x, y, touches)
    if not self._canTouch then return end

    -- 丢雪球
    self:throwSnowBall()

    return SIGN_TOUCH_BEGAN_SWALLOWS
end


----------------------
-- 结点析构
----------------------

function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]

function M:onExitTransitionStart()

end

----------------------
-- 验证
----------------------

-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end

return M
